# frozen_string_literal: true

module Xikolo
  module V2
    class StatAPI < Grape::API::Instance
      helpers do
        def available_metrics
          @available_metrics ||= begin
            metrics_list = lanalytics_api.rel(:metrics).get.value!
            metrics_list.select {|metric| metric['available'] }.pluck('name')
          end
        end

        def metrics_available?(*names)
          # Checks the availability of the metric's datasource
          # AND whether the metric exists (name) at all
          names.map(&:underscore).all? {|name| available_metrics.include?(name) }
        end

        def fetch_metric(name:, **)
          return lanalytics_api.rel(:metric).get({name:}.merge(**)) if metrics_available?(name)

          Restify::Promise.fulfilled(nil)
        end

        def account_api
          @account_api ||= Xikolo.api(:account).value!
        end

        def course_api
          @course_api ||= Xikolo.api(:course).value!
        end

        def lanalytics_api
          @lanalytics_api ||= Xikolo.api(:learnanalytics).value!
        end
      end

      # this endpoint is deprecated and only used by the marketing dashboard
      # see https://github.com/openhpi/jenz
      namespace 'stats' do
        namespace 'global' do
          get do
            Rails.cache.fetch('deprecated/stats/global/', expires_in: 3.hours, race_condition_ttl: 1.minute) do
              Restify::Promise.new([
                account_api.rel(:statistics).get,
                course_api.rel(:stats).get(key: 'global'),
              ]) do |account_stats, course_stats|
                external_enrollments = 0
                enrollments = course_stats[:platform_enrollments] + Xikolo.config.global_enrollment_delta + external_enrollments

                {
                  global_statistic: {
                    confirmed_users:              account_stats[:confirmed_users] + Xikolo.config.global_users_delta,
                    confirmed_users_last_day:     account_stats[:confirmed_users_last_day],
                    confirmed_users_last_7days:   account_stats[:confirmed_users_last_7days],
                    unconfirmed_users:            account_stats[:unconfirmed_users],
                    unconfirmed_users_last_day:   account_stats[:unconfirmed_users_last_day],
                    deleted_users:                account_stats[:users_deleted],
                    users_with_suspended_email:   account_stats[:users_with_suspended_email],
                    total_enrollments:            enrollments,
                    external_enrollments:,
                    global_enrollment_delta:      Xikolo.config.global_enrollment_delta,
                    courses_per_learner:          enrollments.to_f / account_stats[:confirmed_users],
                    platform_enrollments:         course_stats[:platform_enrollments],
                    unenrollments:                course_stats[:unenrollments],
                    platform_custom_completed:    course_stats[:platform_custom_completed],
                    certificates_count:           course_stats[:certificates_count],
                    courses_count:                course_stats[:courses_count],
                    unique_enrollments_last_day:  course_stats[:platform_last_day_unique_enrollments],
                    total_enrollments_last_day:   course_stats[:platform_last_day_enrollments],
                    total_enrollments_last_7days: course_stats[:platform_last_7days_enrollments],
                  },
                }
              end.value!
            end
          end
        end

        namespace 'active_users' do
          params do
            optional :start_date, type: DateTime, desc: 'The start date'
            optional :end_date, type: DateTime, desc: 'The end date'
          end
          get do
            Rails.cache.fetch(
              "deprecated/stats/active_users/#{params[:start_date]}_#{params[:end_date]}",
              expires_in: 1.hour,
              race_condition_ttl: 1.minute
            ) do
              user_activity = fetch_metric(name: 'user_activity_count')

              fetch_active_users = proc do |start_date, end_date|
                fetch_metric(
                  name: 'active_user_count',
                  start_date:,
                  end_date:
                ).then {|response| response&.fetch('active_users') }
              end

              stats = {current_user_activity: user_activity}

              stats[:count] = if params[:start_date].present? && params[:end_date].present?
                                fetch_active_users.call(params[:start_date], params[:end_date])
                              else
                                fetch_active_users.call # proc arguments default to nil
                              end

              stats.transform_values {|r| r&.value! || r }
            end
          end
        end

        namespace 'geo' do
          params do
            optional :start_date, type: DateTime, desc: 'The start date'
            optional :end_date, type: DateTime, desc: 'The end date'
          end
          get do
            Rails.cache.fetch(
              "deprecated/stats/geo/#{params[:start_date]}_#{params[:end_date]}",
              expires_in: 3.hours,
              race_condition_ttl: 1.minute
            ) do
              fetch_metric(
                name: 'geo_activity',
                start_date: params[:start_date],
                end_date: params[:end_date]
              ).value!
            end
          end
        end
      end
    end
  end
end
