/**
 * ToastUI editor initializer
 *
 * This script transforms form textarea fields into markdown editors.
 * For more details on how to use it, please refer to the preview template
 * (app/components/util/markdown_input/markdown_input.html.slim).
 *
 * This script also works with the simple form input "MarkdownInput"
 * (app/inputs/markdown_input.rb).
 *
 */

// Import locales
import '@toast-ui/editor/dist/i18n/de-de';
import '@toast-ui/editor/dist/i18n/es-es';
import '@toast-ui/editor/dist/i18n/fr-fr';
import '@toast-ui/editor/dist/i18n/pt-br';
import '@toast-ui/editor/dist/i18n/ru-ru';
import '@toast-ui/editor/dist/i18n/uk-ua';
import '@toast-ui/editor/dist/i18n/zh-cn';

import ToastUi from '@toast-ui/editor';
import ready from '../../../util/ready';
import parseMarkdownToHTML from './html-parser';
import setResizeBtn from './resize-editor';

const isImage = (url: string) =>
  url.toLowerCase().match(/\.(jpeg|jpg|gif|png|apng|svg|avif|webp)$/) != null;

export default function initializeMarkdownEditor(
  scope: Document | HTMLElement = document,
) {
  const editors = scope.querySelectorAll(
    '[data-behavior="markdown-editor-wrapper"]',
  );

  editors.forEach((wrapper) => {
    const editor = wrapper.querySelector<HTMLElement>(
      '[data-behavior="markdown-editor-widget"]',
    );
    const formInput = wrapper.querySelector<HTMLInputElement>(
      '[data-behavior="markdown-form-input"]',
    );

    if (!editor || !formInput) return;

    let preview = editor.querySelector<HTMLElement>(
      '.toastui-editor-md-preview .toastui-editor-contents',
    );

    let uploadUrl = '';
    const urlMapping = wrapper.querySelector<HTMLInputElement>(
      `${formInput.dataset.urlMapping}`,
    );
    const uploadedFilesMapping = urlMapping ? JSON.parse(urlMapping.value) : {};

    const errorElement = scope.querySelector<HTMLElement>(
      `#${formInput.id}-error`,
    );

    const toastEditor = new ToastUi({
      el: editor,
      initialValue: formInput.value,
      placeholder: formInput.placeholder,
      previewHighlight: false,
      height: '300px',
      autofocus: false,
      usageStatistics: false,
      language: I18n.locale,
      toolbarItems: [
        ['heading', 'bold', 'italic'],
        ['link', 'quote', 'code', 'table'],
        ['ul', 'ol'],
      ],
      customHTMLRenderer: {
        // We are not using the toastUI preview although it is still being built in the background.
        // To avoid unnecessary image requests we disable toastUI preview image rendering.
        image: () => null,
      },
      hideModeSwitch: true, // Hide WYSIWYG mode
      initialEditType: 'markdown',
      events: {
        change: () => {
          // Keep <textarea> in sync
          const content = toastEditor.getMarkdown();
          formInput.value = content;
          // Keep preview in sync (with markdown-it parser)
          preview ||= editor.querySelector<HTMLElement>(
            '.toastui-editor-md-preview .toastui-editor-contents',
          );

          if (editor.dataset.imageUpload === 'true') {
            const dropzone = wrapper.querySelector<HTMLElement>(
              `[data-textarea-id='#${formInput.id}']`,
            );
            uploadUrl ||= dropzone!.dataset.mdupload as string;
          }

          preview!.innerHTML = parseMarkdownToHTML(
            content,
            uploadUrl,
            uploadedFilesMapping,
          );

          // Reset error message
          errorElement!.textContent = '';
        },
      },
    });

    setResizeBtn(formInput, toastEditor);

    // Prevent user from drag'n'dropping images in the editor
    toastEditor.removeHook('addImageBlobHook');

    // Custom event to programmatically insert a link to cursor position
    // Fired from mdupload.js
    editor.addEventListener('markdownEditor::addLink', ((e: CustomEvent) => {
      if (e.detail) {
        if (isImage(e.detail.url)) {
          toastEditor.exec('addImage', {
            altText: 'Insert image description',
            imageUrl: e.detail.url,
          });
        } else {
          toastEditor.exec('addLink', {
            linkText: e.detail.name,
            linkUrl: e.detail.url,
          });
        }
      }
    }) as EventListener);

    // Delegate invalid error message from form input to our error element
    if (errorElement) {
      formInput.addEventListener('invalid', (e) => {
        const target = e.target as HTMLInputElement;
        errorElement.textContent = target.validationMessage;
      });
    }

    // Delegate focus from form input to toast ui editor
    formInput.addEventListener('focus', () => {
      toastEditor.focus();
    });

    // Override toastUI preview with HTML parsed by markdown-it
    // to avoid bugs in their parser
    // Referenced links: https://github.com/nhn/tui.editor/issues/1635
    // Softbreak: https://github.com/nhn/tui.editor/issues/485
    const content = toastEditor.getMarkdown();
    preview ||= editor.querySelector<HTMLElement>(
      '.toastui-editor-md-preview .toastui-editor-contents',
    );

    // Set parsing rule to use right urls for previewing images
    if (editor.dataset.imageUpload === 'true') {
      const dropzone = wrapper.querySelector<HTMLElement>(
        `[data-textarea-id='#${formInput.id}']`,
      );
      uploadUrl ||= dropzone!.dataset.mdupload as string;
    }
    preview!.innerHTML = parseMarkdownToHTML(
      content,
      uploadUrl,
      uploadedFilesMapping,
    );
    // We can save some events since we don't use the toastUI preview
    toastEditor.eventEmitter.removeEventHandler('updatePreview');
    toastEditor.eventEmitter.removeEventHandler('beforePreviewRender');
    toastEditor.eventEmitter.removeEventHandler('scroll');
  });
}

ready(() => {
  initializeMarkdownEditor();
});
