# frozen_string_literal: true

##
# Listen to events that map to important milestones for learners (such as
# course enrollments and completion) and hand them to configurable handlers.
# Those can (asynchronously) do things with them, e.g. report them to external
# APIs or other integrations.
#
class LearnerEventsConsumer < Msgr::Consumer
  ##
  # Handle course enrollments
  #
  # All handlers will receive the user ID, a verb and a payload compatible with
  # usage in a JSON Activity Streams 1.0 message.
  #
  # (Reduced) Example event payload:
  #
  # {
  #   id: "94e4d9f3-fdae-4b64-a8f0-9cf610ef3742",
  #   user_id: "00000001-3100-4444-9999-000000000002",
  #   course_id: "00000001-3300-4444-9999-000000000001",
  # }
  #
  def on_enrollment
    handlers.each do |klass|
      klass.perform_async(
        payload[:user_id], 'enroll',
        LearnerEvents::EnrollmentEvent.new(payload).for_json_activity_streams
      )
    end
  end

  private

  def handlers
    Xikolo.config.learner_events['handlers'].map(&:constantize)
  end
end
