# frozen_string_literal: true

class Course::Admin::ObjectivesController < Abstract::FrontendController
  include CourseContextHelper

  before_action do
    raise AbstractController::ActionNotFound unless Xikolo.config.learningobjectives['enabled']
  end

  inside_course
  require_permission 'course.content.edit'

  def index
    Acfs.run # because `inside_course` and before actions need Acfs

    @objectives = learningobjectives_service
      .rel(:objectives)
      .get(context_id: course['id'], embed: 'learning_units')
      .value!
      .map {|objective| ObjectivePresenter.new objective: RestifyResource.new(resource: objective) }
  end

  def new
    Acfs.run

    @objective = ObjectivePresenter.new(
      objective: RestifyResource.new(
        params: {
          learning_units: [],
        }
      ),
      course_items:
    )
  end

  def edit
    Acfs.run

    objective = learningobjectives_service
      .rel(:objective)
      .get(id: params[:id], embed: 'learning_units')
      .value!

    @objective = ObjectivePresenter.new(
      objective: RestifyResource.new(resource: objective),
      course_items:
    )
  end

  def create
    Acfs.run

    @objective = learningobjectives_service
      .rel(:objectives)
      .post(objective_params.merge(context_id: course['id']))
      .value!

    add_flash_message :success, t(:'flash.success.objective_created')
    redirect_to course_objectives_path
  rescue Restify::ClientError
    add_flash_message :error, t(:'flash.error.objective_not_created')
    redirect_to course_objectives_path
  end

  def update
    learningobjectives_service
      .rel(:objective)
      .patch(objective_params, id: params[:id])
      .value!

    add_flash_message :success, t(:'flash.success.objective_updated')
    redirect_to course_objectives_path
  rescue Restify::ClientError
    add_flash_message :error, t(:'flash.error.objective_not_updated')
    redirect_to course_objectives_path
  end

  def destroy
    learningobjectives_service
      .rel(:objective)
      .delete(id: params[:id])
      .value!

    add_flash_message :success, t(:'flash.success.objective_deleted')
    redirect_to course_objectives_path
  rescue Restify::ClientError
    add_flash_message :error, t(:'flash.error.objective_not_deleted')
    redirect_to course_objectives_path
  end

  private

  def objective_params
    params.require(:objective).permit(
      :id,
      :title,
      :description,
      :completion_type,
      :final,
      :position,
      learning_units: [
        :id,
        {knowledge_acquisition_items: [],
        knowledge_examination_items: []},
      ]
    ).to_h
  end

  def course
    @course ||= course_promise.value!
  rescue Restify::ClientError
    raise Status::NotFound
  end

  def course_promise
    raise Status::NotFound unless course_identifier

    course_service.rel(:course).get(id: course_identifier)
  end

  def course_identifier
    params[:course_code] || params[:course_id]
  end

  def auth_context
    course_promise.value!['context_id']
  end

  def course_service
    @course_service ||= Xikolo.api(:course).value!
  end

  def learningobjectives_service
    @learningobjectives_service ||= Xikolo.api(:learningobjectives).value!
  end

  def course_items
    @course_items ||= course_service.rel(:items).get(course_id: course['id'], published: true, per_page: 1000).value!
  end
end
