# frozen_string_literal: true

Rails.application.routes.draw do
  # The `BrandRoutes` module encapsulates brand-specific routes, which are specific
  # to certain integrations of individual brands. Therefore, they are excluded from
  # the community version.
  #
  # Routes can be added to `config/routes/brand_routes.rb` as follows:
  # module BrandRoutes
  #   def self.extended(router)
  #     router.instance_exec do
  #       # Your route definitions go here.
  #     end
  #   end
  # end
  extend BrandRoutes if defined?(BrandRoutes)

  # Sitemap served from S3 bucket
  get '/sitemap.xml.gz', to: 'files#sitemap'

  resources :reports, only: %i[index create destroy], module: 'admin'

  resources :documents, controller: 'admin/documents' do
    resources :localizations, controller: 'admin/document_localizations', except: %i[show index]
  end

  post 'cookie_consent', to: 'cookie_consent#create'

  def pinboard_routes
    resources :pinboard, only: [:index]
    resources :question, only: %i[show create edit update destroy] do
      resources :pinboard_comment, only: %i[create update edit destroy] do
        member do
          post :abuse_report
          post :block
          post :unblock
        end
      end
      member do
        post :upvote
        post :accept_answer
        post :close
        post :reopen
        post :abuse_report
        post :block
        post :unblock
      end
    end
    resources :answer, only: %i[create update edit destroy] do
      resources :pinboard_comment, only: %i[create update edit destroy] do
        member do
          post :abuse_report
          post :block
          post :unblock
        end
      end
      member do
        post :upvote
        post :downvote
        post :abuse_report
        post :block
        post :unblock
      end
    end
  end

  # Authentication and Registration
  scope module: :account do
    resources :sessions, only: %i[new create]
    delete '/session/logout', to: 'sessions#destroy', as: :logout

    # this route is actually intercepted by the omniauth middleware, sessions#new is not called
    get '/auth/:provider', to: 'sessions#new', as: :auth
    match '/auth/:provider/callback', to: 'sessions#authorization_callback',
      via: %i[get post], as: :auth_callback

    post 'sessions/new', to: 'sessions#new', as: :login
    resource :account, only: %i[new create] do
      resources :password_resets, only: %i[new show create update], as: :reset, path: :reset
      resources :confirmations, only: %i[show new create update], path: :confirm
      get 'policies', to: 'policies#show'
      put 'policies', to: 'policies#update'
      put 'consents', to: 'consents#update'
      get 'verify', to: 'accounts#verify'
    end

    get 'accounts/connect', to: 'connect#new', as: :connect_accounts

    resources :treatments, only: [:index]
    post 'treatments', to: 'treatments#consent', as: :consent_treatments

    scope path: :dashboard do
      get 'profile' => 'profiles#show', as: :dashboard_profile
      post 'profile' => 'profiles#update'
      scope path: :profile do
        delete 'emails/:id', to: 'profiles#delete_email', as: :delete_email
        patch 'change_primary_email/:id', to: 'profiles#change_primary_email', as: :change_primary_email
        post 'visual', to: 'profiles#update_visual', as: :dashboard_profile_visual
        post 'password', to: 'profiles#change_my_password', as: :change_my_password
        delete 'auth/:id', to: 'profiles#delete_authorization', as: :auth_delete
        get 'unsuspend_primary_email', to: 'profiles#unsuspend_primary_email', as: :unsuspend_primary_email
      end
    end

    resource :preferences, only: %i[update show]
  end

  resources :videos, only: %i[index destroy], module: 'admin'
  resources :subtitles, only: %i[show destroy]
  post '/videos/thumbnail_api_callback', to: 'admin/videos#thumbnail_api_callback'

  resources :playlists, only: %i[show], module: 'video'

  resources :streams, only: [], module: 'stream' do
    resource :download, path: 'downloads/:quality', only: :show
    resource :sync, only: :create
  end

  # Certificates
  get '/certificate/render', to: 'course/certificates#render_certificate', as: :certificate_render, module: 'course'
  get '/verify/:id', to: 'course/certificates#verify', as: :certificate_verification, module: 'course'

  # Open Badges
  scope :openbadges do
    get 'issuer', to: 'open_badges#issuer', as: :openbadges_issuer
    get 'public_key', to: 'open_badges#public_key', as: :openbadges_public_key
    get 'revocation_list', to: 'open_badges#revocation_list', as: :openbadges_revocation_list

    scope :v2, module: :open_badges do
      get 'issuer', to: 'open_badges#issuer', as: :openbadges_issuer_v2
      get 'public_key', to: 'open_badges#public_key', as: :openbadges_public_key_v2
      get 'revocation_list', to: 'open_badges#revocation_list', as: :openbadges_revocation_list_v2
    end
  end

  resources :users, only: %i[index create new], module: 'admin' do
    resource :masquerade, only: %i[create destroy]
    resource :manual_confirmations, only: :create
  end
  resources :users, only: %i[show destroy]

  resources :teachers, except: :destroy, module: 'admin'
  resources :permissions, only: [:index], module: 'admin'
  resources :vouchers, only: %i[index create], module: 'admin'
  get '/vouchers/stats', to: 'admin/vouchers#stats', as: :vouchers_stats
  get '/vouchers/query', to: 'admin/vouchers#query', as: :vouchers_query
  resources :groups, only: [], module: 'admin' do
    resources :members, only: %i[create destroy], controller: 'permissions'
  end
  get '/admin/dashboard', to: 'admin/dashboard#show', as: :admin_dashboard, module: 'admin'
  post '/users/:id/change_user_password', to: 'users#change_user_password'
  resources :feeds, only: [:index]
  get '/feeds/courses', to: 'feeds#courses', as: :courses_feed, defaults: {format: 'json'}
  post '/courses/:id/clone', to: 'admin/courses#clone', as: :course_clone, module: 'admin'
  post '/courses/:id/generate_ranking', to: 'admin/course_management#generate_ranking', as: :generate_ranking_course
  get '/admin/courses(/:category)', to: 'admin/courses#index', as: :admin_courses
  resources :courses, only: %i[create update new edit destroy], module: 'admin'
  namespace :admin do
    scope module: :ajax do
      resources :streams, only: :index
      get '/find_courses', to: 'courses#index', constraints: ->(r) { r.xhr? }
      get '/find_users', to: 'users#index', constraints: ->(r) { r.xhr? }
    end
    resources :polls, except: %i[show] do
      resources :options, only: %i[create destroy], controller: 'poll_options'
    end
  end
  get 'polls/next', to: 'polls#next', as: :next_poll
  get 'polls/archive', to: 'polls#archive', as: :polls_archive
  post 'polls/:id/vote', to: 'polls#vote', as: :vote_poll

  scope path: 'admin/statistics', module: 'admin', as: :admin_statistics do
    get 'activity', to: 'statistics#activity', as: :activity
    get 'courses', to: 'statistics#courses', as: :courses
    get 'geo', to: 'statistics#geo', as: :geo
    get 'news', to: 'statistics#news', as: :news
    get 'referrer', to: 'statistics#referrer', as: :referrer
    get 'social', to: 'statistics#social', as: :social
  end
  scope '/courses/:course_id/statistics', module: 'course/admin', as: :course_statistics do
    get 'activity', to: 'statistics#activity', as: :activity
    get 'geo', to: 'statistics#geo', as: :geo
    get 'news', to: 'statistics#news', as: :news
    get 'pinboard', to: 'statistics#pinboard', as: :pinboard
    get 'social', to: 'statistics#social', as: :social
    get 'referrer', to: 'statistics#referrer', as: :referrer
    get 'item_visits', to: 'statistics#item_visits', as: :item_visits
    get 'quiz', to: 'statistics#quiz', as: :quiz
    get 'item_details', to: 'statistics#item_details', as: :item_details
    get 'videos', to: 'statistics#videos', as: :videos
    get 'downloads', to: 'statistics#downloads', as: :downloads
    get 'rich_texts', to: 'statistics#rich_texts', as: :rich_texts
  end

  scope module: 'peer_assessment' do
    resources :peer_assessments do
      member do
        post :start_assessment
        get :files
        post :upload
        delete :remove_file
        put :advance
      end

      resources :conflicts do
        member do
          post :reconcile
          put :reconcile # to use the create_review stuff with no_review conflicts
        end
      end

      resources :submissions, only: [] do
        resources :files, only: [], controller: 'submission_files' do
          member do
            get :gallery
          end
        end
      end

      resources :submission_management, only: %i[index show] do
        member do
          post :rate
          post :grant_attempt
          get :submission_details
          post :request_regrading
        end

        collection do
          get :generate_gallery
        end
      end

      resource :error, only: [:show]

      resources :rubrics do
        member do
          post :moveup
          post :movedown
          post :remove_option
        end
      end

      resources :team_evaluation_rubrics, except: %i[edit update] do
        member do
          post :moveup
          post :movedown
        end
      end

      resources :train_samples do
        collection do
          post :open_training
        end

        member do
          patch :autosave
          post :extend_deadline
        end
      end

      # Controllers handling the individual steps
      resources :steps do
        member do
          get :locked
          get :deadline_passed
          get :inaccessible
          # get :skip    # Decision whether to advance or skip will be made in the peer assessment service
          get :advance
        end

        collection do
          post :setup
          post :update
        end

        resource :submission do
          member do
            # Advancement is handled through the update method
            get :additional_attempt
            patch :additional_attempt_update
            patch :autosave
            post :upload
            delete :remove_file
          end
        end

        resources :training, only: %i[new update] do
          member do
            patch :autosave
          end

          collection do
            get :evaluate
            put :advance
          end
        end

        resources :reviews do
          member do
            patch :autosave
            post :extend_deadline
          end

          collection do
            post :report
            put :advance
          end
        end

        resource :self_assessments do
          member do
            # Advancement is handled through the update method
            patch :autosave
            put :advance
          end
        end

        resource :results do
          collection do
            # No advancement required
            post :report
            get '/:review_id', controller: :results, action: :show_review, as: :review_result
            post '/rate/:review_id', controller: :results, action: :rate_review, as: :rate_review
            post :request_regrading
          end
        end
      end
    end

    # Notes are somewhat separate from the rest of the peer assessment components
    resources :notes
  end

  resources :channels, only: %i[create update new edit destroy], module: 'admin'
  resources :channels, only: %i[show], module: 'home'
  namespace :admin do
    resources :channels, only: %i[index]
    get '/channels/order', to: 'channels_order#index'
    post '/channels/order', to: 'channels_order#update'
  end

  resources :courses, only: %i[index], module: 'home'
  resources :courses, only: %i[create update new edit], module: 'admin'
  resources :courses, only: %i[show], module: 'course' do
    resources :free_reactivations, only: [:create]
  end
  resources :courses, only: [] do
    get 'launch(/:auth)', controller: 'course/launch', to: 'course/launch#launch', as: 'launch'
    get 'certificates', to: 'course/certificates#index', as: :certificates
    get 'certificate', to: 'course/certificates#show', as: :certificate, module: 'course'

    # Open Badges
    get 'badge', to: 'open_badges#badge_class', as: :badge_class
    get 'assertion/:id', to: 'open_badges#assertion', as: :assertion

    namespace :openbadges, module: :open_badges do
      scope :v2 do
        get 'class', to: 'open_badges#badge_class', as: :class_v2
        get 'assertion/:id', to: 'open_badges#assertion', as: :assertion_v2
      end
    end

    resources :peer_assessments, except: [:show], module: 'peer_assessment'
    # make course announcements availabe as /courses/[id]/announcements
    resources :announcements, except: [:show], module: 'course'
    resources :lti_providers, except: %i[edit new show]

    get 'book/:product', to: 'course/voucher_redemptions#new', as: 'redeem_voucher'
    post 'book/:product', to: 'course/voucher_redemptions#create'

    resources :abuse_reports, only: [:index], module: 'course/admin'

    # Course permissions
    resources :permissions, only: [:index], module: 'course/admin'
    resources :groups, only: [], module: 'course/admin' do
      resources :members, only: %i[create destroy], controller: 'permissions'
    end
    resource :visual, only: %i[edit update], module: 'course/admin'
    resource :metadata, only: %i[show edit update destroy], module: 'course/admin'
    resources :offers, only: %i[index create update new edit destroy], module: 'course/admin'
    resources :certificate_templates, only: %i[index create update new edit destroy], module: 'course/admin' do
      member do
        get :preview, action: :preview_certificate
      end
    end
    resources :open_badge_templates, only: %i[index create update new edit destroy], module: 'course/admin'

    resources :objectives, only: %i[index new create edit update destroy], module: 'course/admin'

    resource :recalculations, only: %i[create], module: 'course/admin/learning_evaluation'

    resource :progress, only: [:show], module: 'course'

    resources :items, only: %i[show edit] do
      resources :quiz_submission, only: %i[new show create]
      resources :quiz_submission_snapshot, only: [:create]
      resources :topics, only: [:create], module: 'course/ajax'
    end

    resources :sections, except: %i[new edit] do
      # resources :pinboard, only: [ :index ]
      pinboard_routes
      member do
        post :move
        post :choose_alternative_section
      end
      resources :items, except: [:index], as: :items do
        member do
          post :move
          post :extract_thumbnails
          post :sync_video
        end
        resources :quiz_questions, except: %i[index new], as: :quiz_questions do
          member do
            post :move
          end
          resources :quiz_answers, except: [:index], as: :quiz_answers do
            member do
              post :move
            end
          end
        end
        resource :time_effort, only: %i[show update destroy], controller: 'item_time_effort', module: 'course/admin'
      end
    end
    pinboard_routes
    resources :learning_rooms, only: [] do
      pinboard_routes
    end
    resources :learning_rooms, controller: :collabspaces, module: :collabspace do
      resource :calendar, only: %i[show]
      resources :calendar_events, only: %i[index update], controller: 'ajax/calendar_events', constraints: ->(r) { r.xhr? }
      resources :calendar_events, only: %i[new create edit update destroy]
      resources :files, only: %i[index create destroy]
      get '/video_chat', to: 'video_chat#index', as: :video_chat
      resources :memberships, only: %i[create destroy update]
    end
  end

  scope '/courses/:course_id/items/:id' do
    get '/lti', to: 'items#show', as: :course_item_lti, defaults: {lti: true}
    get '/tool_launch', to: 'lti#tool_launch', as: :tool_launch_course_item
    post '/tool_grading', to: 'lti#tool_grading', as: :tool_grading_course_item, defaults: {format: 'xml'}
    get '/tool_return', to: 'lti#tool_return', as: :tool_return_course_item, defaults: {format: 'xml'}
  end
  # legacy paths:
  scope '/courses/:course_id/sections/:section_id/items/:id' do
    get '/tool_launch', to: 'lti#tool_launch', as: nil
    post '/tool_grading', to: 'lti#tool_grading', as: nil, defaults: {format: 'xml'}
    get '/tool_return', to: 'lti#tool_return', as: nil, defaults: {format: 'xml'}
  end

  scope module: :course do
    resources :enrollments, only: %i[create destroy] do
      member do
        post '/completion', to: 'enrollments/completion#create', as: :complete
      end
    end
    # TODO: refactor ugly hack to allow enrollments after redirect to login
    get '/enrollments', to: 'enrollments#create', as: 'create_enrollment'
  end

  # legacy route
  get '/course/:id', to: 'course/courses#show'
  get '/pinboard_tags', to: 'pinboard#tags'
  get 'dbpb', to: 'pinboard#admin_tags'
  delete '/tag/:id', to: 'pinboard#destroy', as: :tag_delete
  resources :ping, only: [:index]

  post '/helpdesk', to: 'helpdesk#send_helpdesk'
  get '/helpdesk', to: 'helpdesk#show'
  get '/dashboard', to: 'dashboard#dashboard'
  get '/dashboard/achievements', to: 'dashboard#achievements'
  get '/dashboard/documents', to: 'dashboard#documents'

  post 'courses/:course_id/add_attempt', to: 'course/admin/quiz_submissions#add_attempt', as: :add_attempt
  post 'courses/:course_id/add_fudge_points', to: 'course/admin/quiz_submissions#add_fudge_points', as: :add_fudge_points
  post 'courses/:course_id/exclude_from_proctoring', to: 'course/admin/quiz_submissions#exclude_from_proctoring', as: :exclude_from_proctoring
  get '/courses/:id/dashboard', to: 'course/admin/dashboard#show', as: :course_dashboard
  get '/courses/:id/statistic', to: 'admin/course_management#statistic', as: :course_statistic
  get '/courses/:id/documents', to: 'course/documents#index', as: :course_documents
  get '/courses/:id/submission_statistics', to: 'admin/course_management#submission_statistics', as: :course_submission_statistics
  get '/courses/:course_id/items/:item_id/stats', to: 'course/admin/item_stats#show', as: :course_item_statistics
  get '/courses/:course_id/enrollments', to: 'course/admin/enrollments#index', as: :course_enrollments
  post '/courses/:course_id/enrollments', to: 'course/admin/enrollments#create', as: :teacher_create_enrollment
  delete '/courses/:course_id/enrollments/:user_id', to: 'course/admin/enrollments#destroy', as: :teacher_destroy_enrollment
  get '/courses/:id/submissions', to: 'admin/course_management#submissions', as: :course_submissions
  post 'courses/:id/convert_submission', to: 'admin/course_management#convert_submission', as: :convert_submission
  get '/courses/:id/resume', to: 'course/courses#resume', as: :course_resume
  post '/courses/:id/preview_quizzes', to: 'admin/course_management#preview_quizzes', as: :preview_quizzes
  post '/courses/:id/import_quizzes', to: 'admin/course_management#import_quizzes', as: :import_quizzes
  post '/courses/:id/import_quizzes_by_service', to: 'admin/course_management#import_quizzes_by_service', as: :import_quizzes_by_service
  get '/news', to: 'home/announcements#index', as: 'news_index'
  resources :announcements, path: 'news', only: %i[new create edit update destroy]
  get '/news/:id/', to: 'home/announcements#index' # as we dont have a index route yet
  get '/courses/:course_id/announcements/:id', to: 'course/announcements#index'
  get '/courses/:course_id/overview', to: 'course/syllabus#show', as: :course_overview
  root to: 'home/home#index'

  # The web manifest (for home screen installation on mobile devices)
  get '/web_manifest.json', to: 'web_manifest#show'

  # ChromeCast Styled Media Receiver
  get '/chromecast.css', to: 'chromecast#stylesheet', defaults: {format: 'css'}

  scope '.well-known', module: 'well_known' do
    get 'assetlinks.json', to: 'app_links#android'
    get 'apple-app-site-association', to: 'app_links#ios'
    get ':filename', to: 'files#show', constraints: {filename: %r{[^/]+}}
  end

  get '/pages/:id', to: 'home/pages#show', as: 'page'
  resources :pages, only: %i[edit edit update]

  get '/files/logo', to: 'files#logo'
  get '/favicon.ico', to: 'files#favicon'
  get '/avatar/:id(/size/:size)', to: 'files#avatar', as: :avatar

  post '/subscriptions/toggle_subscription/:question_id', to: 'subscriptions#toggle_subscription', as: :toggle_subscription
  post '/subscriptions/unsubscribe/:question_id', to: 'subscriptions#unsubscribe', as: :unsubscribe
  get '/subscriptions/subscription_count_text', to: 'subscriptions#subscription_count_text', as: :subscription_count_text

  resource :notification_user_disables, path: '/notification_user_settings/disable', only: %i[show create]

  resources :user_tests, module: 'admin'
  patch 'user_tests/:identifier/finish', to: 'admin/user_tests#finish'

  get '/learn', to: 'learning_mode#index'
  get '/learn/review', to: 'learning_mode#review'

  resources :video_providers, module: 'admin' do
    member do
      resource :sync, only: %i[create], controller: 'video_provider_sync', as: :sync_video_provider
    end
  end

  get 'ical', to: 'ical#index'

  # mount Rack based API app
  mount Xikolo::API => '/api'

  get '/courses/:course_id/proctoring/registration_details', to: 'proctoring#registration_details', as: :proctoring_registration_details
  get '/courses/:course_id/proctoring/register_at_smowl', to: 'proctoring#register_at_smowl', as: :proctoring_register_at_smowl

  namespace :admin do
    resources :clusters, only: %i[index show edit update] do
      resources :classifiers, only: %i[new create edit update destroy] do
        get '/courses/order', to: 'classifier/courses_order#index'
        post '/courses/order', to: 'classifier/courses_order#update'
      end
      get '/classifiers/order', to: 'cluster/classifiers_order#index'
      post '/classifiers/order', to: 'cluster/classifiers_order#update'
    end
    resources :lti_providers, except: [:show]
    resources :announcements, except: %i[show destroy] do
      member do
        resource :email, as: 'announcement_email', controller: 'announcement_emails', only: %i[show new create]
      end
    end
    namespace :announcement do
      resources :recipients, only: [:index]
    end
  end

  # all routes for go, the shortcut for redirects etc.
  scope 'go', module: :home, as: 'go' do
    get '/items/:id', to: 'go#item'
    get '/items/:id/pinboard', to: 'go#pinboard'
    get '/link', to: 'go#redirect'
    get '/launch/:course_id(/:auth)', to: 'go#course'
    get '/survey/:id', to: 'go#survey'
  end

  # Public API for customers portal
  scope 'portalapi-beta', module: :portal_api, as: 'portal' do
    resources :courses, only: %i[index show]
    resources :users, only: %i[update destroy]
    resources :enrollments, only: %i[index create]
  end

  scope 'bridges', module: :bridges, as: 'bridges' do
    # Internal Bridge Transpipe API
    scope 'transpipe', module: :transpipe, as: 'transpipe' do
      resources :courses, only: %i[index show]
      resources :videos, only: :show do
        member do
          get 'subtitles/:lang', to: 'subtitles#show'
          patch 'subtitles/:lang', to: 'subtitles#update'
        end
      end
    end

    scope 'lanalytics', module: :lanalytics, as: 'lanalytics', defaults: {format: 'json'} do
      get '/', to: 'root#index'

      scope 'courses/:course_id', as: 'course' do
        resource :ticket_stats, controller: 'course_ticket_stats', only: %i[show]
        resource :open_badge_stats, controller: 'course_open_badge_stats', only: %i[show]
      end
    end

    scope 'moochub', module: :mooc_hub, as: 'moochub' do
      resources :courses, only: %i[index]
    end

    scope 'chatbot', module: :chatbot, as: 'chatbot' do
      post 'authenticate', to: 'authentications#create'
      get 'my_courses', to: 'my_courses#index'
      post 'my_courses/:id', to: 'my_courses#create'
      delete 'my_courses/:id', to: 'my_courses#destroy'
      get 'my_courses/:id/achievements', to: 'my_course_achievements#show'
      get 'courses', to: 'courses#index'
      get 'user', to: 'users#show'
    end

    scope 'shop', module: :shop, as: 'shop', defaults: {format: 'json'} do
      resources :vouchers, only: %i[index create]
    end
  end

  if Rails.env.development?
    mount Lookbook::Engine, at: '/rails/components'
  end
end
