import { inject as service } from '@ember/service';
import { computed } from '@ember/object';
import { alias } from '@ember/object/computed';
import Controller from '@ember/controller';
import { random, range, sampleSize, shuffle } from 'lodash-es';

export default Controller.extend({
  queryParams: ['course_id'],
  page: 0,
  pageIndex: 1,
  questions: alias('model'),
  pages: computed('questions', function() {
    const totalPages = this.get('questions.length');
    return shuffle(range(totalPages));
  }),
  currentQuestion: computed('questions', 'page', function() {
    return this.get('questions').objectAt(this.get('page'));
  }),
  recapSessionId: null,
  activeLearnSession: false,
  showResults: false,
  reviewSession: false,

  tracking: service(),

  init() {
    this._super(...arguments);

    this.currentPages = this.currentPages || [];
  },

  course: computed('course_id', function() {
    const courseId = this.get('course_id');
    if (courseId) {
      return this.store.find('course', courseId);
    }
  }),

  correctQuestions: computed('questions.@each.correctByUser', function() {
    return this.get('questions').filterBy('correctByUser', true).get('length');
  }),
  wrongQuestions: computed('questions.@each.correctByUser', function() {
    return this.get('questions').filterBy('correctByUser', false).get('length');
  }),
  wrongCounter: 0,
  totalAttempts: computed('wrongCounter', 'correctQuestions', function() {
    return this.get('wrongCounter') + this.get('correctQuestions');
  }),

  finishLearnSession({ stoppedByUser } = { stoppedByUser: false }) {
    this.set('page', 0);
    this.set('pageIndex', 1);
    this.set('activeLearnSession', false);
    this.set('showResults', true);

    this.get('tracking').track({
      verb: 'recap_stopped',
      resource: this.get('course_id'),
      resourceType: 'course',
      context: {
        recap_session_id: this.get('recapSessionId'),
        correct_count: this.get('correctQuestions'),
        wrong_count: this.get('wrongCounter'),
        stopped_by_user: stoppedByUser,
      },
    });
  },
  resetLearnSession() {
    this.store.peekAll('question').forEach(function(question) {
      question.set('correctByUser', null);
      question.set('count', 0);
      question.resetAnswers();
    });
    this.set('recapSessionId', window.generateUUID());
    this.set('activeLearnSession', true);
    this.set('showResults', false);
    this.set('reviewSession', false);
    this.set('wrongCounter', 0);
  },
  checkQuestion(question) {
    question.checkUserAnswers();
    question.set('showSolution', true);
    question.set('count', question.get('count') + 1);
    let seconds = 2;
    if (!question.get('correctByUser')) {
      this.set('wrongCounter', this.get('wrongCounter') + 1);
      seconds = 4;
    }

    this.get('tracking').track({
      verb: 'recap_answered',
      resource: this.get('course_id'),
      resourceType: 'course',
      context: {
        recap_session_id: this.get('recapSessionId'),
        question_id: question.id,
        correct: question.get('correctByUser'),
      },
    });

    setTimeout(() => this.nextQuestion(question), seconds * 1000);
  },
  nextQuestion(question) {
    question.set('showSolution', false);
    const pages = this.get('currentPages');
    const index = this.get('pageIndex');

    if (!question.get('correctByUser') && question.get('count') < 3) {
      this.set('reviewSession', true);
      // Adaptive Algorithm: Repeat wrong questions after 5-10 questions
      // with maximum of 3 attempts
      const newIndex = random(5, 10);
      const page = this.get('page');
      if (index + newIndex >= pages.length) {
        pages.push(page);
      } else {
        pages.splice(index + newIndex, 0, page);
      }
      // TODO reset should shuffle answers again
      question.resetAnswers();
      this.set('currentPages', pages);
    }

    if (index >= pages.length) {
      this.finishLearnSession();
    } else {
      this.set('page', pages[index]);
      this.set('pageIndex', index + 1);
    }
  },

  actions: {
    start(sample) {
      this.resetLearnSession();

      if (sample) {
        this.set('currentPages', sampleSize(this.get('pages'), sample));
      } else {
        this.set('currentPages', this.get('pages').slice());
      }

      this.get('tracking').track({
        verb: 'recap_started',
        resource: this.get('course_id'),
        resourceType: 'course',
        context: {
          recap_session_id: this.get('recapSessionId'),
          questions_count: this.get('currentPages.length'),
          questions_total: this.get('pages.length'),
        },
      });

      this.set('page', this.get('currentPages')[0]);
    },
    answerSelector(answer, question) {
      if (question.get('isMultipleChoiceType')) {
        question.get('answers').forEach(function(answerItem) {
          answerItem.set('selected', false);
        });
        answer.set('selected', true);
        this.checkQuestion(question);
      } else if (question.get('isMultipleAnswerType')) {
        answer.set('selected', true);
        if (
          answer.get('correct') === false ||
          question.correctAnswersByUser()
        ) {
          this.checkQuestion(question);
        }
      }
    },
    stopLearnSession() {
      this.finishLearnSession({ stoppedByUser: true });
    },
  },
});
