import { Promise } from 'rsvp';
import { bool } from '@ember/object/computed';
import { computed } from '@ember/object';
import { inject as service } from '@ember/service';
import DS from 'ember-data';
import moment from 'moment';

export default DS.Model.extend({
  title: DS.attr('string'),
  slug: DS.attr('string'),
  startAt: DS.attr('moment'),
  endAt: DS.attr('moment'),
  abstract: DS.attr('string'),
  language: DS.attr('string'),
  status: DS.attr('string'),
  classifiers: DS.attr(),
  teachers: DS.attr('string'),
  certificates: DS.attr(),

  channel: DS.belongsTo('channel'),
  sections: DS.hasMany('course-section', { async: false }),
  userEnrollment: DS.belongsTo('enrollment', { async: false }),
  documents: DS.hasMany('document'),
  dates: DS.hasMany('course-dates'),
  repetitionSuggestions: DS.hasMany('repetition-suggestions'),

  features: service(),
  config: service(),

  lang: computed('language', function() {
    const language = this.get('language');
    return language ? this.intl.t('languages.name.' + language) : '';
  }),

  isEnrolled: bool('userEnrollment'),

  current: computed('future', 'old', function() {
    return !this.get('future') && !this.get('old');
  }),

  future: computed('startAt', function() {
    if (!this.get('startAt')) {
      return true;
    }

    return moment().isBefore(this.get('startAt'));
  }),

  old: computed('endAt', function() {
    if (!this.get('endAt')) {
      return false;
    }

    return moment().isAfter(this.get('endAt'));
  }),

  items: computed('sections.{[],items.[]}', function() {
    return DS.PromiseArray.create({
      promise: Promise.all(this.get('sections').mapBy('items')).then(
        (results) =>
          results.reduce(
            (allItems, sectionItems) =>
              allItems.toArray().concat(sectionItems.toArray()),
            []
          )
      ),
    });
  }),

  progress: computed('id', function() {
    return this.store.findRecord('course-progress', this.get('id'), {
      include: 'section_progresses',
    });
  }),

  searchHaystack: computed(
    'title',
    'slug',
    'abstract',
    'teachers',
    'classifiers.*',
    function() {
      return [
        this.get('title'),
        this.get('slug'),
        this.get('abstract'),
        this.get('teachers'),
        ...Object.keys(this.get('classifiers')).map((prop) =>
          this.get('classifiers')[prop].join(' ')
        ),
      ]
        .filter((val) => typeof val === 'string')
        .map((str) => str.toLowerCase());
    }
  ),

  matches(val) {
    if (val.length === 0) {
      return true;
    }

    const needles = val.toLowerCase().split(' ');

    return needles.every((needle) =>
      this.get('searchHaystack').any((str) => str.indexOf(needle) !== -1)
    );
  },
});
