# frozen_string_literal: true

require 'spec_helper'

describe 'Patches: Rails: Cache: Reconnect on ReadOnlyError Redis error' do
  subject(:store) { ActiveSupport::Cache::RedisCacheStore.new(**redis_opts) }

  let(:redis_opts) do
    {
      error_handler:,
      url: ENV.fetch('REDIS_URL', nil),
    }
  end
  let(:errors) do
    [
      Redis::ReadOnlyError.new,
    ]
  end
  let(:error_handler) { double('Error handler', call: true) }

  before do
    store.clear

    # Stub multiple Redis writes to only succeed after a few errors
    allow_any_instance_of(Redis).to(
      receive(:set).and_wrap_original do |original, *args|
        raise errors.shift unless errors.empty?

        original.call(*args)
      end
    )
  end

  it 'ignores the first ReadOnlyError error and re-connects' do
    expect(store.redis).to receive(:disconnect!).once.and_call_original

    val = store.fetch('key') { 'value' } # rubocop:disable Style/RedundantFetchBlock
    expect(val).to eq 'value'

    expect(store.redis).to have_received(:set).twice
    expect(error_handler).not_to have_received(:call)
  end

  context 'with error on retry' do
    let(:errors) do
      [
        Redis::ReadOnlyError.new('attempt #1'),
        Redis::ReadOnlyError.new('attempt #2'),
      ]
    end

    it 'retries once and then handles the second error as usual (by swallowing and reporting it)' do
      val = store.fetch('key') { 'value' } # rubocop:disable Style/RedundantFetchBlock
      expect(val).to eq 'value'

      expect(store.redis).to have_received(:set).twice
      expect(error_handler).to have_received(:call).once.with(
        hash_including(exception: an_object_having_attributes(message: /attempt #2/))
      )
    end
  end

  # When configured to use a connection pool, the store's internal "redis"
  # object has a very different type, leading to awkward implementations. ;)
  context 'when configured to use connection pooling' do
    let(:redis_opts) { super().merge(pool_size: 2) }

    it 'ignores the first ReadOnlyError error and dumps all connections from the pool' do
      expect(store.redis).to receive(:reload).once.and_call_original

      val = store.fetch('key') { 'value' } # rubocop:disable Style/RedundantFetchBlock
      expect(val).to eq 'value'

      expect(error_handler).not_to have_received(:call)
    end

    context 'with error on retry' do
      let(:errors) do
        [
          Redis::ReadOnlyError.new('attempt #1'),
          Redis::ReadOnlyError.new('attempt #2'),
        ]
      end

      it 'retries once and then handles the second error as usual (by swallowing and reporting it)' do
        expect(store.redis).to receive(:reload).once.and_call_original

        val = store.fetch('key') { 'value' } # rubocop:disable Style/RedundantFetchBlock
        expect(val).to eq 'value'

        expect(error_handler).to have_received(:call).once.with(
          hash_including(exception: an_object_having_attributes(message: /attempt #2/))
        )
      end
    end
  end
end
