# frozen_string_literal: true

class BrowserSupport
  def initialize(browser)
    @browser = browser
  end

  ##
  # A browser that we actively discourage using.
  #
  def unsupported?
    return false if @browser.bot?
    return true if @browser.ie?

    @browser.edge? && @browser.version.to_i < 107
  end

  VERSIONS_CONSIDERED_MODERN = {
    chrome: 108,
    edge: 109,
    firefox: 102,
    opera: 94,
    safari: 15,
  }.freeze

  ##
  # A browser that we may not fully support.
  #
  def old?
    return false if @browser.bot?
    return true if @browser.ie?
    return false if firefox_on_ipad? && (@browser.version.to_i > 68)

    VERSIONS_CONSIDERED_MODERN.any? do |browser, minimum|
      @browser.send("#{browser}?") && @browser.version.to_i < minimum
    end
  end

  VERSIONS_SUPPORTING_ES6 = {
    chrome: 49,
    firefox: 52,
    opera: 36,
    safari: 10,
  }.freeze

  ##
  # A browser with support for modern JavaScript syntax.
  # This type of browser can often be served smaller JS bundles.
  #
  def es6?
    return false if @browser.bot?

    VERSIONS_SUPPORTING_ES6.any? do |browser, minimum|
      @browser.send("#{browser}?") && @browser.version.to_i >= minimum
    end
  end

  private

  def firefox_on_ipad?
    # Firefox version numbers are different for iOS and
    # the browser gem does not detect iPads with iOS 13 as iPad but Mac.
    # Thus, we check for platform.mac? in combination with webkit? instead.
    @browser.platform.mac? && @browser.webkit? && @browser.firefox?
  end
end
