# frozen_string_literal: true

class Group < ApplicationRecord
  NAME_REGEXP = /\A\w+(\.(\w|-)+)+\z/
  ALLOWED_TAGS = %w[access custom_recipients].freeze

  module NAME
    ACTIVE              = 'active'
    ADMINISTRATORS      = 'xikolo.admins'
    ADMINISTRATORS_GDPR = 'xikolo.gdpr_admins'
    AFFILIATED          = 'xikolo.affiliated'
    ALL                 = 'all'
    ARCHIVED            = 'archived'
    CONFIRMED           = 'confirmed'
    UNCONFIRMED         = 'unconfirmed'
  end

  has_many :memberships, dependent: :destroy
  has_many :features, as: :owner, dependent: :destroy, inverse_of: :owner
  has_many :grants, as: :principal, dependent: :destroy, inverse_of: :principal

  has_many :members, -> { active }, through: :memberships, source: :user

  validates :name,
    presence: {message: 'required'},
    uniqueness: {case_sensitive: false, message: 'invalid'}

  validates :name,
    format: {message: 'invalid', with: NAME_REGEXP},
    if: -> { validation.nil? || validation }

  validate :allowed_tags

  attr_accessor :validation

  def members
    case name
      when NAME::ACTIVE
        User.active
      when NAME::ALL
        User.all
      when NAME::ARCHIVED
        User.archived
      when NAME::CONFIRMED
        User.confirmed
      when NAME::UNCONFIRMED
        User.unconfirmed
      else
        super
    end
  end

  def member_ids_relation
    case name
      when NAME::ACTIVE
        User.active.select(:id)
      when NAME::ALL
        User.select(:id)
      when NAME::ARCHIVED
        User.archived.select(:id)
      when NAME::CONFIRMED
        User.confirmed.select(:id)
      when NAME::UNCONFIRMED
        User.unconfirmed.select(:id)
      else
        memberships.select(:user_id)
    end
  end

  class << self
    def prefix(value)
      where('name LIKE ?', "#{sanitize_sql_like(value)}%")
    end

    def resolve(param)
      case param.to_s
        when NAME::ACTIVE
          active_users
        when NAME::ADMINISTRATORS
          administrators
        when NAME::ADMINISTRATORS_GDPR
          gdpr_admins
        when NAME::AFFILIATED
          affiliated_users
        when NAME::ALL
          all_users
        when NAME::ARCHIVED
          archived_users
        when NAME::CONFIRMED
          confirmed_users
        when NAME::UNCONFIRMED
          unconfirmed_users
        else
          find_by! name: param.to_s
      end
    end

    def for(principal)
      if principal.respond_to?(:all_groups)
        principal.all_groups
      else
        with_member(principal)
      end
    end

    def with_member(member)
      where id: Membership.with_member(member).select(:group_id)
    end

    def active_users
      ensure! name: NAME::ACTIVE, description: 'Active users'
    end

    def gdpr_admins
      ensure! name: NAME::ADMINISTRATORS_GDPR, description: 'Administrative users (with access to personal information)'
    end

    def administrators
      ensure! name: NAME::ADMINISTRATORS, description: 'Administrative users'
    end

    def affiliated_users
      ensure! name: NAME::AFFILIATED, description: 'Affiliated users', tags: %w[access]
    end

    def all_users
      ensure! name: NAME::ALL, description: 'All users'
    end

    def archived_users
      ensure! name: NAME::ARCHIVED, description: 'Archived users'
    end

    def confirmed_users
      ensure! name: NAME::CONFIRMED, description: 'Confirmed users'
    end

    def unconfirmed_users
      ensure! name: NAME::UNCONFIRMED, description: 'Unconfirmed users'
    end

    %i[
      active_users
      all_users
      archived_users
      confirmed_users
      unconfirmed_users
    ].each do |name|
      module_eval <<-RUBY, __FILE__, __LINE__ + 1
        def #{name}_id                                           # def all_users_id
          Rails.cache.fetch('group/id/#{name}') { #{name}.id }   #   Rails.cache.fetch('group/id/all_users') { all_users.id }
        end                                                      # end

        def #{name}_scope                                        # def all_users_scope
          where id: #{name}_id                                   #   where id: all_users_id
        end                                                      # end
      RUBY
    end

    private

    def ensure!(name:, description:, **)
      Group
        .where(name:)
        .first_or_create!(description:, validation: false, **)
    end
  end

  def to_param
    name
  end

  private

  def allowed_tags
    errors.add(:tags, 'invalid') if (tags - ALLOWED_TAGS).any?
  end
end
