# frozen_string_literal: true

require 'spec_helper'

RSpec.describe ActivityStream::CreateEventJob, type: :job do
  let!(:collab_space) { create(:collab_space) }

  before do
    Stub.service(
      :account,
      group_url: '/groups/{id}',
      user_url: '/users/{id}'
    )
    Stub.service(
      :course,
      course_url: '/courses/{id}'
    )
    Stub.service(
      :notification,
      events_url: '/events'
    )
  end

  describe 'scheduling via ActivityStreamEvent' do
    subject(:enqueue_job) { event.notify_later! }

    let(:event) do
      ActivityStreamEvent.new(
        'the_event',
        collab_space,
        url: '/path/to/resource',
        payload: {}
      )
    end

    it 'enqueues correctly' do
      expect { enqueue_job }.to have_enqueued_job(described_class)
        .with('the_event', collab_space.id, hash_including(:url, :payload))
    end
  end

  describe '#perform_now' do
    subject(:perform_job) do
      described_class.perform_now(
        'the_event',
        collab_space.id,
        url: '/path',
        payload:
      )
    end

    let(:payload) { {} }
    let(:admin1_id) { SecureRandom.uuid }
    let(:admin2_id) { SecureRandom.uuid }

    let!(:create_event) do
      Stub.request(
        :notification, :post, '/events'
      ).to_return Stub.json({})
    end

    before do
      Stub.request(
        :course, :get, "/courses/#{collab_space.course_id}"
      ).to_return Stub.json({course_code: 'the_course'})

      Stub.request(
        :account, :get, '/groups/course.the_course.admins'
      ).to_return Stub.json({members_url: '/groups/course.the_course.admins/members'})

      Stub.request(
        :account, :get, '/groups/course.the_course.admins/members'
      ).to_return Stub.json([
        {id: admin1_id},
        {id: admin2_id},
      ])
    end

    it 'sends the event to the activity stream' do
      perform_job
      expect(
        create_event.with(
          body: hash_including(
            key: 'learning_room.the_event',
            public: false,
            link: '/path'
          )
        )
      ).to have_been_requested
    end

    describe 'the subscribers array' do
      it 'is included' do
        create_event.with do |request|
          expect(JSON.parse(request.body)).to match hash_including('subscribers' => kind_of(Array))
        end
        perform_job
      end

      it 'contains the IDs of all members and the course\'s admins' do
        create_event.with do |request|
          expect(
            JSON.parse(request.body)['subscribers']
          ).to contain_exactly(admin1_id, admin2_id, collab_space.memberships.first.user_id)
        end
        perform_job
      end

      context 'when it includes a user ID' do
        let(:payload) { super().merge(new_member_id: user_id) }
        let(:user_id) { SecureRandom.uuid }

        before do
          Stub.request(
            :account, :get, "/users/#{user_id}"
          ).to_return Stub.json({name: 'Bob'})
        end

        it 'is enriched with the user\'s name' do
          create_event.with do |request|
            expect(JSON.parse(request.body)['payload']).to include 'new_member_name' => 'Bob'
          end
          perform_job
        end
      end
    end

    describe 'the event payload' do
      it 'is included' do
        create_event.with do |request|
          expect(JSON.parse(request.body)).to match hash_including('payload' => kind_of(Hash))
        end
        perform_job
      end

      context 'when it includes a user ID' do
        let(:payload) { super().merge(new_member_id: user_id) }
        let(:user_id) { SecureRandom.uuid }

        before do
          Stub.request(
            :account, :get, "/users/#{user_id}"
          ).to_return Stub.json({name: 'Bob'})
        end

        it 'is enriched with the user\'s name' do
          create_event.with do |request|
            expect(JSON.parse(request.body)['payload']).to include 'new_member_name' => 'Bob'
          end
          perform_job
        end
      end
    end
  end
end
