# frozen_string_literal: true

require 'spec_helper'

describe 'Richtext: Create', type: :request do
  subject(:creation) do
    api.rel(:richtexts).post(
      params
    ).value!
  end

  let(:api) { Restify.new(:test).get.value! }
  let(:course) { create(:course) }
  let(:cid) { UUID4(course.id).to_str(format: :base62) }
  let(:text) { 'Some Text' }
  let(:params) { {text:, course_id: course.id} }

  it { is_expected.to respond_with :created }

  it 'creates a new result object' do
    expect { creation }.to change(Richtext, :count).from(0).to(1)
  end

  context 'with description file reference' do
    let(:file_ref) { 's3://xikolo-public/course/34/rtfiles/1/course.jpg' }
    let(:text) { "Headline\n#{file_ref}" }

    context 'from current course' do
      let(:course) { create(:course, description: "Some\n#{file_ref}") }

      it 'accept the references' do
        expect { creation }.to change(Richtext, :count).by(1)
        expect(Richtext.find(creation['id']).text).to include file_ref
      end
    end

    context 'from other course' do
      before { create(:course, description: "Some\n#{file_ref}") }

      it 'rejects the references' do
        expect { creation }.to raise_error(Restify::ClientError) do |error|
          expect(error.status).to eq :unprocessable_entity
          expect(error.errors).to eq 'text' => ['rtfile_unknown_ref']
        end
      end
    end
  end

  context 'with richtext file reference' do
    let(:file_ref) { 's3://xikolo-public/course/34/rtfiles/1/course.jpg' }
    let(:text) { "Headline\n#{file_ref}" }

    context 'from current course' do
      before { create(:richtext, course:, text: "Some\n#{file_ref}") }

      it 'accept the references' do
        expect { creation }.to change(Richtext, :count).by(1)
        expect(Richtext.find(creation['id']).text).to include file_ref
      end
    end

    context 'from other course' do
      before { create(:richtext, text: "Some\n#{file_ref}") }

      it 'rejects the references' do
        expect { creation }.to raise_error(Restify::ClientError) do |error|
          expect(error.status).to eq :unprocessable_entity
          expect(error.errors).to eq 'text' => ['rtfile_unknown_ref']
        end
      end
    end
  end

  context 'with file upload references' do
    let(:text) { 'upload://b5f99337-224f-40f5-aa82-44ee8b272579/company1.jpg' }

    it 'stores valid upload and creates a new richtext' do
      stub_request(
        :head,
        'https://s3.xikolo.de/xikolo-uploads/' \
        'uploads/b5f99337-224f-40f5-aa82-44ee8b272579/company1.jpg'
      ).and_return(
        status: 200,
        headers: {
          'X-Amz-Meta-Xikolo-Purpose' => 'course_richtext',
          'X-Amz-Meta-Xikolo-State' => 'accepted',
        }
      )
      store_regex = %r{https://s3.xikolo.de/xikolo-public
                       /courses/#{cid}/rtfiles/[0-9a-zA-Z]+/company1.jpg}x
      stub_request(:head, store_regex).and_return(status: 404)
      stub_request(:put, store_regex).and_return(status: 200, body: '<xml></xml>')
      expect { creation }.to change(Richtext, :count).from(0).to(1)
    end

    it 'rejects invalid upload and does not creates a new page' do
      stub_request(
        :head,
        'https://s3.xikolo.de/xikolo-uploads/' \
        'uploads/b5f99337-224f-40f5-aa82-44ee8b272579/company1.jpg'
      ).and_return(
        status: 200,
        headers: {
          'X-Amz-Meta-Xikolo-Purpose' => 'course_richtext',
          'X-Amz-Meta-Xikolo-State' => 'rejected',
        }
      )

      expect { creation }.to raise_error(Restify::ClientError) do |error|
        expect(error.status).to eq :unprocessable_entity
        expect(error.errors).to eq 'text' => ['rtfile_rejected']
      end
    end

    it 'rejects upload on storage errors' do
      stub_request(
        :head,
        'https://s3.xikolo.de/xikolo-uploads/' \
        'uploads/b5f99337-224f-40f5-aa82-44ee8b272579/company1.jpg'
      ).and_return(
        status: 200,
        headers: {
          'X-Amz-Meta-Xikolo-Purpose' => 'course_richtext',
          'X-Amz-Meta-Xikolo-State' => 'accepted',
        }
      )
      store_regex = %r{https://s3.xikolo.de/xikolo-public
                       /courses/#{cid}/rtfiles/[0-9a-zA-Z]+/company1.jpg}x
      stub_request(:head, store_regex).and_return(status: 404)
      stub_request(:put, store_regex).and_return(status: 503)

      expect { creation }.to raise_error(Restify::ClientError) do |error|
        expect(error.status).to eq :unprocessable_entity
        expect(error.errors).to eq 'text' => ['rtfile_error']
      end
    end
  end
end
