# frozen_string_literal: true

class XikoloItemConsumer < Msgr::Consumer
  require 'errors'

  def create_or_update
    return unless Xikolo.config.learningobjectives['enabled']

    # 1. Find or create the shadow item for the created/updated course item
    item = retrieve_item

    # 2. Update fields that might have changed for the course item.
    # It is synced with the course service as we cannot rely on the
    # payload due to timing issues (async communication).
    course_item = retrieve_course_item!
    check_completion = meta_changed?(item, course_item)
    item.update! relevant_params(item, course_item)

    # 3. Check the completion of objectives including the (changed) item.
    # Most importantly, this is relevant for updating the user's
    # objective progresses if the item's max_points have changed.
    check_completion! if check_completion
  rescue Errors::CourseItemNotFound
    shadow_item&.destroy
  end

  def destroy
    return unless Xikolo.config.learningobjectives['enabled']

    item = shadow_item
    context_id = item&.meta&.dig('course_id')
    item&.destroy

    return unless context_id

    CompletionHandlers.check_completion_for!(
      context: context_id,
      item: item_id
    )
  end

  private

  def item_id
    payload.fetch :id
  end

  def context_id
    payload.fetch :course_id
  end

  def timestamp
    payload.fetch :updated_at
  end

  def retrieve_item
    Item.find_or_initialize_by(id: item_id) do |i|
      i.item_type = payload.fetch(:content_type)
      i.time_effort = payload.fetch(:time_effort, nil)
      i.meta = meta payload.stringify_keys
      i.save!
    end
  rescue ActiveRecord::RecordNotUnique
    retry
  end

  def retrieve_course_item!
    Xikolo.api(:course).value!.rel(:item).get(id: item_id).value!
  rescue Restify::NotFound
    raise Errors::CourseItemNotFound
  end

  def meta(item_attributes)
    {
      section_id: item_attributes['section_id'],
      course_id: item_attributes['course_id'],
      exercise_type: item_attributes['exercise_type'],
      max_points: item_attributes['max_points'].try(:*, 10.0)&.to_i,
    }.compact.merge(
      deadline: item_attributes['submission_deadline'],
      start_date: item_attributes['effective_start_date'],
      end_date: item_attributes['effective_end_date'],
      published: item_attributes['published']
    )
  end

  def meta_changed?(item, course_item)
    meta(item) != meta(course_item)
  end

  def relevant_params(item, course_item)
    {
      time_effort: course_item['time_effort'],
      meta: item.meta.merge(meta(course_item)),
    }
  end

  def check_completion!
    CompletionHandlers.check_completion_for!(
      context: context_id,
      item: item_id,
      timestamp:
    )
  end

  def shadow_item
    Item.find(item_id)
  rescue ActiveRecord::RecordNotFound
    # Do not fail if Item does not exist anymore
  end
end
