# frozen_string_literal: true

module CompletionHandlers
  module XikoloItemHelper
    ## Availability of items ##
    def was_available?(item)
      return false unless published?(item)
      return true if item.meta['start_date'].nil?

      item.meta['start_date'] <= Time.zone.now
    end

    def ongoing?(item)
      return false unless published?(item)
      return true if item.meta['end_date'].nil?

      item.meta['end_date'] >= Time.zone.now
    end

    def upcoming?(item)
      ongoing?(item) && !deadline_passed?(item)
    end

    def deadline_passed?(item)
      return false if item.meta['deadline'].nil?

      item.meta['deadline'] <= Time.zone.now
    end

    def published?(item)
      item.meta.fetch 'published', false
    end

    ## Course Items ##
    def course_items
      @course_items ||= ::Item.for_course(objective.context_id)
        .select {|item| ongoing?(item) }
    end

    def available_course_items
      @available_course_items ||= course_items
        .select {|item| was_available?(item) }
    end

    ## Item Result ##
    def results_for(item_ids)
      ::ItemResult.for_user(user_objective.user_id).for_item(item_ids)
    end

    def visit_percentage_for(visited_items, available_items)
      return 0 if available_items.empty?

      (visited_items.size * 100).fdiv(available_items.size).floor
    end

    ## Visit Calculation ##
    def item_visits
      @item_visits ||= user_results.select(&:visited?)
    end

    ## Points Calculation ##
    def attempted_exercises_results
      @attempted_exercises_results ||= user_results.select(&:points?)
    end

    def by_exercise_type(items, types)
      items.select {|item| types.include?(item.meta['exercise_type']) }
    end

    def scored_points
      @scored_points ||= scored_points_for(
        scored_points_items.select do |item|
          was_available?(item) &&
            attempted_exercises_results.pluck(:item_id).include?(item.id)
        end
      )
    end

    def open_points
      @open_points ||= max_points_for(
        scored_points_items.select do |item|
          upcoming?(item) &&
            attempted_exercises_results.pluck(:item_id).exclude?(item.id)
        end
      )
    end

    def max_points
      @max_points ||= max_points_for(
        max_points_items.select {|item| was_available?(item) }
      )
    end

    def scored_points_for(items)
      item_ids = items.pluck :id
      attempted_exercises_results.select do |result|
        item_ids.include?(result.item_id)
      end.pluck(:user_points).compact.sum
    end

    def max_points_for(items)
      items.pluck(:meta).pluck('max_points').compact.sum
    end

    def points_percentage
      @points_percentage ||= points_percentage_for(scored_points, max_points)
    end

    def points_percentage_for(scored_points, max_points)
      return 0 if max_points.zero?

      (scored_points * 100).fdiv(max_points).floor
    end

    def points_achievable?
      ((scored_points + open_points) * 100)
        .fdiv(max_points)
        .floor >= points_percentage_threshold
    end

    ## Thresholds ##
    def course
      @course ||= Xikolo.api(:course).value!
        .rel(:course)
        .get(id: objective.context_id)
        .value!
    end

    def points_percentage_threshold
      Rails.cache.fetch(
        "points_percentage_threshold/#{objective.context_id}",
        expires_in: 1.day
      ) do
        course.fetch('roa_threshold_percentage')
      end
    end

    def visit_percentage_threshold
      Rails.cache.fetch(
        "visit_percentage_threshold/#{objective.context_id}",
        expires_in: 1.day
      ) do
        course.fetch('cop_threshold_percentage')
      end
    end
  end
end
