# frozen_string_literal: true

module CompletionHandlers
  class XikoloTopicCompletion < Base
    # Operates on items assigned to objective.
    # Considers acquisition items for visits and examination items for points.
    requires_objective_items!

    def check_completion!
      completed! if all_visited? && required_points_scored?
    end

    def progress
      @progress ||= {
        points_scored: scored_points,
        points_possible: max_points,
        # Percentage of achieved points with regard to
        # objective points (examination items)
        points_percentage:,
        items_visited: item_visits.size,
        items_available: available_objective_items.size,
        # Percentage of visited objective items
        visit_percentage:,
      }
    end

    def achievable?
      # Completed or just need to visit some more items
      return true if completed? || required_points_scored?
      return true if max_points.zero?

      # Remaining exercises (main & bonus) are sufficient to reach the
      # threshold (i.e., the user can visit remaining items).
      # Visits are always possible, see XikoloCourseExploration for details
      return true if points_achievable?

      false
    end

    private

    def objective_items
      @objective_items ||= objective.items
        .select {|item| ongoing?(item) }
    end

    def available_objective_items
      @available_objective_items ||= objective_items
        .select {|item| was_available?(item) }
    end

    def user_results
      @user_results ||= results_for(objective_items.pluck(:id))
    end

    ## Visits ##
    def all_visited?
      visit_percentage == 100
    end

    def visit_percentage
      # Currently, we are considering visits for quizzes although the
      # deadline might have passed. This is consistent with the Xikolo
      # course service implementation and the Xikolo CoPs issuing rule.
      @visit_percentage ||= visit_percentage_for(
        item_visits,
        available_objective_items
      )
    end

    ## Points ##
    def required_points_scored?
      points_percentage >= points_percentage_threshold
    end

    def examination_items
      @examination_items ||= objective.knowledge_examination_items
        .select {|item| was_available?(item) && ongoing?(item) }
    end

    def scored_points_items
      @scored_points_items ||= by_exercise_type(
        examination_items,
        %w[main bonus selftest]
      )
    end

    def max_points_items
      # max_points are determined with regard to the maximal points
      # of all exercises which are examination items except bonus.
      @max_points_items ||= by_exercise_type(
        examination_items,
        %w[main selftest]
      )
    end
  end
end
