# frozen_string_literal: true

class Objective < ApplicationRecord
  validates :context_id, :title, presence: {message: 'required'}
  validates :completion_type,
    inclusion: {in: CompletionHandlers.supported_completion_types,
                message: '%{value} is not a valid completion handler'}

  scope :for_context, ->(context) { where context_id: context }

  belongs_to :parent_objective, class_name: 'Objective', optional: true
  has_many :sub_objectives,
    class_name: 'Objective',
    foreign_key: :parent_objective_id
  has_many :learning_units,
    lambda {
      includes(:knowledge_acquisition_items, :knowledge_examination_items)
        .order(priority: :asc)
    },
    dependent: :destroy
  has_many :user_objectives,
    -> { includes(:progress) },
    dependent: :restrict_with_exception

  acts_as_list scope: [:context_id]

  default_scope { order context_id: :asc, position: :asc }

  after_commit :update_user_objectives!, on: :update

  def details_minutes
    sql = 'SELECT d.item_type, ' \
          'SUM(CEIL(d.time_effort::float / 60))::integer, ' \
          'COUNT(c.item_id) FILTER (WHERE d.time_effort > 0), ' \
          'COUNT(c.item_id) FILTER (WHERE d.time_effort = 0 OR d.time_effort IS NULL) ' \
          'FROM (SELECT objective_id, item_id ' \
          'FROM ((SELECT * FROM knowledge_acquisitions UNION SELECT * FROM knowledge_examinations) as a ' \
          'INNER JOIN learning_units b ' \
          'ON a.learning_unit_id=b.id) ' \
          "WHERE objective_id = '#{id}') as c " \
          'INNER JOIN objectives_items d ON c.item_id=d.id ' \
          'GROUP BY d.item_type;'
    records = ActiveRecord::Base.connection.execute(sql).values
    records.map do |record|
      {
        item_type: record[0],
        time_effort: record[1],
        item_count: record[2],
        item_no_time_effort_count: record[3],
      }
    end
  end

  def update_learning_units(units)
    units.each do |unit|
      unit = {
        id: unit['id'],
        knowledge_acquisition_items: unit['knowledge_acquisition_items'].reject(&:empty?),
        knowledge_examination_items: unit['knowledge_examination_items'].reject(&:empty?),
      }

      if unit[:id].blank?
        learning_units << LearningUnit.create(
          knowledge_acquisition_items: Item.where(id: unit[:knowledge_acquisition_items]),
          knowledge_examination_items: Item.where(id: unit[:knowledge_examination_items])
        )
      else
        u = learning_units.find unit[:id]
        u.knowledge_acquisition_items = Item.where(id: unit[:knowledge_acquisition_items])
        u.knowledge_examination_items = Item.where(id: unit[:knowledge_examination_items])
      end
    end
  end

  def items
    Item
      .from("(#{knowledge_examination_items.to_sql} " \
            "UNION ALL #{knowledge_acquisition_items.to_sql}) " \
            'AS items')
      .select('DISTINCT ON (items.id) items.*')
  end

  def knowledge_examination_items
    Item.joins(knowledge_examinations: :learning_unit)
      .where(knowledge_examinations: {learning_units: {objective_id: id}})
  end

  def knowledge_acquisition_items
    Item.joins(knowledge_acquisitions: :learning_unit)
      .where(knowledge_acquisitions: {learning_units: {objective_id: id}})
  end

  def update_user_objectives!
    user_objectives.each(&:check_completion!)
  end
end
