# frozen_string_literal: true

class UserObjective < ApplicationRecord
  include Notify

  validates :user_id, presence: {message: 'required'}
  validates :user_id,
    uniqueness: {scope: :objective_id,
                 message: 'only one per objective and user'}
  validate :only_one_active_objective

  scope :for_user, ->(user) { where user_id: user }
  scope :for_objective, ->(objective) { where objective_id: objective }
  scope :for_context,
    ->(context) { joins(:objective).where(objectives: {context_id: context}) }
  scope :active, -> { where(active: true) }

  belongs_to :objective
  has_one :progress, dependent: :destroy

  default_scope { includes(:progress) }

  after_create { create_progress }
  after_create :check_completion!
  before_update :check_completion!,
    if: proc {|uo| uo.active_changed? from: false, to: true }

  class << self
    def active_for(user:, context:)
      active.for_user(user).for_context(context)
    end
  end

  def check_completion!(timestamp: nil)
    # Since availability of items at course start is an issue,
    # we want to reflect objective item changes by being able
    # to recover the completed state.
    if completion_handler.completed? && objective.final
      completed_at! timestamp || Time.zone.now
    else
      not_completed!
    end

    completion_handler.set_progress!
  end

  def activate_with!(params)
    transaction do
      active_objective = currently_active_objective
      # Deactivate existing (active) objective if the new
      # objective is active (intentionally or by default).
      if active_objective.exists? && params.fetch(:active, true)
        active_objective.first.update!(active: false)
      end
      # Update with provided params (can now safely be activated)
      update! params
    end
  end

  private

  def completed_at!(timestamp)
    return if completed_at?

    update! completed_at: timestamp
  end

  def not_completed!
    return unless completed_at?

    update! completed_at: nil
  end

  def objective_context
    @objective_context ||= objective.context_id
  end

  def currently_active_objective
    self
      .class
      .active_for(user: user_id, context: objective_context)
      .where.not(objective_id:)
  end

  def completion_handler
    @completion_handler ||= CompletionHandlers.resolve(completion_type).new self
  end

  def completion_type
    objective.completion_type
  end

  def only_one_active_objective
    return unless active?
    return unless currently_active_objective.exists?

    errors.add(:active, 'only one active objective per user and context')
  end
end
