# frozen_string_literal: true

require 'rails_helper'

RSpec.describe XikoloItemConsumer, type: :consumer do
  subject(:item_event) do
    publish.call
    Msgr::TestPool.run count: 1
  end

  before do
    # The order of setting the config and then starting Msgr is important as the
    # routes are registered only when the service is available.
    Xikolo.config.learningobjectives = {'enabled' => true}
    Msgr.client.start
  end

  let(:course_item_id) { '00000001-3300-4444-9999-000000000001' }
  let(:section_id_1) { '00000002-3300-4444-9999-000000000001' }
  let(:section_id_2) { '00000002-3300-4444-9999-000000000002' }
  let(:course_id) { '00000003-3300-4444-9999-000000000001' }
  let(:time_effort_1) { 60 }
  let(:time_effort_2) { 40 }
  let(:payload) do
    {
      id: course_item_id,
      section_id: section_id_1,
      course_id:,
      content_type: 'quiz',
      exercise_type: 'selftest',
      max_points: 3.0,
      submission_deadline: 2.days.from_now,
      effective_start_date: 2.days.ago,
      effective_end_date: 5.days.from_now,
      published: true,
      time_effort: time_effort_1,
      updated_at: DateTime.new(2018, 11, 22),
    }
  end
  let(:course_item) { payload }
  let(:publish) { -> { Msgr.publish(payload, to: msgr_route) } }

  describe '#create_or_update' do
    let(:msgr_route) { 'xikolo.course.item.create' }

    before { Stub.service(:course, item_url: 'http://course.xikolo.tld/items/{id}') }

    context 'w/ course item existing' do
      let!(:course_item_stub) do
        Stub.request(:course, :get, "/items/#{course_item_id}")
          .to_return Stub.json course_item
      end

      context 'w/o existing (shadow) item' do
        it 'creates a new Item' do
          expect { item_event }.to change(Item, :count).from(0).to(1)
        end

        it 'sets the item type correctly' do
          item_event
          expect(Item.last.item_type).to eq payload[:content_type]
        end

        it 'sets the time effort correctly' do
          item_event
          expect(Item.last.time_effort).to eq payload[:time_effort]
        end

        it 'sets the meta attributes correctly' do
          item_event
          expect(Item.last.meta).to include \
            'section_id' => payload[:section_id],
            'course_id' => payload[:course_id],
            'exercise_type' => payload[:exercise_type],
            'published' => payload[:published],
            'max_points' => 30
        end

        it 'sets the submission and end date' do
          item_event
          expect(Item.last.meta.keys).to include 'deadline', 'end_date'
        end

        it 'requests the current item' do
          item_event
          expect(course_item_stub).to have_been_requested
        end

        context 'w/o quiz or exercise attributes' do
          let(:payload) do
            super().merge(
              content_type: 'video',
              exercise_type: nil,
              max_points: nil,
              submission_deadline: nil,
              effective_start_date: nil,
              effective_end_date: nil,
              published: nil
            )
          end

          it 'sets the meta attributes correctly' do
            item_event
            expect(Item.last.meta).to eq \
              'section_id' => payload[:section_id],
              'course_id' => payload[:course_id],
              'deadline' => payload[:submission_deadline],
              'start_date' => payload[:effective_start_date],
              'end_date' => payload[:effective_end_date],
              'published' => payload[:published]
          end
        end

        context 'w/ changed course item attributes' do
          # Can happen due to queue length or events processed in different order
          let(:course_item) { super().merge time_effort: time_effort_2, section_id: section_id_2 }

          it 'sets the time effort to the current value of the course item' do
            item_event
            expect(Item.last.time_effort).to eq course_item[:time_effort]
          end

          it 'sets the section id to the current value of the course item' do
            item_event
            expect(Item.last.meta['section_id']).to eq course_item[:section_id]
          end
        end
      end

      context 'w/ existing (shadow) item' do
        let(:item_params) do
          {
            id: course_item_id,
            meta: {section_id: section_id_1, max_points: 3.0},
            time_effort: time_effort_1,
          }
        end
        let(:item) { create(:item, item_params) }

        before { item }

        it 'does not create a new Item' do
          expect { item_event }.not_to change(Item, :count)
        end

        it 'requests the current item' do
          item_event
          expect(course_item_stub).to have_been_requested
        end

        context 'w/ new time effort' do
          let(:payload) { super().merge time_effort: time_effort_2 }

          it 'updates the time effort with the current value from the course item' do
            expect { item_event }.to change { item.reload.time_effort }
              .from(item_params[:time_effort])
              .to(payload[:time_effort])
          end
        end

        context 'w/ new section id' do
          let(:payload) { super().merge section_id: section_id_2 }

          it 'updates the time effort with the current value from the course item' do
            expect { item_event }.to change { item.reload.meta['section_id'] }
              .from(item_params.dig(:meta, :section_id))
              .to(payload[:section_id])
          end
        end

        context '(completion evaluation)' do
          context 'w/ meta attributes of item have changed' do
            let(:payload) { super().merge max_points: 5.0 }

            it 'triggers the completion evaluation' do
              expect(CompletionHandlers).to receive(:check_completion_for!)
                .with(context: payload[:course_id],
                  item: payload[:id],
                  timestamp: payload[:updated_at])
              item_event
            end
          end

          context 'w/o meta attributes of item have changed' do
            before { item_event }

            it 'does not trigger the completion evaluation' do
              expect(CompletionHandlers).not_to receive :check_completion_for!
              item_event
            end
          end
        end
      end
    end

    context 'w/o course item existing (anymore)' do
      before do
        Stub.request(:course, :get, "/items/#{course_item_id}")
          .to_return Stub.response(status: 404)
      end

      it 'does not raise an error' do
        expect { item_event }.not_to raise_error
      end

      it 'destroys the previously created item' do
        item_event
        expect(Item.count).to eq 0
      end

      it 'does not trigger the completion evaluation' do
        expect(CompletionHandlers).not_to receive :check_completion_for!
        item_event
      end
    end

    context 'w/ different (unhandled) error' do
      before do
        Stub.request(:course, :get, "/items/#{course_item_id}")
          .to_return Stub.response(status: 400)
      end

      it 'raises an error' do
        expect { item_event }.to raise_error Restify::ClientError
      end

      it 'does not destroy the previously created item' do
        expect { item_event }.to raise_error Restify::ClientError do
          expect(Item.count).to eq 1
        end
      end
    end
  end

  describe '#destroy' do
    let(:msgr_route) { 'xikolo.course.item.destroy' }
    let(:item) { create(:item, :with_meta) }
    let(:payload) { {id: item.id} }

    before { item }

    it 'destroys the Item' do
      expect { item_event }.to change(Item, :count).from(1).to(0)
    end

    context 'w/ context id available' do
      it 'triggers the completion check' do
        expect(CompletionHandlers).to receive(:check_completion_for!).once.with(context: item.meta['course_id'], item: item.id)
        item_event
      end
    end

    context 'w/o context id available' do
      let(:item) { create(:item) }

      before { item }

      it 'triggers the completion check' do
        expect(CompletionHandlers).not_to receive(:check_completion_for!)
        item_event
      end
    end

    context 'w/o existing (shadow) item' do
      let(:payload) { {id: generate(:item_id)} }

      it 'does not try to remove the shadow item' do
        expect { item_event }.not_to change(Item, :count).from(1)
      end
    end
  end
end
