# frozen_string_literal: true

require 'rails_helper'

RSpec.describe XikoloResultConsumer, type: :consumer do
  subject(:result_event) do
    publish.call
    Msgr::TestPool.run count: 1
  end

  let(:item_id) { '00000001-3300-4444-9999-000000000001' }
  let(:user_id) { '00000001-3100-4444-9999-000000000001' }
  let(:course_id) { '00000003-3300-4444-9999-000000000001' }
  let(:points) { 5.0 }
  let(:item) { create(:item) }
  let(:payload) do
    {
      item_id:,
      user_id:,
      course_id:,
      points:,
      updated_at: DateTime.new(2018, 11, 22),
    }
  end
  let(:publish) { -> { Msgr.publish(payload, to: msgr_route) } }

  before do
    # The order of setting the config and then starting Msgr is important as the
    # routes are registered only when the service is available.
    Xikolo.config.learningobjectives = {'enabled' => true}
    Msgr.client.start
    item
  end

  describe '#create_or_update' do
    let(:msgr_route) { 'xikolo.course.result.create' }

    context 'w/o existing result' do
      it 'creates a new ItemResult' do
        expect { result_event }.to change(ItemResult, :count).from(0).to(1)
      end

      context 'w/o points' do
        let(:points) { nil }

        it 'creates the result with visited status' do
          result_event
          expect(ItemResult.last.attributes).to include \
            'id' => ItemResult.last.id,
            'item_id' => payload[:item_id],
            'user_id' => payload[:user_id],
            'visited' => true,
            'user_points' => nil
        end
      end

      context 'w/ points' do
        it 'creates the result with the points and the visited status' do
          result_event
          expect(ItemResult.last.attributes).to include \
            'id' => ItemResult.last.id,
            'item_id' => payload[:item_id],
            'user_id' => payload[:user_id],
            'visited' => true,
            'user_points' => 50
        end
      end

      it 'triggers the completion evaluation' do
        expect(CompletionHandlers).to receive(:check_completion_for!)
          .with(context: payload[:course_id],
            item: payload[:item_id],
            user: payload[:user_id],
            timestamp: payload[:updated_at])
        result_event
      end
    end

    context 'w/ existing result' do
      let!(:item_result) { create(:item_result, result_params) }
      let(:result_params) do
        {
          item_id: payload[:item_id],
          user_id: payload[:user_id],
          visited: true,
        }
      end

      it 'does not create a new ItemResult' do
        expect { result_event }.not_to change(ItemResult, :count)
      end

      context 'w/ no previous score (only visited yet)' do
        let(:result_params) { super().merge user_points: nil }

        it 'updates the achieved points' do
          expect { result_event }.to change { item_result.reload.user_points }.from(result_params[:user_points]).to(50)
        end

        it 'triggers the completion evaluation' do
          expect(CompletionHandlers).to receive(:check_completion_for!)
            .with(context: payload[:course_id],
              item: payload[:item_id],
              user: payload[:user_id],
              timestamp: payload[:updated_at])
          result_event
        end
      end

      context 'w/ higher score' do
        let(:result_params) { super().merge user_points: 20 }

        it 'updates the achieved points' do
          expect { result_event }.to change { item_result.reload.user_points }.from(result_params[:user_points]).to(50)
        end

        it 'triggers the completion evaluation' do
          expect(CompletionHandlers).to receive(:check_completion_for!)
            .with(context: payload[:course_id],
              item: payload[:item_id],
              user: payload[:user_id],
              timestamp: payload[:updated_at])
          result_event
        end
      end

      context 'w/o higher score' do
        let(:result_params) { super().merge user_points: 60 }

        it 'does not update the achieved points' do
          expect { result_event }.not_to change { item_result.reload.user_points }
        end

        it 'does not trigger the completion calculation' do
          expect(CompletionHandlers).not_to receive :check_completion_for!
          result_event
        end
      end
    end
  end
end
