# frozen_string_literal: true

require 'rails_helper'

RSpec.describe XikoloVisitConsumer, type: :consumer do
  subject(:visit_event) do
    publish.call
    Msgr::TestPool.run count: 1
  end

  before do
    # The order of setting the config and then starting Msgr is important as the
    # routes are registered only when the service is available.
    Xikolo.config.learningobjectives = {'enabled' => true}
    Msgr.client.start
  end

  let(:item_id) { '00000001-3300-4444-9999-000000000001' }
  let(:user_id) { '00000001-3100-4444-9999-000000000001' }
  let(:course_id) { '00000003-3300-4444-9999-000000000001' }
  let(:payload) do
    {
      item_id:,
      user_id:,
      course_id:,
      created_at: DateTime.new(2018, 11, 22),
    }
  end
  let(:publish) { -> { Msgr.publish(payload, to: msgr_route) } }

  describe '#create_or_update' do
    let(:msgr_route) { 'xikolo.course.visit.create' }

    context 'w/o existing result' do
      it 'creates a new ItemResult' do
        expect { visit_event }.to change(ItemResult, :count).from(0).to(1)
      end

      it 'creates the result with the visited status and no points' do
        visit_event
        expect(ItemResult.last.attributes).to include \
          'id' => ItemResult.last.id,
          'item_id' => payload[:item_id],
          'user_id' => payload[:user_id],
          'visited' => true,
          'user_points' => nil
      end

      it 'triggers the completion evaluation' do
        expect(CompletionHandlers).to receive(:check_completion_for!)
          .with(context: payload[:course_id],
            item: payload[:item_id],
            user: payload[:user_id],
            timestamp: payload[:created_at])
        visit_event
      end
    end

    context 'w/ existing result' do
      let(:item_result) { create(:item_result, result_params) }
      let(:result_params) do
        {
          item_id: payload[:item_id],
          user_id: payload[:user_id],
          visited: true,
        }
      end

      before { item_result }

      it 'does not create a new ItemResult' do
        expect { visit_event }.not_to change(ItemResult, :count)
      end

      it 'does not update the ItemResult' do
        expect { visit_event }.not_to change { ItemResult.last.attributes }
      end

      it 'does not trigger the completion calculation' do
        expect(CompletionHandlers).not_to receive :check_completion_for!
        visit_event
      end
    end
  end
end
