# frozen_string_literal: true

require 'rails_helper'

RSpec.describe CompletionHandlers::XikoloCourseCompletion, type: :model do
  let(:user_objective) { create(:user_objective, objective:) }
  let(:objective) { create(:objective, completion_type: 'xikolo_course_completion') }
  let(:acquisition_items) { create_list(:item, 3, meta:) }
  let(:examination_items) { create_list(:item, 2, meta: quiz_meta, item_type: 'quiz') }
  let(:learning_unit) do
    create(:learning_unit,
      objective:,
      knowledge_acquisitions: acquisition_items,
      knowledge_examinations: examination_items)
  end
  let(:quiz_max_points) { 50 }
  let(:meta) do
    {
      course_id: objective.context_id,
      published: true,
    }
  end
  let(:quiz_meta) do
    meta.merge(
      deadline: 1.day.from_now,
      exercise_type: 'main',
      max_points: quiz_max_points,
      published: true
    )
  end
  let(:threshold) { 50 }
  let(:handler) { described_class.new(user_objective) }

  before do
    Stub.service(:course, course_url: 'http://course.xikolo.tld/courses/{id}')
    Stub.request(:course, :get, "/courses/#{objective.context_id}")
      .to_return Stub.json({roa_threshold_percentage: threshold})

    # Create corresponding learning unit for the objective
    learning_unit
  end

  describe '#check_completion!' do
    subject(:check_completion) { handler.check_completion! }

    context 'w/o course items' do
      let(:learning_unit) { create(:learning_unit, objective:) }

      it 'objective is not completed' do
        expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
      end
    end

    context 'w/o item results' do
      it 'objective is not completed' do
        expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
      end
    end

    context 'w/ item results' do
      before do
        create(:item_result, item: examination_items.first, user_points: 20)
        create(:item_result, item: examination_items.second, user_points: second_user_points)
      end

      context 'w/ all items still available' do
        context 'w/o enough points' do
          let(:second_user_points) { 20 }

          it 'objective is not completed' do
            expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
          end
        end

        context 'w/ enough points' do
          let(:second_user_points) { 30 }

          it 'objective is completed' do
            expect { check_completion }.to change { handler.instance_variable_get(:@completed) }.from(nil).to(true)
          end
        end
      end

      context 'w/ quiz item not available anymore' do
        # 5 items, 2 quizzes, 40pts (+ 35pts) of 150pts in total, but item with 35pts is not available anymore
        let(:second_user_points) { 20 }
        let(:additional_item) { create(:item, meta: quiz_meta.merge(end_date: 1.day.ago), item_type: 'quiz') }
        let(:examination_items) { super() << additional_item }

        before { create(:item_result, item: additional_item, user_points: 35) }

        it 'does not count item for the objective completion' do
          expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
        end
      end

      context 'w/ unpublished quiz item' do
        # 5 items, 2 quizzes, 40pts (+ 35pts) of 150pts in total, but item with 35pts is not published
        let(:second_user_points) { 20 }
        let(:additional_item) { create(:item, meta: quiz_meta.merge(published: false), item_type: 'quiz') }
        let(:examination_items) { super() << additional_item }

        before { create(:item_result, item: additional_item, user_points: 35) }

        it 'does not count item for the objective completion' do
          expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
        end
      end

      context 'w/ main quiz not marked as examination items' do
        let(:second_user_points) { 20 }
        let(:additional_item) { create(:item, meta: quiz_meta, item_type: 'quiz') }
        let(:acquisition_items) { super() << additional_item }

        before { create(:item_result, item: additional_item, user_points: 35) }

        it 'counts item for the objective completion' do
          expect { check_completion }.to change { handler.instance_variable_get(:@completed) }.from(nil).to(true)
        end
      end

      context 'w/ item not belonging to objective' do
        let(:second_user_points) { 20 }
        let(:additional_item) { create(:item, meta: quiz_meta, item_type: 'quiz') }

        before { create(:item_result, item: additional_item, user_points: 35) }
        # 3 quiz items exist (2 knowledge acquisition items, 1 course item)
        # max_points: 50 + 50 + 50 = 150
        # user_points 20 + 20 + 35 = 75

        it 'counts item for the objective completion' do
          expect { check_completion }.to change { handler.instance_variable_get(:@completed) }.from(nil).to(true)
        end
      end
    end
  end

  describe '#progress' do
    subject(:progress) { handler.progress }

    before do
      # Five objective items of which one is visited with points
      create(:item_result, :visited, item: examination_items.first, user_points: 20)
      # Additional item not belonging to objective counts
      create(:item, meta: quiz_meta, item_type: 'quiz')
    end

    it 'returns the correct progress' do
      expect(progress).to eq(
        points_scored: 20,
        points_possible: 150,
        points_percentage: 13,
        items_visited: 1,
        items_available: 6,
        visit_percentage: 16
      )
    end
  end

  describe '#achievable?' do
    context 'w/ user objective completed' do
      before { handler.instance_variable_set :@completed, true }

      it 'is achievable' do
        expect(handler).to be_achievable
      end
    end

    context 'w/o user objective completed' do
      before { handler.instance_variable_set :@completed, false }

      context 'w/o any points available' do
        before { allow(handler).to receive(:max_points).with(no_args).and_return 0 }

        it 'is achievable' do
          expect(handler).to be_achievable
        end
      end

      context 'w/ points available' do
        # 100pts maximum, 20pts scored, 50pts possible
        before do
          create(:item_result, item: examination_items.first, user_points: 20)
        end

        context 'w/ enough points left' do
          let(:threshold) { 60 }

          it 'is achievable' do
            expect(handler).to be_achievable
          end
        end

        context 'w/o enough points left' do
          let(:threshold) { 71 }

          it 'is not achievable' do
            expect(handler).not_to be_achievable
          end
        end

        context 'w/ quiz deadline passed' do
          # 150pts maximum, 20pts scored, 50pts possible, because deadline of last quiz passed
          let(:additional_item) { create(:item, meta: quiz_meta.merge(deadline: 1.day.ago), item_type: 'quiz') }
          let(:examination_items) { super() << additional_item }

          context 'w/ enough points left' do
            let(:threshold) { 45 }

            it 'is achievable' do
              expect(handler).to be_achievable
            end
          end

          context 'w/o enough points left' do
            let(:threshold) { 50 }

            it 'is not achievable' do
              expect(handler).not_to be_achievable
            end
          end
        end
      end
    end
  end
end
