# frozen_string_literal: true

require 'rails_helper'

RSpec.describe CompletionHandlers::XikoloCourseExploration, type: :model do
  let(:user_objective) { create(:user_objective, objective:) }
  let(:objective) { create(:objective, completion_type: 'xikolo_course_exploration') }
  let(:acquisition_items) { create_list(:item, 3, meta:) }
  let(:examination_items) { create_list(:item, 2, meta:, item_type: 'quiz') }
  let(:learning_unit) do
    create(:learning_unit,
      objective:,
      knowledge_acquisitions: acquisition_items,
      knowledge_examinations: examination_items)
  end
  let(:meta) do
    {
      course_id: objective.context_id,
      published: true,
    }
  end
  let(:threshold) { 50 }
  let(:handler) { described_class.new user_objective }

  before do
    Stub.service(:course, course_url: 'http://course.xikolo.tld/courses/{id}')
    Stub.request(:course, :get, "/courses/#{objective.context_id}")
      .to_return Stub.json({cop_threshold_percentage: threshold})

    # Create corresponding learning unit for the objective
    learning_unit
  end

  describe '#check_completion!' do
    subject(:check_completion) { handler.check_completion! }

    context 'w/o course items' do
      let(:learning_unit) { create(:learning_unit, objective:) }

      it 'objective is not completed' do
        expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
      end
    end

    context 'w/o item results' do
      it 'objective is not completed' do
        expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
      end
    end

    context 'w/o enough items visited' do
      # 5 items, 2 visited
      before do
        acquisition_items.take(2).each {|item| create(:item_result, :visited, item:) }
      end

      it 'objective is not completed' do
        expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
      end
    end

    context 'w/ enough items visited' do
      # 5 items, 3 visited
      before do
        acquisition_items.each {|item| create(:item_result, :visited, item:) }
      end

      it 'objective is completed' do
        expect { check_completion }.to change { handler.instance_variable_get(:@completed) }.from(nil).to(true)
      end

      context 'w/ unpublished item' do
        # 7 items, 3 visited, but 2 items are unpublished
        let(:additional_items) { create_list(:item, 2, meta: meta.merge(published: false), item_type: 'quiz') }
        let(:examination_items) { super() + additional_items }

        it 'does not consider unpublished items' do
          expect { check_completion }.to change { handler.instance_variable_get(:@completed) }.from(nil).to(true)
        end
      end

      context 'w/ visited items not available anymore' do
        # 7 items, 3 visited, but 2 items are not available anymore
        let(:additional_items) { create_list(:item, 2, meta: meta.merge(end_date: 1.day.ago), item_type: 'quiz') }
        let(:examination_items) { super() + additional_items }

        it 'does not consider the not available items' do
          expect { check_completion }.to change { handler.instance_variable_get(:@completed) }.from(nil).to(true)
        end
      end

      context 'w/ upcoming items' do
        # 7 items, 3 visited, 2 items are not available yet
        let(:additional_items) { create_list(:item, 2, meta: meta.merge(start_date: 1.day.from_now), item_type: 'quiz') }
        let(:examination_items) { super() + additional_items }

        it 'does not consider the not available items' do
          expect { check_completion }.to change { handler.instance_variable_get(:@completed) }.from(nil).to(true)
        end
      end
    end

    context 'w/ items not marked as objective items' do
      let(:additional_items) { create_list(:item, 2, meta:) }

      before do
        (examination_items + additional_items).each {|item| create(:item_result, :visited, item:) }
      end

      it 'counts item for the objective completion' do
        expect { check_completion }.to change { handler.instance_variable_get(:@completed) }.from(nil).to(true)
      end
    end
  end

  describe '#progress' do
    subject(:progress) { handler.progress }

    before do
      # Five objective items of which one is visited with points
      create(:item_result, :visited, item: examination_items.first, user_points: 20)
      # Additional item not belonging to objective counts
      create(:item, meta:, item_type: 'video')
    end

    it 'returns the correct progress' do
      expect(progress).to eq(
        items_visited: 1,
        items_available: 6,
        visit_percentage: 16
      )
    end
  end

  describe '#achievable?' do
    it 'Course exploration is always achievable' do
      expect(handler).to be_achievable
    end
  end
end
