# frozen_string_literal: true

require 'rails_helper'

RSpec.describe CompletionHandlers::XikoloTopicCompletion, type: :model do
  let(:user_objective) { create(:user_objective, objective:) }
  let(:objective) { create(:objective, completion_type: 'xikolo_topic_completion') }
  let(:acquisition_items) { create_list(:item, 3, meta:) }
  let(:examination_items) { create_list(:item, 2, meta: quiz_meta, item_type: 'quiz') }
  let(:learning_unit) do
    create(:learning_unit,
      objective:,
      knowledge_acquisitions: acquisition_items,
      knowledge_examinations: examination_items)
  end
  let(:quiz_points) { 50 }
  let(:meta) do
    {
      course_id: objective.context_id,
      max_points: quiz_points,
      published: true,
    }
  end
  let(:quiz_meta) do
    meta.merge(
      deadline: 1.day.from_now,
      exercise_type: 'main',
      max_points: quiz_points
    )
  end
  let(:visit_threshold)  { 50 }
  let(:points_threshold) { 50 }
  let(:handler) { described_class.new user_objective }

  before do
    Stub.service(:course, course_url: 'http://course.xikolo.tld/courses/{id}')
    Stub.request(:course, :get, "/courses/#{objective.context_id}")
      .to_return Stub.json({
        cop_threshold_percentage: visit_threshold,
        roa_threshold_percentage: points_threshold,
      })

    # Create corresponding learning unit for the objective
    learning_unit
  end

  describe '#check_completion!' do
    subject(:check_completion) { handler.check_completion! }

    context 'w/o course items' do
      let(:learning_unit) { create(:learning_unit, objective:) }

      it 'objective is not completed' do
        expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
      end
    end

    context 'w/o item results' do
      it 'objective is not completed' do
        expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
      end
    end

    context 'w/o all items visited' do
      before do
        acquisition_items.each {|item| create(:item_result, :visited, item:) }
      end

      it 'objective is not completed' do
        expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
      end
    end

    context 'w/ all items visited' do
      let(:user_points) { 50 }

      before do
        acquisition_items.each {|item| create(:item_result, :visited, item:) }
        create(:item_result, :visited, item: examination_items.first, user_points: 20)
        create(:item_result, :visited, item: examination_items.second, user_points:)
      end

      context 'w/ enough points' do
        it 'objective is completed' do
          expect { check_completion }.to change { handler.instance_variable_get(:@completed) }.from(nil).to(true)
        end
      end

      context 'w/o enough points' do
        let(:user_points) { 20 }

        it 'objective is not completed' do
          expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
        end
      end

      context 'w/ quiz item not available anymore' do
        # 5 items, 2 quizzes, 40pts (+ 35pts) of 150pts in total, but item with 35pts is not available anymore
        let(:user_points) { 20 }
        let(:additional_item) do
          create(:item, meta: quiz_meta.merge(end_date: 1.day.ago), item_type: 'quiz')
        end
        let(:examination_items) { super() << additional_item }

        before { create(:item_result, :visited, item: additional_item, user_points: 35) }

        it 'does not count item for the objective completion' do
          expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
        end
      end

      context 'w/ unpublished quiz item' do
        # 5 items, 2 quizzes, 40pts (+ 35pts) of 150pts in total, but item with 35pts is not published
        let(:user_points) { 20 }
        let(:additional_item) { create(:item, meta: quiz_meta.merge(published: false), item_type: 'quiz') }
        let(:examination_items) { super() << additional_item }

        before { create(:item_result, item: additional_item, user_points: 35) }

        it 'does not count item for the objective completion' do
          expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
        end
      end

      context 'w/ main quiz not marked as examination items' do
        let(:user_points) { 20 }
        let(:additional_item) { create(:item, meta: quiz_meta, item_type: 'quiz') }
        let(:acquisition_items) { super() << additional_item }

        before { ItemResult.find_by(item_id: additional_item.id).update! user_points: 35 }

        it 'does not count for the point completion' do
          expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
        end
      end

      context 'w/ item not belonging to objective' do
        let(:user_points) { 20 }
        let(:additional_item) { create(:item, meta: quiz_meta, item_type: 'quiz') }

        before { create(:item_result, :visited, item: additional_item, user_points: 35) }

        it 'does not count item for the objective completion' do
          expect { check_completion }.not_to change { handler.instance_variable_get(:@completed) }
        end
      end
    end
  end

  describe '#progress' do
    subject(:progress) { handler.progress }

    before do
      # Five objective items of which one is visited with points
      create(:item_result, :visited, item: examination_items.first, user_points: 20)
      # Additional item not belonging to objective does not count
      create(:item, meta: quiz_meta, item_type: 'quiz')
    end

    it 'returns the correct progress' do
      expect(progress).to eq(
        points_scored: 20,
        points_possible: 100,
        points_percentage: 20,
        items_visited: 1,
        items_available: 5,
        visit_percentage: 20
      )
    end
  end

  describe '#achievable?' do
    context 'w/ user objective completed' do
      before { handler.instance_variable_set :@completed, true }

      it 'is achievable' do
        expect(handler).to be_achievable
      end
    end

    context 'w/o user objective completed' do
      before { handler.instance_variable_set :@completed, false }

      context 'w/ required points achieved' do
        before { allow(handler).to receive(:required_points_scored?).with(no_args).and_return true }

        it 'is achievable' do
          expect(handler).to be_achievable
        end
      end

      context 'w/o required points achieved' do
        before { allow(handler).to receive(:required_points_scored?).with(no_args).and_return false }

        context 'w/o any points available' do
          before { allow(handler).to receive(:max_points).with(no_args).and_return 0 }

          it 'is achievable' do
            expect(handler).to be_achievable
          end
        end

        context 'w/ points available' do
          # 100pts maximum, 20pts scored, 50pts possible
          before do
            create(:item_result, item: examination_items.first, user_points: 20)
          end

          context 'w/ enough points left' do
            let(:points_threshold) { 60 }

            it 'is achievable' do
              expect(handler).to be_achievable
            end
          end

          context 'w/o enough points left' do
            let(:points_threshold) { 71 }

            it 'is not achievable' do
              expect(handler).not_to be_achievable
            end
          end

          context 'w/ quiz deadline passed' do
            # 150pts maximum, 20pts scored, 50pts possible, because deadline of last quiz passed
            let(:additional_item) do
              create(:item, meta: quiz_meta.merge(deadline: 1.day.ago), item_type: 'quiz')
            end
            let(:examination_items) { super() << additional_item }

            context 'w/ enough points left' do
              let(:points_threshold) { 45 }

              it 'is achievable' do
                expect(handler).to be_achievable
              end
            end

            context 'w/o enough points left' do
              let(:points_threshold) { 50 }

              it 'is not achievable' do
                expect(handler).not_to be_achievable
              end
            end
          end
        end
      end
    end
  end
end
