# frozen_string_literal: true

require 'rails_helper'

RSpec.describe CompletionHandlers, type: :model do
  describe '#all' do
    subject { described_class.all }

    it { is_expected.to match_array %w[XikoloCourseCompletion XikoloTopicCompletion XikoloCourseExploration] }
  end

  describe '#supported_completion_types' do
    subject { described_class.supported_completion_types }

    it { is_expected.to match_array %w[xikolo_course_completion xikolo_topic_completion xikolo_course_exploration] }
  end

  describe '#handler_objective_items_status' do
    subject { described_class.handler_objective_items_status }

    let(:type_mapping) do
      {
        xikolo_course_completion: false,
        xikolo_topic_completion: true,
        xikolo_course_exploration: false,
      }
    end

    it { is_expected.to eq(type_mapping.stringify_keys) }
  end

  describe '#check_completion_for!' do
    subject(:check_completion) do
      described_class.check_completion_for! context: context_id,
        item: item_id,
        user: user_id_1,
        timestamp: Time.zone.now
    end

    let(:context_id) { '00000002-3100-4444-9999-000000000001' }
    let(:item_id) { '00000001-3300-4444-9999-000000000001' }
    let(:user_id_1) { '00000001-3100-4444-9999-000000000001' }
    let(:user_id_2) { '00000001-3100-4444-9999-000000000002' }
    let(:objective_1) { create(:objective, context_id:) }
    let(:objective_2) { create(:objective, context_id:) }
    let(:user_objective_1) { create(:user_objective, objective: objective_1, user_id: user_id_1) }
    let(:user_objective_2) { create(:user_objective, objective: objective_1, user_id: user_id_2) }

    before do
      objective_1
      objective_2
      user_objective_1
      user_objective_2

      handler = instance_double(CompletionHandlers::XikoloCourseCompletion)
      allow(CompletionHandlers::XikoloCourseCompletion).to receive(:new).and_return(handler)
      allow(handler).to receive(:completed?).once.and_return(false)
      allow(handler).to receive(:set_progress!).once
    end

    it 'requests the affected objectives' do
      expect(described_class).to receive(:affected_objectives).once
        .with(context_id, item_id)
        .and_return [objective_1, objective_2]
      check_completion
    end
  end

  describe '#affected_objectives' do
    subject(:affected_objectives) { described_class.affected_objectives context_id, item_id }

    let(:context_1) { '00000002-3100-4444-9999-000000000001' }
    let(:context_2) { '00000002-3100-4444-9999-000000000002' }
    let(:objective_1) { create(:objective, context_id: context_1) }
    let(:objective_2) { create(:objective, context_id: context_2, completion_type: 'xikolo_course_completion') }
    let(:objective_w_item) { create(:objective, context_id: context_2, completion_type: 'xikolo_topic_completion') }
    let(:objective_wo_item) { create(:objective, context_id: context_2, completion_type: 'xikolo_topic_completion') }
    let(:learning_unit_1) { create(:learning_unit, objective: objective_w_item, knowledge_acquisitions: [item]) }
    let(:item) { create(:item) }

    before do
      objective_1
      objective_2
      objective_w_item
      objective_wo_item
      learning_unit_1
      item
    end

    context 'w/ context' do
      let(:context_id) { context_1 }
      let(:item_id) { nil }

      it 'returns the objectives for the given context' do
        expect(affected_objectives.pluck(:id)).to contain_exactly(objective_1.id)
      end

      context 'w/ item' do
        let(:context_id) { context_2 }
        let(:item_id) { item.id }

        it 'returns the objectives for the given context' do
          expect(affected_objectives.pluck(:id)).to contain_exactly(objective_2.id, objective_w_item.id)
        end
      end
    end
  end
end
