# frozen_string_literal: true

require 'rails_helper'

RSpec.describe Objective, type: :model do
  let(:acquisition_items) { create_list(:item, 4, time_effort: 40) }
  let(:examination_items) { create_list(:item, 2, time_effort: 40, item_type: 'quiz') }
  let(:objective) { create(:objective) }
  let(:learning_unit) do
    create(:learning_unit,
      objective:,
      knowledge_acquisitions: acquisition_items,
      knowledge_examinations: examination_items)
  end
  let(:expected_video_details) do
    {
      item_type: 'video',
      time_effort: 4,
      item_count: 4,
      item_no_time_effort_count: 0,
    }
  end
  let(:expected_quiz_details) do
    {
      item_type: 'quiz',
      time_effort: 2,
      item_count: 2,
      item_no_time_effort_count: 0,
    }
  end

  # Create corresponding learning unit for the objective
  before { learning_unit }

  describe '(validation)' do
    it do
      expect(objective).to accept_values_for(
        :completion_type,
        'xikolo_course_completion',
        'xikolo_course_exploration',
        'xikolo_topic_completion'
      )
    end

    it { is_expected.not_to accept_values_for(:completion_type, 'base', 'xikolo_points_completion') }
  end

  describe '(after_commit)' do
    let(:objective) { create(:objective) }

    it 'does update user objectives when objective is updated' do
      expect(objective).to receive(:update_user_objectives!).once
      objective.save
    end
  end

  describe '#details_minutes' do
    subject(:details) { objective.details_minutes }

    context 'w/ time effort available for all items' do
      it 'returns the expected objective details' do
        expect(details).to contain_exactly(expected_video_details, expected_quiz_details)
      end
    end

    context 'w/o time effort available for all items' do
      let(:acquisition_items) { super().push create :item }
      let(:examination_items) { super().push create :item, item_type: 'rich_text' }
      let(:expected_video_details) do
        {
          item_type: 'video',
          time_effort: 4,
          item_count: 4,
          item_no_time_effort_count: 1,
        }
      end
      let(:expected_rich_text_details) do
        {
          item_type: 'rich_text',
          time_effort: nil,
          item_count: 0,
          item_no_time_effort_count: 1,
        }
      end

      it 'returns the expected objective details' do
        expect(details).to contain_exactly(expected_video_details, expected_quiz_details, expected_rich_text_details)
      end
    end

    context 'w/o learning unit for the objective' do
      let(:learning_unit) { create(:learning_unit) }

      it 'returns empty results' do
        expect(details).to eq []
      end
    end
  end

  describe '#update_learning_units' do
    context 'w/o learning unit existing for objective' do
      subject(:new_objective_updates) { new_objective.update_learning_units learning_units }

      let(:new_objective) { create(:objective) }
      let(:learning_units) do
        [{
          'id' => nil,
          'knowledge_acquisition_items' => acquisition_items.pluck(:id),
          'knowledge_examination_items' => examination_items.pluck(:id),
        }]
      end

      it 'creates a LearningUnit' do
        expect { new_objective_updates }.to change(LearningUnit, :count).by(1)
      end

      it 'creates a learning unit for the objective' do
        expect(new_objective.learning_units).to eq []

        new_objective_updates

        expect(new_objective.reload.learning_units.first.knowledge_acquisition_items.pluck(:id))
          .to match_array acquisition_items.pluck(:id)
        expect(new_objective.reload.learning_units.first.knowledge_examination_items.pluck(:id))
          .to match_array examination_items.pluck(:id)
      end
    end

    context 'w/ learning unit existing for objective' do
      subject(:old_objective_updates) { objective.update_learning_units learning_units }

      let(:new_item) { create(:item) }
      let(:new_acquisition_items) { objective.learning_units.first.knowledge_acquisition_items + [new_item] }
      let(:learning_units) do
        [{
          'id' => objective.learning_units.first.id,
          'knowledge_acquisition_items' => new_acquisition_items.pluck(:id),
          'knowledge_examination_items' => [],
        }]
      end

      it 'creates a LearningUnit' do
        expect { old_objective_updates }.not_to change(LearningUnit, :count)
      end

      it 'updates the learning unit for the objective' do
        expect(objective.learning_units.first.knowledge_acquisition_items.pluck(:id))
          .to match_array acquisition_items.pluck(:id)
        expect(objective.learning_units.first.knowledge_examination_items.pluck(:id))
          .to match_array examination_items.pluck(:id)

        old_objective_updates

        expect(objective.learning_units.first.knowledge_acquisition_items.pluck(:id))
          .to match_array new_acquisition_items.pluck(:id)
        expect(objective.learning_units.first.knowledge_examination_items.pluck(:id))
          .to be_empty
      end
    end
  end

  describe '#items' do
    it 'returns all objective items' do
      expect(objective.items).to match_array(acquisition_items + examination_items)
    end
  end

  describe '#knowledge_examination_items' do
    it 'returns all objective knowledge examination items' do
      expect(objective.knowledge_examination_items).to match_array examination_items
    end
  end

  describe '#knowledge_acquisition_items' do
    it 'returns all objective knowledge acquisition items' do
      expect(objective.knowledge_acquisition_items).to match_array acquisition_items
    end
  end
end
