# frozen_string_literal: true

require 'rails_helper'

RSpec.describe UserObjective, type: :model do
  let(:context_id) { '00000002-3100-4444-9999-000000000001' }
  let(:objective) { create(:objective, context_id:, final:) }
  let(:final) { false }

  # Ensure the user objective exists
  before { user_objective }

  describe '(after create)' do
    subject(:create_user_objective) { user_objective.save! }

    let(:user_objective) { build(:user_objective, objective:) }

    before { expect(user_objective).to receive(:check_completion!).once.with no_args }

    it 'creates a progress' do
      expect { create_user_objective }.to change(Progress, :count).from(0).to(1)
    end

    it 'creates a new progress for the newly created user objective' do
      create_user_objective
      expect(Progress.last.user_objective).to eq user_objective
    end
  end

  describe '(before update)' do
    subject(:update_active) { user_objective.update! active: new_active_status }

    let(:user_objective) { create(:user_objective, objective:) }
    let(:new_active_status) { false }

    it 'does not trigger the completion check for the user objective becoming inactive' do
      expect(user_objective).not_to receive(:check_completion!)
      update_active
    end

    context 'w/ inactive user objective' do
      let(:user_objective) { build(:user_objective, :inactive, objective:) }
      let(:new_active_status) { true }

      it 'does trigger the completion check for the objective becoming active' do
        expect(user_objective).to receive(:check_completion!).once.with no_args
        update_active
      end
    end
  end

  describe '(validation)' do
    subject { new_user_objective }

    let(:user_objective) { create(:user_objective, objective:) }
    let(:new_user_objective) { build(:user_objective, objective:) }

    context 'w/ same objective' do
      it { is_expected.not_to be_valid }
    end

    context 'w/o objective in different context' do
      let(:objective_same_context) { create(:objective, context_id:) }
      let(:new_user_objective) { build(:user_objective, objective: objective_same_context) }

      it { is_expected.not_to be_valid }
    end

    context 'w/ objective in different context' do
      let(:another_context_id) { '00000002-3100-5555-8888-000000000002' }
      let(:objective_different_context) { create(:objective, context_id: another_context_id) }
      let(:new_user_objective) { build(:user_objective, objective: objective_different_context) }

      it { is_expected.to be_valid }
    end
  end

  describe '#check_completion!' do
    subject(:check_completion) { user_objective.check_completion! timestamp: }

    let(:user_objective) { create(:user_objective, objective:) }
    let(:completion_handler) { instance_double(CompletionHandlers::Base, completed?: completed, set_progress!: true) }
    let(:completed) { false }
    let(:timestamp) { DateTime.new(2018, 10, 31) }

    before do
      expect(user_objective).to receive(:completion_handler).twice.and_return completion_handler
    end

    context 'w/o completed user objective' do
      context 'w/o completion handler returns completed' do
        it 'does not set the completed_at attribute' do
          expect { check_completion }.not_to change(user_objective, :completed_at)
        end
      end

      context 'w/ completion handler returns completed' do
        let(:completed) { true }

        context 'w/o objective is final' do
          it 'does not set the completed_at attribute' do
            expect { check_completion }.not_to change(user_objective, :completed_at)
          end
        end

        context 'w/ objective is final' do
          let(:final) { true }

          it 'sets the completed_at attribute correctly' do
            expect { check_completion }.to change(user_objective, :completed_at).from(nil).to(timestamp)
          end
        end
      end
    end

    context 'w/ completed user objective' do
      let(:user_objective) { create(:user_objective, :completed, objective:) }

      context 'w/o completion handler returns completed' do
        it 'removes the completion date (== status)' do
          expect { check_completion }.to change(user_objective, :completed_at).to nil
        end
      end

      context 'w/ completion handler returns completed and objective is final' do
        let(:completed) { true }
        let(:final) { true }

        it 'does not overwrite the completed_at attribute' do
          expect { check_completion }.not_to change(user_objective, :completed_at)
        end
      end
    end

    it 'sets the progress in any case' do
      expect(completion_handler).to receive(:set_progress!).once
      check_completion
    end

    context 'w/o timestamp provided' do
      subject(:check_completion) { user_objective.check_completion! }

      let(:completed) { true }
      let(:final) { true }

      before { Timecop.freeze DateTime.new(2018, 12, 24) }

      after  { Timecop.return }

      it 'uses the current time as completion time' do
        expect { check_completion }.to change(user_objective, :completed_at).from(nil).to(Time.zone.now)
      end
    end
  end
end
