# frozen_string_literal: true

require 'rails_helper'

RSpec.describe 'Objective: Create', type: :request do
  subject(:create_objective) { api.rel(:objectives).post(attrs).value! }

  let(:api) { Restify.new(:test).get.value! }
  let(:context_id) { SecureRandom.uuid }
  let(:attrs) do
    {
      context_id:,
      title: 'Any Objective',
      description: 'Awesome description encouraging learners to choose this objective.',
      completion_type: 'xikolo_course_completion',
      final: true,
    }
  end

  it 'creates new objective record' do
    expect { create_objective }.to change(Objective, :count).from(0).to(1)
  end

  it 'responds with a follow location' do
    expect(create_objective.follow).to eq objective_url Objective.last
  end

  it 'returns objective resource' do
    expect(create_objective.to_h).to include \
      'id' => Objective.last.id,
      'context_id' => attrs[:context_id],
      'title' => attrs[:title],
      'description' => attrs[:description],
      'completion_type' => attrs[:completion_type],
      'final' => attrs[:final],
      'url' => objective_url(Objective.last)
  end

  context 'w/ invalid completion type' do
    let(:attrs) { super().merge completion_type: 'invalid' }

    it 'returns 422 Unprocessable Entity' do
      expect { create_objective }.to raise_error(Restify::ClientError) do |error|
        expect(error.status).to eq :unprocessable_entity
      end
    end
  end

  context 'w/ learning units param' do
    let!(:item_1) { create(:item) }
    let!(:item_2) { create(:item) }
    let(:attrs) do
      super().merge(
        'learning_units' => [{
          'id' => nil,
          'knowledge_acquisition_items' => [item_1.id],
          'knowledge_examination_items' => [item_2.id],
        }]
      )
    end

    it 'creates the learning units' do
      expect do
        create_objective
      end.to change { item_1.knowledge_acquisition_units.count }.from(0).to(1)
        .and change { item_2.knowledge_examination_units.count }.from(0).to(1)
        .and change(LearningUnit, :count).from(0).to(1)
    end
  end
end
