# frozen_string_literal: true

require 'rails_helper'

RSpec.describe 'UserObjective: Create', type: :request do
  subject(:create_user_objective) { api.rel(:user_objectives).post(attrs).value! }

  let(:api) { Restify.new(:test).get.value! }
  let!(:objective) { create(:objective) }
  let(:user_id) { SecureRandom.uuid }
  let(:attrs) do
    {
      objective_id: objective.id,
      user_id:,
    }
  end

  context 'user_id missing' do
    let(:attrs) { {objective_id: objective.id} }

    it 'responds with 422 Unprocessable Entity' do
      expect { create_user_objective }.to raise_error(Restify::UnprocessableEntity) do |error|
        expect(error.status).to eq :unprocessable_entity
        expect(error.errors).to eq 'user_id' => ['required']
      end
    end
  end

  context 'objective_id missing' do
    let(:attrs) { {user_id:} }

    it 'responds with 422 Unprocessable Entity' do
      expect { create_user_objective }.to raise_error(Restify::UnprocessableEntity) do |error|
        expect(error.status).to eq :unprocessable_entity
        expect(error.errors).to eq 'objective_id' => ['required']
      end
    end
  end

  context 'w/o existing user objective' do
    before do
      handler = instance_double(CompletionHandlers::XikoloCourseCompletion)
      allow(CompletionHandlers::XikoloCourseCompletion).to receive(:new).and_return(handler)
      allow(handler).to receive(:completed?).once.and_return(false)
      expect(handler).to receive(:set_progress!).once
    end

    it 'creates new user objective record' do
      expect { create_user_objective }.to change(UserObjective, :count).from(0).to(1)
    end

    it 'responds with a follow location' do
      expect(create_user_objective.follow).to eq user_objective_url UserObjective.last
    end

    it 'returns user objective resource' do
      expect(create_user_objective.to_h).to include \
        'id' => UserObjective.last.id,
        'objective_id' => attrs[:objective_id],
        'user_id' => attrs[:user_id],
        'active' => true, # should be active by default
        'created_at' => UserObjective.last.created_at.iso8601,
        'completed_at' => nil,
        'url' => user_objective_url(UserObjective.last)
    end

    context 'w/ active parameter' do
      let(:attrs) { super().merge(active: false) }

      it 'active can be overwritten' do
        expect(create_user_objective.to_h).to include 'active' => false
      end
    end
  end

  context 'w/ existing user objective' do
    let(:objective_attrs) do
      {
        objective_id: objective.id,
        user_id:,
      }
    end
    let(:user_objective) { UserObjective.create! objective_attrs }

    before do
      handler = instance_double(CompletionHandlers::XikoloCourseCompletion)
      allow(CompletionHandlers::XikoloCourseCompletion).to receive(:new).and_return(handler)
      allow(handler).to receive(:completed?).and_return(false)
      allow(handler).to receive(:set_progress!)

      user_objective
    end

    it 'does not create a new user objective' do
      expect { create_user_objective }.not_to change(UserObjective, :count)
    end

    it 'returns the user objective resource' do
      expect(create_user_objective.to_h).to include \
        'id' => user_objective.id,
        'objective_id' => attrs[:objective_id],
        'user_id' => attrs[:user_id],
        'active' => true, # should be active by default
        'created_at' => user_objective.created_at.iso8601,
        'completed_at' => nil,
        'url' => user_objective_url(user_objective)
    end

    context 'with deviating parameters' do
      let(:attrs) { super().merge(active: false) }

      it 'updates the user objective\'s active status' do
        expect { create_user_objective }.to change { user_objective.reload.active }.from(true).to(false)
      end
    end

    context 'with existing other user objective for the context' do
      let!(:another_objective) { create(:objective) }
      let(:attrs) { super().merge(objective_id: another_objective.id) }

      shared_examples 'activates the respective objective' do
        it 'deactivates the old user objective' do
          expect { create_user_objective }.to change { user_objective.reload.active }.from(true).to(false)
        end

        it 'activates the new user objective' do
          expect(create_user_objective.to_h).to include 'active' => true
        end
      end

      it 'creates the respective user objective' do
        expect { create_user_objective }.to change(UserObjective, :count).from(1).to(2)
      end

      include_examples 'activates the respective objective'

      context 'with provided active parameter for the new objective' do
        # Only active: true makes sense here
        let(:attrs) { super().merge(objective_id: another_objective.id, active: true) }

        include_examples 'activates the respective objective'
      end
    end
  end
end
