# frozen_string_literal: true

require 'rails_helper'

RSpec.describe 'UserStatus: Create', type: :request do
  subject(:create_user_status) { api.rel(:user_statuses).post(attrs).value! }

  let(:api) { Restify.new(:test).get.value! }
  let(:user_id) { SecureRandom.uuid }
  let(:context_id) { SecureRandom.uuid }
  let(:attrs) do
    {
      user_id:,
      context_id:,
      settings: {some_setting: 'true'},
    }
  end

  context 'user_id missing' do
    let(:attrs) { {context_id:} }

    it 'responds with 422 Unprocessable Entity' do
      expect { create_user_status }.to raise_error(Restify::UnprocessableEntity) do |error|
        expect(error.status).to eq :unprocessable_entity
        expect(error.errors).to eq 'user_id' => ['required']
      end
    end
  end

  context 'context_id missing' do
    let(:attrs) { {user_id:} }

    it 'responds with 422 Unprocessable Entity' do
      expect { create_user_status }.to raise_error(Restify::UnprocessableEntity) do |error|
        expect(error.status).to eq :unprocessable_entity
        expect(error.errors).to eq 'context_id' => ['required']
      end
    end
  end

  it 'creates new user status record' do
    expect { create_user_status }.to change(UserStatus, :count).from(0).to(1)
  end

  it 'responds with a follow location' do
    expect(create_user_status.follow).to eq user_status_url UserStatus.last
  end

  it 'returns user status resource' do
    expect(create_user_status.to_h).to include \
      'id' => UserStatus.last.id,
      'user_id' => attrs[:user_id],
      'context_id' => attrs[:context_id],
      'settings' => UserStatus.last.settings,
      'url' => user_status_url(UserStatus.last)
  end

  context 'with existing user status' do
    let(:user_status) { UserStatus.create! attrs }

    before { user_status }

    it 'does not create a new user status' do
      expect { create_user_status }.not_to change(UserStatus, :count)
    end

    it 'returns the user status resource' do
      expect(create_user_status.to_h).to include \
        'id' => user_status.id,
        'user_id' => attrs[:user_id],
        'context_id' => attrs[:context_id],
        'settings' => user_status.settings,
        'url' => user_status_url(user_status)
    end

    context 'with different parameters' do
      let(:user_status) { UserStatus.create! attrs.merge(settings: {some_setting: 'false'}) }

      it 'updates the user status\' settings' do
        expect { create_user_status }.to change { user_status.reload.settings['some_setting'] }.from('false').to('true')
      end
    end
  end
end
