# frozen_string_literal: true

require 'rails_helper'

RSpec.describe Processors::VideoProcessor, type: :model do
  let(:content_type) { 'video' }
  let(:item) { create(:item, content_type:) }
  let(:processor) { described_class.new item }

  describe '#initialize' do
    it 'initializes the video correctly' do
      expect(processor.video).to be_nil
    end
  end

  describe '#load_resources!' do
    subject(:load_resources) { processor.load_resources! }

    let(:video_stub) do
      Stub.request(:video, :get, "/videos/#{item.content_id}")
    end

    before do
      Stub.service(:video, video_url: 'http://video.xikolo.tld/videos/{id}')
      video_stub
    end

    context 'w/ valid content type' do
      it 'requests the video' do
        load_resources
        expect(video_stub).to have_been_requested
      end
    end

    context 'w/o valid content type' do
      let(:content_type) { 'quiz' }

      it 'raises an error' do
        expect { load_resources }.to raise_error Errors::InvalidItemType
        expect(video_stub).not_to have_been_requested
      end
    end

    context 'w/ error while loading video' do
      let(:video_stub) do
        Stub.request(:video, :get, "/videos/#{item.content_id}")
          .to_return Stub.response(status: 404)
      end

      it 'raises an error' do
        expect { load_resources }.to raise_error Errors::LoadResourcesError
        expect(video_stub).to have_been_requested
      end
    end
  end

  describe '#calculate' do
    subject(:calculate_time_effort) { processor.calculate }

    let(:video) { {'duration' => duration} }
    let(:duration) { 120 }

    context 'w/ video resource' do
      before do
        processor.instance_variable_set(:@video, video)
      end

      it 'sets the time effort correctly' do
        calculate_time_effort
        expect(processor.time_effort).to eq duration
      end
    end

    context 'w/o video resource' do
      it 'does not set time effort' do
        expect { calculate_time_effort }.not_to change(processor, :time_effort)
      end
    end
  end
end
