# frozen_string_literal: true

class SubtitlesController < ApplicationController
  responders Responders::DecorateResponder,
    Responders::HttpCacheResponder,
    Responders::PaginateResponder

  respond_to :json
  respond_to :vtt, only: :show

  before_action :json_fallback

  def index
    subtitles = Subtitle.all
    subtitles.where! video_id: params[:video_id] if params[:video_id]
    subtitles.where! lang: params[:lang] if params[:lang]

    respond_with subtitles
  end

  def show
    respond_with Subtitle.find params[:id]
  end

  def create
    raise ArgumentError.new 'WebVTT file cannot be blank' if params[:vtt].blank?

    video = Video.find params[:video_id]
    automatic = params.fetch(:automatic, false)
    Subtitle.attach!(params[:vtt], params[:lang], video, automatic:)

    respond_with video.subtitles.find_by(lang: params[:lang])
  rescue ArgumentError, WebVTT::MalformedFile
    error(:bad_request)
  rescue ActiveRecord::RecordInvalid
    error(:unprocessable_entity)
  rescue InvalidSubtitleError => e
    error(:unprocessable_entity,
      json: {
        vtt: {message: e.message, identifiers: e.identifiers},
      })
  end

  def destroy
    respond_with Subtitle.find(params[:id]).destroy
  end

  private

  def subtitle_params
    params.permit :video_id, :lang, :automatic, :vtt
  end

  # Make sure we always fall back to JSON as default content type (in case
  # no Accept header is present and Rails chooses HTML).
  # This is our replacement for the default :json format as specified in
  # config/routes.rb for other controllers, as this would mean Accept
  # headers are ignored.
  def json_fallback
    request.format = :json if request.format == Mime[:html]
  end
end
