# frozen_string_literal: true

class Provider < ApplicationRecord
  ADAPTER = {
    'kaltura' => KalturaAdapter,
    'vimeo' => VimeoAdapter,
  }.freeze

  validates :name, presence: true
  validates :name, uniqueness: true
  validate :uniqueness_of_default, if: -> { Provider.method_defined? :default }

  def uniqueness_of_default
    # make sure that this is the only default provider
    return unless default &&
                  Provider.where(default: true).where.not(id:).exists?

    errors.add(:default, 'default Provider already exists')
  end

  delegate :sync_single, to: :adapter
  delegate :remove_subtitles!, to: :adapter
  delegate :attach_subtitle!, to: :adapter
  delegate :safe_metadata, to: :adapter
  delegate :downloads_expire?, to: :adapter
  delegate :get_download_links, to: :adapter

  def sync_status
    if run_at != synchronized_at
      :running
    elsif synchronized_at.to_time.to_i == 0
      :never_run
    else
      :completed
    end
  end

  def sync_status_date
    case sync_status
      when :running
        run_at
      when :completed
        synchronized_at
      when :never_run
        nil
    end
  end

  # Fetch information about all videos of the user by querying `/me/videos`
  # See API documentation for more information: https://developer.vimeo.com/api/endpoints/me#/videos
  def sync(full: false)
    with_lock('FOR UPDATE NOWAIT') do
      update! run_at: (start = Time.current)
      adapter.sync(since: synchronized_at, full:)

      # Update time with start time to avoid potential missing
      # changes while having processed pages.
      update! synchronized_at: start
    end

    true
  rescue ActiveRecord::LockWaitTimeout
    # For partial syncs, we silently abort when another sync is still running.
    return false unless full

    raise SyncAlreadyRunning.new "Another sync for provider <#{name}> is still running"
  end

  private

  def adapter
    @adapter ||= ADAPTER[provider_type].new self
  end

  class SyncAlreadyRunning < RuntimeError; end
end
