# frozen_string_literal: true

class Stream < ApplicationRecord
  belongs_to :provider
  validates :provider_video_id,
    presence: true,
    uniqueness: {scope: :provider_id}
  validates :title,
    presence: true,
    allow_blank: false
  validate :url?

  default_scope { order('created_at DESC') }

  after_commit :update_video_data, on: %i[create update]

  class << self
    def by_prefix(str)
      where 'title ILIKE ?', "#{Stream.sanitize_sql_like(str)}%"
    end
  end

  # handle re-upload of videos on external video provider
  def update_video_data
    return unless saved_change_to_sd_md5? ||
                  saved_change_to_hd_md5? ||
                  saved_change_to_hls_md5?

    # update audio extraction if previously created and now outdated:
    if saved_change_to_sd_md5? && audio_uri?
      ExtractAudioWorker.perform_async(id)
    end

    Video.where(pip_stream_id: id).find_each do |video|
      video.trigger_slide_thumbnail_creation force: true
    end
  end

  def sync
    provider.sync_single provider_video_id
  end

  def downloads_expiring?
    return false unless provider.downloads_expire?

    # expiration time not yet stored
    return true if downloads_expire.blank?

    downloads_expire.in_time_zone < 5.minutes.from_now
  end

  def refresh_downloads!
    downloads = provider.get_download_links(provider_video_id).to_hash

    update!(
      hd_download_url: downloads[:hd_url],
      sd_download_url: downloads[:sd_url],
      downloads_expire: downloads[:expires]
    )
  end

  def current_downloads
    if downloads_expiring?
      refresh_downloads!
    end

    {
      hd: hd_download_url,
      sd: sd_download_url,
    }
  end

  def url?
    if sd_url.blank? && hd_url.blank?
      errors.add :sd_url, 'At least one url is required'
      errors.add :hd_url, 'At least one url is required'
    end
  end
end
