# frozen_string_literal: true

class Video < ApplicationRecord
  include FileReference

  belongs_to :slides_stream, class_name: 'Stream', optional: true
  belongs_to :lecturer_stream, class_name: 'Stream', optional: true
  belongs_to :pip_stream, class_name: 'Stream', optional: true
  belongs_to :subtitled_stream, class_name: 'Stream', optional: true

  # TODO: deprecate subtitle_id field in a later stage
  has_many :subtitles
  has_many :thumbnails

  file_reference :slides, \
    key_template: ->(v) { "videos/#{UUID4(v.id).to_str(format: :base62)}/" }, \
    default_attributes: {
      acl: 'public-read',
      content_type: 'application/pdf',
      cache_control: 'public, max-age=7776000',
      content_disposition: 'attachment',
    }

  file_reference :transcript, \
    key_template: ->(v) { "videos/#{UUID4(v.id).to_str(format: :base62)}/" }, \
    default_attributes: {
      acl: 'public-read',
      content_type: 'application/pdf',
      cache_control: 'public, max-age=7776000',
      content_disposition: 'attachment',
    }

  file_reference :reading_material, \
    key_template: ->(v) { "videos/#{UUID4(v.id).to_str(format: :base62)}/" }, \
    default_attributes: {
      acl: 'public-read',
      content_type: 'application/pdf',
      cache_control: 'public, max-age=7776000',
      content_disposition: 'attachment',
    }

  default_scope { order('updated_at DESC') }

  validates :title,
    presence: true,
    allow_blank: false

  validates :description,
    presence: true,
    allow_blank: true

  validate :stream?
  validate :streams_exist?

  after_commit(on: %i[create update]) do
    trigger_slide_thumbnail_creation
    trigger_audio_extraction
  end

  # TODO: ActiveModel contract violation
  #
  # The Video::Update operation assigns ActiveModel errors to an
  # "attribute" `:invalid_subtitle`. Since Rails 6.1, responding or
  # rendering with the errors will raise an error if that attribute is
  # not defined on this ActiveModel model.
  attribute :invalid_subtitle

  def stream?
    if lecturer_stream_id.nil? &&
       slides_stream_id.nil? &&
       pip_stream_id.nil?

      errors.add :lecturer_stream_id, 'At least one stream is required'
      errors.add :slides_stream_id, 'At least one stream is required'
      errors.add :pip_stream_id, 'At least one stream is required'
    end
  end

  def streams_exist?
    if lecturer_stream_id && (!Stream.exists? lecturer_stream_id)
      errors.add  :lecturer_stream_id, 'The Stream does not exist'
    end
    if slides_stream_id && (!Stream.exists? slides_stream_id)
      errors.add  :slides_stream_id, 'The Stream does not exist'
    end
    if pip_stream_id && (!Stream.exists? pip_stream_id)
      errors.add  :pip_stream_id, 'The Stream does not exist'
    end
    if subtitled_stream_id && (!Stream.exists? subtitled_stream_id)
      errors.add  :subtitled_stream_id, 'The Stream does not exist'
    end
  end

  def thumbnail
    if pip_stream&.poster
      pip_stream.poster
    elsif lecturer_stream&.poster
      lecturer_stream.poster
    elsif slides_stream&.poster
      slides_stream.poster
    end
  end

  def duration
    if pip_stream
      pip_stream.duration
    else
      durations = []
      durations << lecturer_stream.duration if lecturer_stream
      durations << slides_stream.duration if slides_stream
      durations.max
    end
  end

  def audio_url
    uri = pip_stream&.audio_uri || lecturer_stream&.audio_uri
    Xikolo::S3.object(uri).public_url if uri
  end

  def trigger_slide_thumbnail_creation(force: false)
    # Trigger the worker when forced or whenever there is an update
    # to the `pip_stream_id` or `slides_uri`.
    return unless force ||
                  saved_change_to_pip_stream_id? ||
                  saved_change_to_slides_uri?

    ThumbnailGenerationWorker.perform_async(id)
  end

  def trigger_audio_extraction
    return unless Xikolo.config.audio_extraction
    return unless saved_change_to_pip_stream_id? ||
                  (!pip_stream_id && saved_change_to_lecturer_stream_id?)

    ExtractAudioWorker.perform_async(pip_stream_id || lecturer_stream_id)
  end

  def trigger_subtitle_sync(language)
    SyncSubtitleWorker.perform_async(id, language)
  end

  def sync
    [slides_stream, lecturer_stream, pip_stream, subtitled_stream].map {|stream| stream&.sync }
  end

  def self.referenced?(uri)
    # Check for remaining file references
    [
      where(transcript_uri: uri),
      where(reading_material_uri: uri),
      where(slides_uri: uri),
      where('description LIKE ?', "%#{uri}%"),
    ].reduce(:or).exists?
  end
end
