# frozen_string_literal: true

require 'xikolo/s3'

class Video::Update < ApplicationOperation
  attr_reader :video

  def initialize(video, params)
    super()

    @video = video
    @params = params
    @uploads = {}
    @new_objects = []
    @replaced_objects = []
    @errors = []
  end

  def call
    extract_uploads!
    Video.transaction do
      handle_file_removals!
      video.update @params
      handle_description!
      # verify uploads are valid:
      validate_uploads!
      if @upload_objects.key? :subtitles
        process_subtitles! @upload_objects.delete(:subtitles)
      end
      store_attached_files!
      video.save! if @errors.empty?
    end
    if @errors.empty?
      delete_replaced_files!
      processor&.commit!
    else
      process_errors!
    end
    video
  rescue ActiveRecord::RecordInvalid
    process_errors!
    video
  end

  private

  attr_reader :processor

  def configure_processor!(input)
    @processor = Xikolo::S3::TextWithUploadsProcessor.new \
      bucket: :video,
      purpose: 'video_description',
      current: video.description,
      text: input
    processor.on_new do |upload|
      vid = UUID4(video.id).to_str(format: :base62)
      id = UUID4.new.to_str(format: :base62)
      {
        key: "videos/#{vid}/rtfiles/#{id}/#{upload.sanitized_name}",
        acl: 'public-read',
        cache_control: 'public, max-age=7776000',
        content_disposition: 'inline',
        content_type: upload.content_type,
      }
    end
  end

  def handle_description!
    return unless @params.key? :description

    description = @params.delete(:description)
    if description.blank?
      video.description = description
      return
    end
    configure_processor! description
    processor.parse!
    video.description = processor.result
    return @replaced_objects += processor.obsolete_uris if processor.valid?

    processor.errors.each do |_url, code, _message|
      @errors << [:description, code.to_s]
    end
  end

  def extract_uploads!
    @uploads = {}
    {
      reading_material: :video_material,
      subtitles: :video_subtitles,
      slides: :video_slides,
      transcript: :video_transcript,
    }.each do |type, purpose|
      id = @params.delete(:"#{type}_upload_id")
      next unless id

      upload = Xikolo::S3::SingleFileUpload.new(id, purpose:)
      @uploads[type] = upload unless upload.empty?
    rescue Aws::S3::Errors::ServiceError => e
      ::Mnemosyne.attach_error(e)
      ::Sentry.capture_exception(e)
      @errors << [:"#{type}_upload_id", 'could not process file upload']
    end
  end

  def validate_uploads!
    @upload_objects = {}
    @uploads.each_pair do |type, upload|
      @upload_objects[type] = upload.accepted_file!
    rescue RuntimeError
      @errors << [:"#{type}_upload_id", 'invalid upload']
    end
    raise ActiveRecord::RecordInvalid if @errors.any?
  end

  def process_subtitles!(upload_object)
    case upload_object.extname
      when '.zip'
        Subtitle.extract! upload_object, video
      when '.vtt'
        lang = Subtitle.extract_lang upload_object.sanitized_name
        if lang
          content = upload_object.get.body.read
          Subtitle.attach! content, lang, video, automatic: false
        end
    end
  rescue InvalidSubtitleError => e
    # TODO: ActiveModel contract violation
    #
    # This code assigns an error to "attributes" (e.g. `invalid_subtitles`)
    # that DO NOT exist on a Video. This does not work in Rails 6.1 anymore,
    # as the ActiveModel errors collection and error classes will try to look
    # up the attribute definitions and values for localization.
    # Any respond or rendering of errors with an error added to
    # `:invalid_subtitle` ends up raising this exception:
    #
    #     #<NoMethodError: undefined method `invalid_subtitle' for #<Video>>
    #
    # As a workaround and to not change the API contract by changing
    # fields, an unused dump attribute called :invalid_subtitle exists
    # to the Video model.
    @errors << [e.message.to_sym, {identifiers: e.identifiers}]
    raise ActiveRecord::RecordInvalid
  end

  def handle_file_removals!
    %i[reading_material slides transcript].each do |type|
      next unless @params.key? :"#{type}_url"

      value = @params.delete :"#{type}_url"
      next if value == @video.send(:"#{type}_url") # unchanged!

      if value.nil? # file should be removed
        next if @video[:"#{type}_uri"].nil?

        FileDeletionWorker.perform_async @video[:"#{type}_uri"]
        @params[:"#{type}_uri"] = nil
      else
        @errors << [:"#{type}_url", 'field can only be cleared (nil), not set']
      end
    end
  end

  def store_attached_files!
    # Store each file
    @upload_objects.each_pair do |upload, upload_object|
      old_file = video.send :"#{upload}_uri"
      if old_file && Video.where("#{upload}_uri": old_file) \
          .where.not(id: video.id).any?
        old_file = nil
      end
      override_params = {
        metadata_directive: 'REPLACE',
        content_disposition: "attachment; filename=\"#{upload_object.sanitized_name}\"",
        key: upload_object.unique_sanitized_name,
      }
      video.send :"new_#{upload}_file", override_params do |object, params|
        object.copy_from upload_object, params
      end
      @new_objects << video.send(:"#{upload}_uri")
      @replaced_objects << old_file if old_file
    rescue Aws::S3::Errors::ServiceError => e
      ::Mnemosyne.attach_error(e)
      ::Sentry.capture_exception(e)
      @errors << [:"#{upload}_upload_id", 'could not process file upload']
    end
  end

  def delete_replaced_files!
    # Schedule the deletion of all replaced S3 objects
    @replaced_objects.each do |uri|
      FileDeletionWorker.perform_in 1.hour, uri
    end
  end

  def process_errors!
    delete_already_uploaded_files!
    processor&.rollback!
    @errors.each do |field, error|
      # In some cases, `error` is not a String but a Hash with additional
      # details. We must force the code here to pass `error` as the second
      # position argument.
      video.errors.add(field, error, **{})
    end
  end

  def delete_already_uploaded_files!
    # we remove all new_objects:
    @new_objects.each do |uri|
      Xikolo::S3.object(uri).delete
    end
  end
end
