# frozen_string_literal: true

require 'zip'

class FetchThumbnailsWorker
  require 'json'
  include Sidekiq::Job
  sidekiq_options queue: :default

  # we do this in a worker because this might fail
  # or be too slow for sync requests
  def perform(video_id, counter, url)
    uploaded_objects = []
    replaced_objects = []

    video = Video.find video_id
    vid = UUID4(video.id).to_s(format: :base62)
    bucket = Xikolo::S3.bucket_for(:video)
    response = Typhoeus.get(url, followlocation: true)

    Video.transaction do
      # Upload zip for archive purpose
      thumbnails_object = bucket.put_object(
        key: "videos/#{vid}/thumbnails_v#{counter}.zip",
        body: response.body,
        acl: 'private',
        content_type: 'application/zip'
      )
      uploaded_objects << thumbnails_object
      video.thumbnails_uri = thumbnails_object.storage_uri

      zip = Zip::File.open_buffer(response.body)

      # Parse json files with thumbnail references:
      slides_index = zip.find_entry('slides.json')
      thumbnails_index = zip.find_entry('thumbnails.json')
      if slides_index
        thumbnails_data = JSON.parse zip.read slides_index
        subdir = 'slides'
      elsif thumbnails_index
        thumbnails_data = JSON.parse zip.read thumbnails_index
        subdir = 'thumbnails'
      else
        raise
      end

      sub = Thumbnail.arel_table.alias('other')
      # delete all s3 files that are references only by our thumbnails
      # (that will now be deleted)
      replaced_objects = Thumbnail.where(video_id: video.id) \
        .where.not(Thumbnail.from(sub)
          .where(sub[:file_uri].eq(Thumbnail.arel_table[:file_uri]))
          .where(sub[:id].not_eq(Thumbnail.arel_table[:id])).arel.exists) \
        .pluck(:file_uri)
      Thumbnail.where(video_id: video.id).delete_all

      thumbnails_data['images'].each do |image|
        start_position = Time.parse(image['start']).seconds_since_midnight.to_i

        entry = zip.find_entry(image['path']
                   .sub('thumbnails', 'thumbnails_small'))
        next unless entry

        object = bucket.put_object(
          key: "videos/#{vid}/#{subdir}_v#{counter}/#{File.basename(image['path'])}",
          body: zip.read(entry),
          acl: 'public-read',
          content_type: 'image/jpeg',
          cache_control: 'public, max-age=7776000'
        )

        uploaded_objects << object
        Thumbnail.create video_id: video.id, \
          start_time: start_position, \
          file_uri: object.storage_uri
      end

      video.save!
    end

    replaced_objects.each do |uri|
      FileDeletionWorker.perform_in 1.hour, uri
    end
  rescue StandardError
    uploaded_objects.each do |o|
      o.delete
    rescue StandardError
      # noop
    end
    raise
  end
end
