# frozen_string_literal: true

class ThumbnailGenerationWorker
  include Sidekiq::Job
  sidekiq_options queue: :default

  # Retry: Only five times, and more slowly when extraction service seems to be down
  sidekiq_options retry: 5
  sidekiq_retry_in do |_count, exception|
    case exception
      when ServiceDown
        1.day.to_i
    end
  end

  # Should be triggered on update of a videos pdf or slide
  # this always re-downloads the video file, this could be
  # optimized later when all uploads go through the portal
  def perform(video_id)
    return if Rails.env.test? || Rails.env.integration?
    return unless thumbnail_api_key.present? && thumbnail_api_url.present?

    video = Video.find_by id: video_id
    return unless video # Don't do anything if the video has been deleted in the meantime

    stream_url = get_stream_from_video video
    return unless stream_url

    checksum = Digest::SHA256.hexdigest(video_id + Xikolo.base_url.host)
    counter = video.thumbnail_job_counter + 1
    video.update(thumbnail_job_counter: counter)

    callback_url = Xikolo.base_url.join('videos/thumbnail_api_callback')
    callback_url.query_values = {
      video_id:,
      checksum:,
      counter:,
    }

    body = {
      lang: 'en', # TODO: get from course
      callback: callback_url.to_s,
      vidUrl: stream_url,
    }

    if (io = slides_blob video)
      # HTTParty checks for :path and :read on the IO object,
      # thus we define it here on the StringIO instance
      io.define_singleton_method(:path) { "#{video_id}_slides.pdf" }

      body[:file] = io
    end

    trigger_thumbnail_generation! body
  end

  private

  def thumbnail_api_key
    Rails.application.secrets.thumbnail_api_key
  end

  def thumbnail_api_url
    Xikolo.config.thumbnail_api_url
  end

  def slides_blob(video)
    Xikolo::S3.object(video.slides_uri).get.body if video.slides_uri?
  end

  def get_stream_from_video(video)
    stream = video.slides_stream || video.pip_stream
    stream&.hd_url.presence || stream&.sd_url.presence
  end

  def trigger_thumbnail_generation!(body)
    HTTParty.post(
      thumbnail_api_url,
      body:,
      headers: {'X-api-key' => thumbnail_api_key},
      open_timeout: 5
    )
  rescue Errno::EHOSTUNREACH, Net::OpenTimeout => e
    ::Mnemosyne.attach_error(e)
    raise ServiceDown.new 'Thumbnail service down'
  end

  class ServiceDown < RuntimeError; end
end
