# frozen_string_literal: true

require 'spec_helper'

RSpec.describe LiveEventsController, type: :controller do
  let(:live_event) { create(:live_event) }
  let(:deleted_live_event) { create(:live_event, deleted: true) }
  let(:json) { JSON.parse response.body }

  describe '#index' do
    subject(:list_events) { -> { get :index, params: } }

    let(:params) { {} }

    it 'is successful' do
      expect(list_events.call.status).to eq 200
    end

    context 'with existing item' do
      before { live_event }

      describe 'response' do
        before { list_events.call }

        it 'is successful' do
          expect(response).to have_http_status :ok
        end

        describe 'json' do
          it 'responds with live event' do
            expect(json.size).to eq 1
          end
        end
      end
    end

    context 'no deleted item shown' do
      before { deleted_live_event }

      describe 'response' do
        before { list_events.call }

        it 'is successful' do
          expect(response).to have_http_status :ok
        end

        describe 'json' do
          it 'is empty' do
            expect(json).to be_empty
          end
        end
      end
    end
  end

  describe '#show' do
    subject(:show_event) { -> { get :show, params: {id: live_event.id} } }

    context 'response' do
      before { show_event.call }

      it 'is successful' do
        expect(response).to have_http_status :ok
      end
    end

    context 'json' do
      before { show_event.call }

      it 'responds with the correct live event' do
        expect(json).to eq LiveEventDecorator.new(live_event).as_json.stringify_keys
      end
    end
  end

  describe '#create' do
    subject(:create_event) { -> { post :create, params: } }

    let(:params) { attributes_for(:live_event) }

    it 'responds with 201 Created' do
      create_event.call
      expect(response).to have_http_status :created
    end

    it 'creates a new live event' do
      expect { create_event.call }.to change(LiveEvent, :count).from(0).to(1)
    end

    context 'with minimal valid data' do
      let(:params) { {title: 'test', live_video_id: '12345', description: 'test'} }

      it 'responds with 201 Created' do
        create_event.call
        expect(response).to have_http_status :created
      end
    end

    context 'with insufficient data' do
      let(:params) { {event_type: 'test'} } # needed are title, description and live_video_id

      it 'responds with 422 Unprocessable Entity' do
        create_event.call
        expect(response).to have_http_status :unprocessable_entity
      end
    end
  end

  describe '#update' do
    before { live_event }

    let(:request) { -> { patch :update, params: } }
    let(:params) { {id: live_event.id, title: 'A new title'} }

    context 'params' do
      subject(:updated_event) { live_event.reload }

      before { request.call }

      it 'changes the title' do
        expect(updated_event.title).to eq 'A new title'
      end
    end

    context 'response' do
      before { request.call }

      it 'responds with 204 No Content' do
        expect(response).to have_http_status :no_content
      end
    end
  end

  describe '#destroy' do
    subject(:delete_event) { -> { delete :destroy, params: } }

    before { live_event }

    let(:params) { {id: live_event.id} }

    it 'soft deletes the live event' do
      delete_event.call
      expect(live_event.reload.deleted).to be true
    end
  end
end
