# frozen_string_literal: true

require 'spec_helper'

RSpec.describe ProvidersController, type: :controller do
  let(:provider) { create(:provider) }
  let(:json) { JSON.parse response.body }

  # remove default provider for these tests
  before { Provider.destroy_all }

  describe '#index' do
    subject(:list_all_providers) { -> { get :index } }

    before { provider; list_all_providers.call }

    it 'answers with a list' do
      expect(json.size).to eq 1
    end

    it 'answers with provider objects' do
      expect(json[0]).to eq ProviderDecorator.new(provider)
        .as_json
        .stringify_keys
    end

    context 'with filter for default provider' do
      subject(:list_default_providers) { -> { get :index, params: {default: true} } }

      before do
        create(:provider, default: true)
        list_default_providers.call
      end

      it 'answers with one provider having a default flag' do
        expect(json.size).to eq 1
        expect(json.first['default']).to be_truthy
      end
    end
  end

  describe '#show' do
    subject(:show_provider) { -> { get :show, params: {id: provider.id} } }

    before { show_provider.call }

    it 'answers with a provider object' do
      expect(json).to eq ProviderDecorator.new(provider)
        .as_json
        .stringify_keys
    end
  end

  describe '#create' do
    subject(:create_provider) { -> { post :create, params: } }

    let(:params) { attributes_for(:provider) }

    it 'creates a new provider' do
      expect { create_provider.call }.to change(Provider, :count).by 1
    end

    context 'with invalid data' do
      let(:params) { {name: ''} }

      it 'does not create a new provider' do
        expect { create_provider.call }.not_to change(Provider, :count)
      end
    end

    context 'with providers with same name' do
      let(:name) { 'test' }
      let(:params) { super().merge! name: }

      before { create(:provider, name:) }

      it 'does not create a new provider' do
        expect { create_provider.call }.not_to change(Provider, :count)
      end
    end

    context 'with multiple default providers' do
      before { create(:provider, default: true) }

      let(:params) { super().merge! default: true }

      it 'does not create a new provider' do
        expect { create_provider.call }.not_to change(Provider, :count)
      end
    end
  end

  describe '#update' do
    subject(:update_provider) { -> { put :update, params: {id: provider.id, name: 'new_name'} } }

    before { provider }

    it 'updates the record' do
      expect { update_provider.call }.to change { provider.reload.name }.to 'new_name'
    end
  end

  describe '#delete' do
    subject(:delete_provider) { -> { delete :destroy, params: {id: provider.id} } }

    before { provider }

    it 'removes the provider' do
      expect { delete_provider.call }.to change(Provider, :count).by(-1)
    end
  end
end
