# frozen_string_literal: true

require 'spec_helper'

RSpec.describe StreamsController, type: :controller do
  include_context 'default_provider'

  let(:stream) { create(:stream) }
  let(:params) { attributes_for(:stream) }
  let(:json) { JSON.parse response.body }

  describe '#index' do
    subject(:list_streams) { get :index, params: }

    before { stream }

    let(:params) { {} }

    it 'is successful' do
      list_streams
      expect(response).to have_http_status :ok
    end

    it 'answers with a list' do
      list_streams
      expect(json.size).to eq 1
    end

    it 'answers with stream objects' do
      list_streams
      expect(json[0]).to eq StreamDecorator.new(stream).as_json.stringify_keys
    end

    describe '(order)' do
      let(:stream) { create(:stream, title: 'foobaz', created_at: 1.day.ago) }

      before do
        create(:stream, title: 'barbaz', created_at: 2.days.ago)
        create(:stream, title: 'foobar', created_at: 1.hour.ago)
      end

      it 'orders streams by creation date descending' do
        list_streams
        expect(json).to match [
          hash_including('title' => 'foobar'),
          hash_including('title' => 'foobaz'),
          hash_including('title' => 'barbaz'),
        ]
      end
    end

    context 'with similar titles' do
      before do
        create(:stream, title: 'foobaz')
        create(:stream, title: 'barbaz')
        create(:stream, title: 'foobar')
        create(:stream, title: 'other')
      end

      describe '(prefix filter)' do
        let(:params) { {prefix: 'foo'} }

        it 'returns the matching streams' do
          list_streams
          expect(json.size).to eq 2
        end

        it 'matches upper and lower case' do
          create(:stream, title: 'FOOBAR')

          list_streams
          expect(json.size).to eq 3
          expect(json).to contain_exactly(hash_including('title' => 'foobar'), hash_including('title' => 'foobaz'), hash_including('title' => 'FOOBAR'))
        end

        it 'returns video list sorted by title ascending' do
          list_streams
          expect(json).to match [
            hash_including('title' => 'foobar'),
            hash_including('title' => 'foobaz'),
          ]
        end
      end
    end
  end

  describe '#show' do
    subject(:show_stream) { get :show, params: {id: stream.id} }

    it 'is successful' do
      show_stream
      expect(response).to have_http_status :ok
    end

    it 'answers with a stream object' do
      show_stream
      expect(json).to eq StreamDecorator.new(stream).as_json.stringify_keys
    end
  end

  describe '#create' do
    subject(:create_stream) { -> { post :create, params: } }

    it 'responds with 201 Created' do
      create_stream.call
      expect(response).to have_http_status :created
    end

    it 'creates a new stream' do
      expect { create_stream.call }.to change(Stream, :count).from(0).to(1)
    end

    it 'has the default provider' do
      create_stream.call
      expect(json['provider_id']).to eq default_provider.id
    end

    context 'with provider video ID' do
      let(:params) { super().merge(provider_video_id: '12345') }

      it 'stores the ID' do
        expect { create_stream.call }.to change(Stream, :count).from(0).to(1)

        stream = Stream.first
        expect(stream.provider_video_id).to eq '12345'
      end
    end

    context 'with provider name' do
      let(:provider) { create(:provider) }
      let(:params) { super().merge provider_name: provider.name }

      it 'creates stream with the referenced provider' do
        create_stream.call
        expect(json['provider_id']).to eq provider.id
      end

      context 'with wrong provider name' do
        let(:params) { super().merge provider_name: 'wrong name' }

        it 'does not create a stream' do
          expect { create_stream.call }.not_to change(Stream, :count).from(0)
        end
      end
    end

    context 'with provider id' do
      let(:provider) { create(:provider) }
      let(:params) { super().merge provider_id: provider.id }

      it 'creates stream with the referenced provider' do
        create_stream.call
        expect(json['provider_id']).to eq provider.id
      end

      context 'with provider id explicitly set to nil' do
        let(:params) { super().merge provider_id: nil }

        it 'uses the default provider' do
          create_stream.call
          expect(json['provider_id']).to eq default_provider.id
        end
      end
    end

    context 'with hls_url' do
      let(:params) { super().merge(hls_url: 'http://hasso.de/vid.m3u8') }

      it 'creates stream with that HLS URL' do
        create_stream.call
        expect(json['hls_url']).to eq params[:hls_url]
      end
    end

    context 'with invalid data' do
      let(:params) { super().except :sd_url, :hd_url }

      it 'responses with 422 on invalid data' do
        create_stream.call
        expect(response).to have_http_status :unprocessable_entity
      end
    end
  end

  describe '#update' do
    subject(:update_stream) { put :update, params: params.merge(id: stream.id, width: 900) }

    it 'answers' do
      update_stream
      expect(response).to have_http_status :no_content
    end

    it 'changes the width' do
      expect { update_stream }.to change { stream.reload.width }.from(1600).to(900)
    end

    context 'with new hls_url' do
      let(:params) { super().merge(hls_url: 'https://new.video/hls.m3u8') }

      it 'changes the attribute on the model' do
        expect { update_stream }.to change { stream.reload.hls_url }.to('https://new.video/hls.m3u8')
      end
    end
  end

  describe '#destroy' do
    subject(:delete_stream) { -> { delete :destroy, params: {id: stream.id} } }

    before { stream }

    it 'deletes the stream' do
      expect { delete_stream.call }.to change(Stream, :count).by(-1)
    end
  end
end
