# frozen_string_literal: true

require 'spec_helper'

RSpec.describe SubtitlesController, type: :controller do
  let(:video) { create(:video) }
  let(:other_video) { create(:video) }
  let(:params) { {video_id: video.id, lang: 'en'} }
  let(:subtitle) { create(:subtitle, :with_cues, params) }
  let(:other_subtitle) { create(:subtitle, :with_cues, video_id: other_video.id) }
  let(:json) { JSON.parse response.body }

  describe '#index' do
    subject(:list_subs) { get :index }

    before { subtitle; other_subtitle }

    it 'is successful' do
      list_subs
      expect(response).to have_http_status :ok
    end

    it 'answers with a list' do
      list_subs
      expect(json.size).to eq 2
    end

    it 'answers with subtitle objects' do
      list_subs
      expect(json[0]).to eq SubtitleDecorator.new(subtitle).as_json.stringify_keys
    end

    context 'with matching video_id and lang parameters' do
      subject(:filtered_sub) { get :index, params: query_params }

      let(:query_params) { {video_id: subtitle.video_id, lang: subtitle.lang} }

      it 'answers with a filtered list' do
        filtered_sub
        expect(json.size).to eq 1
      end
    end

    context 'with non-matching lang parameter' do
      subject(:filtered_sub) { get :index, params: query_params }

      let(:query_params) { {video_id: subtitle.video_id, lang: 'pl'} }

      it 'answers an empty list' do
        filtered_sub
        expect(json).to be_empty
      end
    end
  end

  describe '#show' do
    subject(:show_sub) { get :show, params: {id: subtitle.id} }

    it 'is successful' do
      show_sub
      expect(response).to have_http_status :ok
    end

    it 'answers with a subtitle object' do
      show_sub
      expect(json).to eq SubtitleDecorator.new(subtitle).as_json.stringify_keys
    end
  end

  describe '#create' do
    subject(:create_sub) { post :create, params: params.merge(vtt:) }

    let(:vtt) { Rails.root.join('spec/support/files/subtitles/valid_en.vtt').read }

    it 'is successful' do
      create_sub
      expect(response).to have_http_status :created
    end

    it 'answers with a subtitle object' do
      create_sub
      new_subtitle = Subtitle.find_by(params)
      expect(json).to eq SubtitleDecorator.new(new_subtitle).as_json.stringify_keys
    end

    describe 'errors' do
      before { create_sub }

      context 'when the vtt file is blank' do
        let(:vtt) { '' }

        it { expect(response).to have_http_status :bad_request }
      end

      context 'when the vtt file is invalid' do
        let(:vtt) { 'Invalid VTT' }

        it { expect(response).to have_http_status :bad_request }
      end

      context 'when the vtt file has an invalid interval' do
        let(:vtt) { Rails.root.join('spec/support/files/subtitles/invalid_en.vtt').read }

        it 'errors, with identifiers in the response body' do
          expect(response).to have_http_status :unprocessable_entity
          expect(json).to eq(
            'vtt' => {'message' => 'invalid_subtitle', 'identifiers' => [1]}
          )
        end
      end

      context 'when the vtt file has multiple invalid intervals' do
        let(:vtt) { Rails.root.join('spec/support/files/subtitles/multiple_invalid_en.vtt').read }

        it 'errors, with identifiers in the response body' do
          expect(response).to have_http_status :unprocessable_entity
          expect(json).to eq(
            'vtt' => {'message' => 'invalid_subtitle', 'identifiers' => [1, 2]}
          )
        end
      end
    end
  end
end
