# frozen_string_literal: true

require 'spec_helper'

RSpec.describe VideosController, type: :controller do
  let(:video) { create(:video) }
  let(:stream) { create(:stream) }
  let(:params) { attributes_for(:video) }
  let(:json) { JSON.parse response.body }

  describe '#index' do
    subject(:list_videos) { -> { get :index } }

    before { video }

    it 'is successful' do
      list_videos.call
      expect(response).to have_http_status :ok
    end

    it 'answers with a list' do
      list_videos.call
      expect(json.size).to eq 1
    end

    it 'answers with video objects' do
      list_videos.call
      expect(json[0]).to eq VideoDecorator.new(video).as_json.stringify_keys
    end
  end

  describe '#show' do
    subject(:show_video) { -> { get :show, params: } }

    let(:params) { {id: video.id} }

    it 'is successful' do
      show_video.call
      expect(response).to have_http_status :ok
    end

    it 'answers with a video object' do
      show_video.call
      expect(json).to eq VideoDecorator.new(video).as_json.stringify_keys
    end

    context 'with embed=streams' do
      let(:params) { super().merge(embed: 'streams') }

      it 'answers with a embedded streams' do
        show_video.call

        streams = {
          lecturer_stream: video.lecturer_stream.decorate,
          slides_stream: video.slides_stream.decorate,
          pip_stream: video.pip_stream.decorate,
          subtitled_stream: video.subtitled_stream.decorate,
        }.as_json
        expect(json).to eq VideoDecorator.new(video).as_json.merge(streams:).stringify_keys
      end
    end
  end

  describe '#create' do
    subject(:create_video) { -> { post :create, params: } }

    let(:params) { super().merge lecturer_stream_id: stream.id }

    it 'responds with a 201' do
      create_video.call
      expect(response).to have_http_status :created
    end

    it 'creates a new video' do
      expect { create_video.call }.to change(Video, :count).from(0).to(1)
    end

    context 'with invalid data' do
      let(:params) { {title: 'test'} }

      it 'responses with 422 on invalid data' do
        create_video.call
        expect(response).to have_http_status :unprocessable_entity
      end
    end
  end

  describe '#update' do
    subject(:update_video) { -> { put :update, params: params.merge(id: video.id, title: 'New') } }

    it 'is successful' do
      update_video.call
      expect(response).to have_http_status :no_content
    end

    it 'changes the title' do
      expect { update_video.call }.to change { Video.find(video.id).title }
        .from('Test Video')
        .to('New')
    end
  end

  describe '#delete' do
    subject(:delete_video) { -> { delete :destroy, params: {id: video.id} } }

    it 'responds with 204 No Content' do
      delete_video.call
      expect(response).to have_http_status :no_content
    end

    it 'removes a video' do
      video
      expect { delete_video.call }.to change(Video, :count).from(1).to(0)
    end
  end
end
