# frozen_string_literal: true

require 'spec_helper'

RSpec.describe StreamDecorator, type: :decorator do
  let(:stream) { create(:stream) }

  context 'as_json' do
    subject(:json) { stream.decorate.as_json(api_version: 1) }

    let(:stream) { create(:stream, :from_api) }

    it 'exposes all interesting stream information' do
      expect(json).to match({
        'id' => an_instance_of(String),
        'title' => an_instance_of(String),
        'hd_url' => an_instance_of(String),
        'sd_url' => an_instance_of(String),
        'hls_url' => an_instance_of(String),
        'hd_size' => an_instance_of(Integer),
        'sd_size' => an_instance_of(Integer),
        'hls_size' => an_instance_of(Integer),
        'width' => an_instance_of(Integer),
        'height' => an_instance_of(Integer),
        'poster' => an_instance_of(String),
        'duration' => an_instance_of(Integer),
        'provider_id' => an_instance_of(String),
        'downloads' => an_instance_of(Hash),
        'provider_video_id' => an_instance_of(String),
        'provider' => an_instance_of(Hash),
        'created_at' => an_instance_of(String),
      })
    end

    describe 'provider metadata' do
      subject(:provider_metadata) { json['provider'] }

      context 'for a Vimeo video' do
        let(:stream) { create(:stream, :from_api, provider: vimeo_provider) }
        let(:vimeo_provider) { create(:provider, :vimeo) }

        it 'only exposes type, provider ID, and provider name' do
          expect(provider_metadata).to eq({
            'type' => 'vimeo',
            'id' => vimeo_provider.id,
            'name' => vimeo_provider.name,
            'meta' => {},
          })
        end
      end

      context 'for a Kaltura video' do
        let(:stream) { create(:stream, provider: kaltura_provider) }
        let(:kaltura_provider) { create(:provider, :kaltura) }

        it 'exposes type, provider ID, provider name, and a partner ID' do
          expect(provider_metadata).to eq({
            'type' => 'kaltura',
            'id' => kaltura_provider.id,
            'name' => kaltura_provider.name,
            'meta' => {
              'partner_id' => '1234567', # value from "credentials" hash in seeds
            },
          })
        end
      end
    end

    describe 'download URLs' do
      subject(:downloads) { json['downloads'] }

      let(:stream) { create(:stream, :from_api, **stream_attrs) }

      context 'when HD and SD download URLs are known' do
        let(:stream_attrs) do
          {hd_download_url: 'https://vimeo.com/file.mp4?type=hd', sd_download_url: 'https://vimeo.com/file.mp4?type=sd'}
        end

        it 'exposes stored URLs' do
          expect(downloads).to eq({
            'hd' => 'https://vimeo.com/file.mp4?type=hd',
            'sd' => 'https://vimeo.com/file.mp4?type=sd',
          })
        end
      end

      context 'when only HD download URL is known' do
        let(:stream_attrs) do
          {hd_download_url: 'https://vimeo.com/file.mp4?type=hd', sd_download_url: nil}
        end

        it 'exposes stored URLs only for available types' do
          expect(downloads).to eq({
            'hd' => 'https://vimeo.com/file.mp4?type=hd',
          })
        end
      end

      context 'when only SD download URL is known' do
        let(:stream_attrs) do
          {hd_download_url: nil, sd_download_url: 'https://vimeo.com/file.mp4?type=sd'}
        end

        it 'exposes stored URLs only for available types' do
          expect(downloads).to eq({
            'sd' => 'https://vimeo.com/file.mp4?type=sd',
          })
        end
      end
    end
  end
end
