# frozen_string_literal: true

require 'spec_helper'

RSpec.describe VideoDecorator, type: :decorator do
  include Rails.application.routes.url_helpers

  let(:video) { create(:video) }
  let(:video_decorator) { described_class.new video }

  context 'as_json' do
    subject(:video_resource) { json }

    let(:json) { video_decorator.as_json.stringify_keys }

    it 'expects default fields' do
      expect(video_resource.keys).to match_array %w[
        id
        title
        description
        slides_stream_id
        lecturer_stream_id
        pip_stream_id
        subtitled_stream_id

        thumbnail
        duration
        subtitles

        self_url
        subtitles_url
        thumbnails_url
        slides_url
        audio_url
        transcript_url
        reading_material_url
        thumbnails_extraction_url
        sync_url
      ]
    end

    context 'with subtitles' do
      let!(:subs) { create(:subtitle, video_id: video.id) }

      it 'includes all meta information about subtitles' do
        expect(video_resource['subtitles']).to eq [
          {
            id: subs.id,
            language: subs.lang,
            automatic: subs.automatic,
            url: subtitle_url(id: subs.id),
            clone_url: clone_subtitle_url(id: subs.id),
          }.stringify_keys,
        ]
      end

      it 'includes the correct subtitles_url' do
        expect(video_resource['subtitles_url']).to eq subtitles_url(video_id: video.id)
      end
    end

    context 'with attached s3 files' do
      let(:video) do
        create(:video, \
          slides_uri: 's3://xikolo-video/videos/1/slides.pdf', \
          transcript_uri: 's3://xikolo-video/videos/1/transcript.pdf', \
          reading_material_uri: 's3://xikolo-video/videos/1/read.pdf')
      end

      it 'returns slides download url' do
        expect(video_resource['slides_url']).to eq 'http://s3.xikolo.de/xikolo-video/videos/1/slides.pdf'
      end

      it 'returns transcript download url' do
        expect(video_resource['transcript_url']).to eq 'http://s3.xikolo.de/xikolo-video/videos/1/transcript.pdf'
      end

      it 'returns reading material url' do
        expect(video_resource['reading_material_url']).to eq 'http://s3.xikolo.de/xikolo-video/videos/1/read.pdf'
      end
    end

    context 'with audio s3 files on pip stream' do
      let(:video) do
        create(:video,
          pip_stream: create(:stream,
            audio_uri: 's3://xikolo-video/streams/1/audio_v1.mp3'))
      end

      it 'returns audio download url' do
        expect(video_resource['audio_url']).to eq 'http://s3.xikolo.de/xikolo-video/streams/1/audio_v1.mp3'
      end
    end

    context 'with audio s3 files on lecturer stream' do
      let(:video) do
        create(:video,
          pip_stream: create(:stream, audio_uri: nil),
          lecturer_stream: create(:stream,
            audio_uri: 's3://xikolo-video/streams/2/audio_v1.mp3'))
      end

      it 'returns audio download url' do
        expect(video_resource['audio_url']).to eq 'http://s3.xikolo.de/xikolo-video/streams/2/audio_v1.mp3'
      end
    end

    context 'without thumbnails' do
      let(:video) { create(:video, thumbnails_uri: nil) }

      it 'does not have a thumbnails relation' do
        expect(video_resource.keys).not_to include('thumbnails_url')
      end
    end
  end
end
