# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Provider#attach_subtitle!' do
  subject(:attach) { provider.attach_subtitle! subtitle.video.pip_stream, subtitle }

  before do
    # Remove default provider for these tests
    Provider.destroy_all

    stub_request(:get, "https://api.vimeo.com/videos/#{subtitle.video.pip_stream.provider_video_id}")
      .with(
        headers: {'Authorization' => "Bearer #{provider.token}"},
        query: {fields: 'metadata.connections.texttracks.uri'}
      ).to_return(vimeo_video_response)
  end

  let(:provider) { create(:provider) }

  let(:vimeo_video_response) do
    {
      body: {
        metadata: {
          connections: {
            texttracks: {uri: 'https://api.vimeo.com/texttracks'},
          },
        },
      }.to_json,
    }
  end

  let!(:stub_vimeo_texttrack) do
    stub_request(:post, 'https://api.vimeo.com/texttracks')
      .with(
        headers: {'Authorization' => "Bearer #{provider.token}"}
      ).to_return(
        status: 201,
        body: {
          link: 'https://storage.vimeo.com/rawtrack',
          uri: 'https://api.vimeo.com/apitrack',
        }.to_json
      )
  end

  let!(:stub_vimeo_texttrack_body) do
    stub_request(:put, 'https://storage.vimeo.com/rawtrack')
      .with(
        headers: {'Authorization' => "Bearer #{provider.token}"},
        body: /^WEBVTT/
      ).to_return(status: 204)
  end

  let!(:stub_vimeo_texttrack_active) do
    stub_request(:patch, 'https://api.vimeo.com/apitrack')
      .with(
        headers: {'Authorization' => "Bearer #{provider.token}"},
        body: {active: true}
      ).to_return(body: {}.to_json)
  end

  context 'for a manually created subtitle' do
    let(:subtitle) { create(:subtitle, :with_cues, automatic: false, lang: 'de') }

    it 'calls the Vimeo API relying on Vimeo\'s labeling' do
      attach

      expect(
        stub_vimeo_texttrack.with(body: {
          type: 'subtitles',
          language: 'de',
        })
      ).to have_been_requested.once
      expect(stub_vimeo_texttrack_body).to have_been_requested.once
      expect(stub_vimeo_texttrack_active).to have_been_requested.once
    end
  end

  context 'for certain languages' do
    let(:subtitle) { create(:subtitle, :with_cues, automatic: false, lang: 'cn') }

    it 'calls the Vimeo API using Vimeo\'s language identifiers' do
      attach

      expect(
        stub_vimeo_texttrack.with(body: {
          type: 'subtitles',
          language: 'zh',
        })
      ).to have_been_requested.once
      expect(stub_vimeo_texttrack_body).to have_been_requested.once
      expect(stub_vimeo_texttrack_active).to have_been_requested.once
    end
  end

  context 'when the video does not exist on Vimeo' do
    let(:subtitle) { create(:subtitle, :with_cues, automatic: false, lang: 'de') }

    let(:vimeo_video_response) { {status: 404} }

    it 'does not try to create text tracks for the video' do
      attach

      expect(stub_vimeo_texttrack).not_to have_been_requested.once
      expect(stub_vimeo_texttrack_body).not_to have_been_requested
      expect(stub_vimeo_texttrack_active).not_to have_been_requested
    end
  end
end
