# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Provider#remove_subtitles!' do
  before do
    # Remove default provider for these tests
    Provider.destroy_all

    stub_request(:get, "https://api.vimeo.com/videos/#{stream.provider_video_id}")
      .with(
        headers: {'Authorization' => "Bearer #{provider.token}"},
        query: {fields: 'metadata.connections.texttracks.uri'}
      ).to_return(vimeo_video_response)

    stub_request(:get, 'https://api.vimeo.com/texttracks')
      .with(
        headers: {'Authorization' => "Bearer #{provider.token}"}
      ).to_return(
        body: {data: [
          {language: 'de', type: 'subtitles', uri: '/thetexttrack/first'},
          {language: 'zh', type: 'subtitles', uri: '/thetexttrack/second'},
          {language: 'de', type: 'subtitles', uri: '/thetexttrack/third'},
          {language: 'de', type: 'captions', uri: '/thetexttrack/fourth'},
        ]}.to_json
      )
  end

  let(:provider) { stream.provider }
  let(:stream) { create(:stream) }

  let(:vimeo_video_response) do
    {
      body: {
        metadata: {
          connections: {
            texttracks: {uri: 'https://api.vimeo.com/texttracks'},
          },
        },
      }.to_json,
    }
  end

  let!(:stub_vimeo_delete_texttrack_1) do
    stub_request(:delete, 'https://api.vimeo.com/thetexttrack/first')
      .with(
        headers: {'Authorization' => "Bearer #{provider.token}"}
      ).to_return(status: 204)
  end

  let!(:stub_vimeo_delete_texttrack_2) do
    stub_request(:delete, 'https://api.vimeo.com/thetexttrack/second')
      .with(
        headers: {'Authorization' => "Bearer #{provider.token}"}
      ).to_return(status: 204)
  end

  let!(:stub_vimeo_delete_texttrack_3) do
    stub_request(:delete, 'https://api.vimeo.com/thetexttrack/third')
      .with(
        headers: {'Authorization' => "Bearer #{provider.token}"}
      ).to_return(status: 204)
  end

  context 'for a language where text tracks exist' do
    it 'requests deletion of the matching subtitle text tracks via the Vimeo API' do
      provider.remove_subtitles! stream, 'de'

      expect(stub_vimeo_delete_texttrack_1).to have_been_requested.once
      expect(stub_vimeo_delete_texttrack_2).not_to have_been_requested
      expect(stub_vimeo_delete_texttrack_3).to have_been_requested.once
    end

    it 'respects Vimeo\'s differing language identifiers' do
      provider.remove_subtitles! stream, 'cn' # Vimeo does not know this one

      expect(stub_vimeo_delete_texttrack_1).not_to have_been_requested
      expect(stub_vimeo_delete_texttrack_2).to have_been_requested.once
      expect(stub_vimeo_delete_texttrack_3).not_to have_been_requested
    end
  end

  context 'for a language where no text tracks exist' do
    it 'does not request any text track deletion via the Vimeo API' do
      provider.remove_subtitles! stream, 'fr'

      expect(stub_vimeo_delete_texttrack_1).not_to have_been_requested
      expect(stub_vimeo_delete_texttrack_2).not_to have_been_requested
      expect(stub_vimeo_delete_texttrack_3).not_to have_been_requested
    end
  end

  context 'when the video does not exist on Vimeo' do
    let(:vimeo_video_response) { {status: 404} }

    it 'does not try to remove any text tracks' do
      provider.remove_subtitles! stream, 'de'

      expect(stub_vimeo_delete_texttrack_1).not_to have_been_requested
      expect(stub_vimeo_delete_texttrack_2).not_to have_been_requested
      expect(stub_vimeo_delete_texttrack_3).not_to have_been_requested
    end
  end
end
