# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Provider do
  # Remove default provider for these tests
  before { described_class.destroy_all }

  describe '(validation)' do
    subject(:new_provider) { build(:provider) }

    it { is_expected.to accept_values_for(:name, 'test') }
    it { is_expected.to accept_values_for(:default, true) }

    it { is_expected.not_to accept_values_for(:name, '') }

    context 'with existing provider' do
      let(:name) { 'test' }

      before { create(:provider, name:) }

      it { is_expected.not_to accept_values_for(:name, name) }
    end

    context 'with existing default provider' do
      before { create(:provider, default: true) }

      it { is_expected.not_to accept_values_for(:default, true) }
    end
  end

  describe '#update!' do
    context 'with default provider' do
      subject(:update_provider) { provider.update! name: 'abc' }

      let(:provider) { create(:provider, default: true) }

      before { expect(provider).to be_default }

      it 'can be updated' do
        expect { update_provider }.to change { provider.reload.name }.to('abc')
      end
    end
  end

  describe '#sync' do
    subject(:sync_provider) { provider.sync }

    let(:provider) { create(:provider) }

    let(:videos_url) do
      uri = Addressable::Template.new \
        'https://api.vimeo.com/me/videos{?fields,page,sort}'

      uri = uri.partial_expand \
        sort: 'modified_time',
        fields: %w[
          uri
          name
          duration
          width
          height
          status
          modified_time
          pictures.sizes.link
          pictures.sizes.width
          files.quality
          files.size
          files.md5
          files.link
          download
        ]

      ->(**kwargs) { uri.expand(kwargs) }
    end

    let!(:page1) do
      stub_request(:get, videos_url.call)
        .with(headers: {
          'Authorization' => "Bearer #{provider.token}",
          'Accept' => 'application/vnd.vimeo.*+json;version=3.4',
        })
        .to_return(body: page1_response)
    end

    let(:page1_response) do
      File.read 'spec/support/vimeo_api_responses/me_videos/1.json'
    end

    let!(:page2) do
      stub_request(:get, videos_url.call(page: 2))
        .with(headers: {
          'Authorization' => "Bearer #{provider.token}",
          'Accept' => 'application/vnd.vimeo.*+json;version=3.4',
        })
        .to_return(body: File.read('spec/support/vimeo_api_responses/me_videos/2.json'))
    end

    let!(:page3) do
      stub_request(:get, videos_url.call(page: 3))
        .with(headers: {
          'Authorization' => "Bearer #{provider.token}",
          'Accept' => 'application/vnd.vimeo.*+json;version=3.4',
        })
        .to_return(body: File.read('spec/support/vimeo_api_responses/me_videos/3.json'))
    end

    it 'calls the Vimeo API' do
      sync_provider
      expect(page1).to have_been_requested.once
      expect(page2).to have_been_requested.once
      expect(page3).to have_been_requested.once
    end

    it 'chooses the larger one out of two SD streams in a Vimeo video resource' do
      sync_provider
      expect(Stream.find_by(provider_video_id: '124597457').sd_size).to eq 250
    end

    it 'chooses the largest download URL both for SD and HD in a Vimeo video resource' do
      sync_provider

      stream = Stream.find_by(provider_video_id: '124597457')
      expect(stream.sd_download_url).to eq 'https://download.vimeo.com/larger.sd.mp4'
      expect(stream.hd_download_url).to eq 'https://download.vimeo.com/larger.hd.mp4'
    end

    context 'without any data' do
      let(:page1_response) do
        File.read 'spec/support/vimeo_api_responses/me_videos/no_data.json'
      end

      it 'raises' do
        expect { sync_provider }.to raise_error KeyError
      end
    end

    context 'without downloads in response' do
      let(:page1_response) do
        File.read 'spec/support/vimeo_api_responses/me_videos/no_downloads.json'
      end

      it 'creates a stream' do
        expect { sync_provider }.to change(Stream, :count).from(0).to(1)
      end
    end

    context 'without expiration date for downloads' do
      let(:page1_response) do
        File.read 'spec/support/vimeo_api_responses/me_videos/no_expiration.json'
      end

      it 'creates a stream' do
        expect { sync_provider }.to change(Stream, :count).from(0).to(1)
      end
    end

    context 'with synchronized_at date' do
      let(:sync_date) do
        # A date between before last item on page 2
        Time.iso8601('2017-10-15T16:00:00+02:00')
      end

      before { provider.update! synchronized_at: sync_date }

      it 'stops iteration before page 3' do
        sync_provider
        expect(page1).to have_been_requested.once
        expect(page2).to have_been_requested.once
        expect(page3).not_to have_been_requested
      end

      it 'updates synchronized_at' do
        Timecop.freeze do
          # Force microsecond precision instead of nanoseconds
          time = Time.iso8601 Time.zone.now.iso8601(6)

          expect do
            sync_provider
          end.to change {
            provider.reload.synchronized_at
          }.from(sync_date).to(time)
        end
      end
    end

    # These tests must not be wrapped in a transaction, as each thread needs
    # its own transaction in order to acquire a lock.
    describe 'concurrent syncs', transaction: false do
      subject(:sync_in_thread) do
        lambda do |provider|
          allow(provider.send(:adapter)).to receive(:sync) do
            # We are letting each thread sleep a bit to ensure the locks conflict.
            sleep 1

            # Here (and in the thread below), we set a thread-local variable
            # that can be asserted against in the test scenarios below, to
            # check whether a provider did actually try to sync, or whether an
            # error was raised.
            Thread.current[:synced] = true
          end

          Thread.new do
            provider.sync(full:)
          rescue Provider::SyncAlreadyRunning
            Thread.current[:aborted] = true
          end
        end
      end

      context 'for the same provider' do
        subject(:concurrent_syncs) do
          # We load the provider anew for each thread to avoid sharing internal state
          [
            sync_in_thread.call(Provider.find(provider.id)),
            sync_in_thread.call(Provider.find(provider.id)),
          ]
        end

        context 'full syncs' do
          let(:full) { true }

          it 'executes only one sync' do
            # These assertions are written to check that e.g. one thread syncs,
            # but we don't care which one it is. This avoids order dependencies
            # on how the threads are scheduled.
            expect(concurrent_syncs.each(&:join)).to contain_exactly(satisfy {|t| t[:synced] == true }, satisfy {|t| t[:synced].nil? })
          end

          it 'raises an error from the failing sync' do
            expect(concurrent_syncs.each(&:join)).to contain_exactly(satisfy {|t| t[:aborted] == true }, satisfy {|t| t[:aborted].nil? })
          end
        end

        context 'partial syncs' do
          let(:full) { false }

          it 'executes only one sync' do
            expect(concurrent_syncs.each(&:join)).to contain_exactly(satisfy {|t| t[:synced] == true }, satisfy {|t| t[:synced].nil? })
          end

          it 'swallows the error from the failing sync' do
            expect(concurrent_syncs.each(&:join)).to contain_exactly(satisfy {|t| t[:aborted].nil? }, satisfy {|t| t[:aborted].nil? })
          end
        end
      end

      context 'for a different provider of the same type' do
        subject(:concurrent_syncs) do
          [
            sync_in_thread.call(Provider.find(provider.id)),
            sync_in_thread.call(create(:provider)),
          ]
        end

        context 'full syncs' do
          let(:full) { true }

          it 'fully executes both syncs' do
            expect(concurrent_syncs.each(&:join)).to contain_exactly(satisfy {|t| t[:synced] == true }, satisfy {|t| t[:synced] == true })
          end
        end

        context 'partial syncs' do
          let(:full) { false }

          it 'fully executes both syncs' do
            expect(concurrent_syncs.each(&:join)).to contain_exactly(satisfy {|t| t[:synced] == true }, satisfy {|t| t[:synced] == true })
          end
        end
      end
    end

    context 'for Kaltura provider' do
      let(:provider) { create(:provider, :kaltura) }
      let(:start_session_response) { File.read('./spec/support/kaltura_api_responses/privileged_session_response.xml') }
      let(:widget_session_response) { File.read('./spec/support/kaltura_api_responses/widget_session_response.xml') }
      let(:media_list_xml) do
        <<~XML
          <?xml version="1.0" encoding="UTF-8"?>
          <xml>
            <result>
              <objectType>KalturaMediaListResponse</objectType>
              <objects></objects>
              <totalCount>0</totalCount>
              <relatedObjects></relatedObjects>
            </result>
          </xml>
        XML
      end

      let!(:kaltura_session_stub) do
        stub_request(:post, 'https://www.kaltura.com/api_v3/service/session/action/startWidgetSession')
          .to_return(status: 200, body: widget_session_response, headers: {'Content-Type' => 'text/xml;charset=UTF-8'})
      end

      before do
        stub_request(:post, 'https://www.kaltura.com/api_v3/service/apptoken/action/startSession')
          .to_return(status: 200, body: start_session_response, headers: {'Content-Type' => 'text/xml;charset=UTF-8'})
        stub_request(:post, 'https://www.kaltura.com/api_v3/service/media/action/list')
          .to_return(status: 200, body: media_list_xml, headers: {'Content-Type' => 'text/xml;charset=UTF-8'})
      end

      it 'always performs a full sync' do
        # subject calls provider.sync with default params, i.e. full: false
        sync_provider

        # KalturaAdapter does not return even though partial sync is unsupported,
        # but performs the Kaltura API requests
        expect(kaltura_session_stub).to have_been_requested
      end

      context 'with a different API endpoint' do
        let!(:kaltura_session_stub) do
          stub_request(:post, 'https://api.eu.kaltura.com/api_v3/service/session/action/startWidgetSession')
            .to_return(status: 200, body: widget_session_response, headers: {'Content-Type' => 'text/xml;charset=UTF-8'})
        end

        before do
          stub_request(:post, 'https://api.eu.kaltura.com/api_v3/service/apptoken/action/startSession')
            .to_return(status: 200, body: start_session_response, headers: {'Content-Type' => 'text/xml;charset=UTF-8'})
          stub_request(:post, 'https://api.eu.kaltura.com/api_v3/service/media/action/list')
            .to_return(status: 200, body: media_list_xml, headers: {'Content-Type' => 'text/xml;charset=UTF-8'})

          xi_config <<~YML
            kaltura:
              api_url: https://api.eu.kaltura.com
          YML
        end

        it 'uses the configured URL' do
          sync_provider

          expect(kaltura_session_stub).to have_been_requested
        end
      end
    end
  end
end
