# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Stream, type: :model do
  let(:params) { attributes_for(:stream) }
  let(:stream) { build(:stream, params) }

  describe 'validations' do
    subject(:the_stream) { stream }

    it { is_expected.not_to accept_values_for(:title, '') }
    it { is_expected.not_to accept_values_for(:provider, nil) }
    it { is_expected.not_to accept_values_for(:provider_video_id, '', nil) }

    describe 'multiple streams' do
      context 'with same provider_video_id' do
        let(:provider_video_id) { '1234567' }
        let(:params) { super().merge(provider_video_id:) }
        let!(:existing_stream) { create(:stream, provider_video_id:) }

        context 'for same provider' do
          let(:params) { super().merge(provider_id: existing_stream.provider_id) }

          it { is_expected.not_to be_valid }
        end

        context 'for different provider' do
          it { is_expected.to be_valid }
        end
      end
    end

    describe 'download URLs' do
      let(:stream) { build(:stream, params) }

      context 'one missing each' do
        context 'hd missing' do
          let(:params) { super().merge! hd_url: '' }

          it { is_expected.to be_valid }
        end

        context 'sd missing' do
          let(:params) { super().merge! sd_url: '' }

          it { is_expected.to be_valid }
        end
      end

      context 'both missing' do
        let(:params) { super().merge! sd_url: '', hd_url: '' }

        it { is_expected.not_to be_valid }
      end
    end
  end

  describe '#update_video_data' do
    let(:stream) { create(:stream, attributes) }
    let(:attributes) { {} }

    before do
      create(:video, pip_stream: stream)
      ExtractAudioWorker.jobs.clear
    end

    it 'is triggered when saving a stream' do
      expect(stream).to receive(:update_video_data)
      stream.save!
    end

    shared_examples 'stream change' do
      before do
        # rubocop:disable RSpec/AnyInstance
        allow_any_instance_of(Video).to \
          receive(:trigger_slide_thumbnail_creation)
        # rubocop:enable RSpec/AnyInstance
      end

      context 'without previously generated audio file' do
        let(:attributes) { {audio_uri: nil} }

        it 'does not queue audio reextraction' do
          expect(ExtractAudioWorker.jobs.size).to eq 0
          stream.update!(**params)
          expect(ExtractAudioWorker.jobs.size).to eq 0
        end
      end

      it 'invokes slide thumbnail creation' do
        # rubocop:disable RSpec/AnyInstance
        expect_any_instance_of(Video).to \
          receive(:trigger_slide_thumbnail_creation).with(force: true)
        # rubocop:enable RSpec/AnyInstance

        stream.update!(**params)
      end
    end

    context 'with changed SD checksum' do
      let(:params) { {sd_md5: '239b99869a742f9677a50600725e4c8e'} }

      include_examples 'stream change'

      context 'with previously generated audio file' do
        let(:attributes) { {audio_uri: 's3://xikolo-video/streams/1/audio_v1.mp3'} }

        it 'queues audio reextraction' do
          expect(ExtractAudioWorker.jobs.size).to eq 0
          stream.update!(**params)
          expect(ExtractAudioWorker.jobs.size).to eq 1
          expect(ExtractAudioWorker.jobs[0]['args']).to eq [stream.id]
        end
      end
    end

    context 'with changed HD checksum' do
      let(:params) { {hd_md5: '239b99869a742f9677a50600725e4c8e'} }

      include_examples 'stream change'
    end

    context 'with changed HLS checksum' do
      let(:params) { {hls_md5: '239b99869a742f9677a50600725e4c8e'} }

      include_examples 'stream change'
    end
  end

  describe '#sync' do
    subject(:sync_stream) { stream.sync }

    let(:provider_video_id) { '207611533' }
    let(:provider) { create(:provider) }
    let!(:stream) { create(:stream, provider:, provider_video_id:) }

    let(:vimeo_url) do
      "https://api.vimeo.com/videos/#{provider_video_id}?fields=uri,name,duration,width,height,status,modified_time,pictures.sizes.link,pictures.sizes.width,files.quality,files.size,files.md5,files.link,download"
    end

    let!(:vimeo_request) do
      stub_request(:get, vimeo_url)
        .with(headers: {
          'Authorization' => "Bearer #{provider.token}",
          'Accept' => 'application/vnd.vimeo.*+json;version=3.4',
        })
        .to_return(body: vimeo_response)
    end

    let(:vimeo_response) do
      File.read 'spec/support/vimeo_api_responses/videos_id/full_response.json'
    end

    it 'calls the Vimeo API' do
      sync_stream
      expect(vimeo_request).to have_been_requested.once
    end

    it 'does not store a new stream' do
      expect { sync_stream }.not_to change(Stream, :count)
    end

    it 'updates the stream data' do
      sync_stream
      stream.reload

      # See full_response.json for new data
      expect(stream.title).to eq 'nanou-webtech2017-w1-1-pip-p1'
      expect(stream.poster).to eq 'https://i.vimeocdn.com/video/622733127_1920x1080.jpg?r=pad'
    end
  end

  describe '#current_downloads' do
    context 'for a Kaltura provider' do
      let(:provider) { create(:provider, :kaltura) }
      let(:params) { super().merge provider_id: provider.id }

      it 'returns the stored download links' do
        expect(stream.current_downloads[:hd]).to eq stream.hd_download_url
        expect(stream.current_downloads[:sd]).to eq stream.sd_download_url
      end
    end

    context 'for a Vimeo provider' do
      let(:expiration_date) { 12.hours.from_now.iso8601.to_s }
      let!(:vimeo_api_stub) do
        stub_request(:get, "https://api.vimeo.com/videos/#{stream.provider_video_id}?fields=download")
          .to_return body:
            {
              download: [
                {
                  quality: 'hd',
                  size: '10000',
                  link: "https://player.vimeo.com/progressive_redirect/download/#{stream.provider_video_id}/hd_stream.mp4",
                  expires: expiration_date,
                },
                {
                  quality: 'sd',
                  size: '5000',
                  link: "https://player.vimeo.com/progressive_redirect/download/#{stream.provider_video_id}/sd_stream.mp4",
                  expires: expiration_date,
                },
                {
                  quality: 'sd',
                  size: '2000',
                  link: "https://player.vimeo.com/progressive_redirect/download/#{stream.provider_video_id}/other_sd_stream.mp4",
                  expires: expiration_date,
                },
              ],
            }.to_json
      end

      context 'for non-expired download links' do
        let(:expire_at) { 10.minutes.from_now }

        it 'does not refresh the download links' do
          expect(vimeo_api_stub).not_to have_been_requested
        end

        it 'returns the stored download links' do
          expect(stream.current_downloads[:hd]).to eq stream.hd_download_url
          expect(stream.current_downloads[:sd]).to eq stream.sd_download_url
        end
      end

      context 'for a stream without expiration date stored' do
        it 'refreshes the download links in the database' do
          stream.current_downloads

          expect(vimeo_api_stub).to have_been_requested
          expect(stream.downloads_expire).to eq expiration_date
        end

        it 'returns the refreshed download links' do
          expect(stream.current_downloads[:hd]).to eq "https://player.vimeo.com/progressive_redirect/download/#{stream.provider_video_id}/hd_stream.mp4"
          expect(stream.current_downloads[:sd]).to eq "https://player.vimeo.com/progressive_redirect/download/#{stream.provider_video_id}/sd_stream.mp4"
        end
      end

      context 'for expired download links' do
        let(:expire_at) { 5.minutes.before }

        it 'refreshes the download links in the database' do
          stream.current_downloads

          expect(vimeo_api_stub).to have_been_requested
          expect(stream.downloads_expire).to eq expiration_date
        end

        it 'returns the refreshed download links' do
          expect(stream.current_downloads[:hd]).to eq "https://player.vimeo.com/progressive_redirect/download/#{stream.provider_video_id}/hd_stream.mp4"
          expect(stream.current_downloads[:sd]).to eq "https://player.vimeo.com/progressive_redirect/download/#{stream.provider_video_id}/sd_stream.mp4"
        end
      end

      context 'with failing Vimeo API' do
        let(:vimeo_api_stub) do
          stub_request(:get, "https://api.vimeo.com/videos/#{stream.provider_video_id}?fields=download")
            .to_timeout
        end

        it 'raises an exception' do
          expect { stream.current_downloads }.to raise_exception Vimeo::API::RequestTimeout
        end
      end
    end
  end
end
