# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Subtitle, type: :model do
  let(:video)    { create(:video) }
  let(:params)   { attributes_for(:subtitle).merge video: }
  let(:subtitle) { build(:subtitle, params) }

  describe 'validations' do
    subject(:the_sub) { subtitle }

    it { is_expected.not_to accept_values_for(:video, nil) }
    it { is_expected.not_to accept_values_for(:lang, '') }
  end

  context 'double video_id and language' do
    before { described_class.create params }

    describe 'creation fails' do
      subject { described_class.create params }

      it { is_expected.not_to be_valid }
    end
  end

  describe 'extract!' do
    subject(:extracted_sub) { described_class.extract!(object, video) }

    let(:tempfile) { Rails.root.join('spec/support/files/video-subtitles-temp.zip') }
    let(:url) { 'http://s3.xikolo.de/xikolo-uploads/uploads/1/subtiles.zip' }
    let(:object) { Xikolo::S3.object('s3://xikolo-uploads/uploads/1/subtiles.zip') }

    before do
      stub_request(:get, url).to_return(
        body: File.new('spec/support/files/video-subtitles.zip'),
        status: 200
      )
    end

    it 'creates 2 subtitles and attaches them to the video' do
      expect { extracted_sub }.to change { video.reload.subtitles.count }.from(0).to(2)
    end

    it 'creates the correct number of subtitle cues' do
      expect { extracted_sub }.to change(SubtitleCue, :count).from(0).to(16)
    end
  end

  describe 'attach!' do
    subject(:attach_subtitle) { described_class.attach! src_vtt, 'en', video, automatic: false }

    let(:src_vtt) { File.binread('spec/support/files/video-subtitles-en.vtt') }

    it 'creates 1 subtitle and attaches it to the video' do
      expect { attach_subtitle }.to change { video.reload.subtitles.count }.from(0).to(1)
    end

    it 'does not save the subtitle as auto-generated' do
      expect { attach_subtitle }.to change {
        video.reload.subtitles.where(automatic: false).count
      }.from(0).to(1)
    end

    context 'with automatic attribute' do
      subject(:attach_automatic_subtitle) { described_class.attach! src_vtt, 'en', video, automatic: true }

      it 'saves the subtitle as auto-generated' do
        expect { attach_automatic_subtitle }.to change {
          video.reload.subtitles.where(automatic: true).count
        }.from(0).to(1)
      end
    end

    context '(sync subtitle worker)' do
      before { Sidekiq::Worker.clear_all }

      it 'starts the worker when creating subtitle' do
        expect { attach_subtitle }.to change(SyncSubtitleWorker.jobs, :size).from(0).to(1)
      end
    end

    context 'with existing subtitle' do
      # Create initial subtitle (with 8 cues)
      before { described_class.attach! src_vtt, 'en', video, automatic: false }

      it 'reuses the existing subtitle' do
        expect { attach_subtitle }.not_to change { video.reload.subtitles.count }
      end

      it '(re)creates the correct number of subtitle cues' do
        expect { attach_subtitle }.not_to change(SubtitleCue, :count)
      end

      context 'with existing subtitle cues' do
        before do
          # Create additional cue for same subtitle
          video.subtitles.first.cues.create! identifier: 9,
            start: 32.seconds,
            stop: 34.seconds,
            text: 'Additional cue text.'
        end

        it '(re)creates the correct number of subtitle cues (updates the subtitle)' do
          expect { attach_subtitle }.to change(SubtitleCue, :count).from(9).to(8)
        end
      end
    end

    context 'with empty subtitles' do
      subject(:attach_empty_subtitle) { described_class.attach! src_vtt_empty, 'en', video, automatic: false }

      let(:src_vtt_empty) { File.binread('spec/support/files/video-subtitles-empty-en.vtt') }

      before { described_class.attach! src_vtt, 'en', video, automatic: false }

      it 'destroys the subtitle' do
        expect { attach_empty_subtitle }.to change(SubtitleCue, :count).from(8).to(0)
      end

      it 'destroys the respective cues' do
        expect { attach_empty_subtitle }.to change(described_class, :count).from(1).to(0)
      end
    end
  end

  describe 'extract_lang' do
    subject(:lang) { described_class.extract_lang filename }

    let(:filename) { 'subtitle-en.vtt' }

    it 'extracts the correct language' do
      expect(lang).to eq 'en'
    end

    context 'with invalid filename' do
      let(:filename) { 'subtitle.vtt' }

      it 'returns false' do
        expect(lang).to be_falsey
      end
    end

    context 'with OS X archive metadata files' do
      let(:filename) { '__MACOSX_subtitle-en.vtt' }

      it 'returns false' do
        expect(lang).to be_falsey
      end
    end

    context 'with 3-letter language code' do
      let(:filename) { 'subtitle-tet.vtt' }

      it 'extracts the correct language' do
        expect(lang).to eq 'tet'
      end
    end
  end

  describe 'clone' do
    subject(:clone_sub) { subtitle.clone(video_id: new_video.id) }

    let(:subtitle) { create(:subtitle) }
    let(:cue) { create_list(:subtitle_cue, 2, subtitle:) }
    let(:new_video) { create(:video) }

    before { subtitle; cue }

    it 'clones the subtitle object' do
      expect { clone_sub }.to change(described_class, :count).from(1).to(2)
      expect(clone_sub.video_id).to eq(new_video.id)
      expect(clone_sub.id).not_to eq(subtitle.id)
      expect(clone_sub.attributes.except('id', 'video_id')).to \
        eq(subtitle.attributes.except('id', 'video_id'))
    end

    it 'adds two new subtitle cues' do
      expect { clone_sub }.to change(SubtitleCue, :count).from(2).to(4)
    end

    it 'clones the subtitles cues' do
      expect { clone_sub }.to change {
        SubtitleCue.where.not(subtitle_id: subtitle.id).count
      }.from(0).to(2)
    end
  end

  describe 'create_cues!' do
    subject(:create_cues) { subtitle.create_cues!(blob) }

    let(:subtitle) { create(:subtitle) }
    let(:blob) { file.read }

    context 'when no file file is provided' do
      let(:blob) { nil }

      it { expect(create_cues).to be_nil }
    end

    context 'with a valid vtt file' do
      let(:file) { Rails.root.join('spec/support/files/subtitles/valid_en.vtt').open }

      it { expect { create_cues }.to change { subtitle.reload.cues.count }.from(0).to(3) }
    end

    context 'with a vtt file containing blank spaces' do
      let(:file) { Rails.root.join('spec/support/files/subtitles/with_blank_spaces_en.vtt').open }

      it { expect { create_cues }.to change { subtitle.reload.cues.count }.from(0).to(3) }
    end

    context 'with an invalid vtt file' do
      let(:file) { Rails.root.join('spec/support/files/subtitles/multiple_invalid_en.vtt').open }

      it 'raises an error and does not create subtitle cues' do
        expect { create_cues }.to raise_error(InvalidSubtitleError) do |error|
          expect(error.message).to eq 'invalid_subtitle'
          expect(error.identifiers).to eq [1, 2]
          expect(subtitle.reload.cues.count).to be_zero
        end
      end
    end
  end
end
