# frozen_string_literal: true

require 'spec_helper'

RSpec.describe Video, type: :model do
  subject(:video) { create(:video) }

  let(:stream) { create(:stream) }

  before do
    stub_request(:get, 'http://player.vimeo.com/external/73209171.hd.mp4?s=9d2233269d6cbd749c78c80274f09387')
      .with(headers: {'User-Agent' => 'Typhoeus - https://github.com/typhoeus/typhoeus'})
      .to_return(status: 200, body: '', headers: {})
    stub_request(:post, 'http://fb10mas01:81/api/1.0/lecture-video-ocr/')
      .with(body: 'callback=https%3A%2F%2Flocalhost%3A3000%2Fvideos%2Fthumbnail_api_callback%3Fvideo_id%3D5ac36bdb-8a64-4314-bc07-7e56ddfbbb63%26checksum%3D0e1acbf0319fad7c1aedf1445909e3649d728ddbaf11792fe902b66bacce6629&content=this%20is%20my%20test&lang=en&pdf&title=test%20post&video=%23%3CTyphoeus%3A%3AResponse%3A0x007fa4822530b8%3E',
        headers: {'Content-Type' => 'multipart/form-data', 'User-Agent' => 'Typhoeus - https://github.com/typhoeus/typhoeus', 'X-Api-Key' => '73240c3206abc1356e952246375e16f3'})
      .to_return(status: 200, body: '', headers: {})
  end

  it { is_expected.not_to accept_values_for(:title, '') }
  it { is_expected.to accept_values_for(:description, '') }
  it { is_expected.to accept_values_for(:lecturer_stream, nil) }
  it { is_expected.to accept_values_for(:slides_stream, nil) }
  it { is_expected.to accept_values_for(:pip_stream, nil) }
  it { is_expected.to accept_values_for(:subtitled_stream, nil) }

  it 'fails validation if streams do not exist' do
    invalid_record = build(:video, :without_streams)
    invalid_record.lecturer_stream_id = '00000000-0000-4444-0000-000000000000'
    invalid_record.slides_stream_id = '00000000-0000-4444-0000-000000000000'
    invalid_record.pip_stream_id = '00000000-0000-4444-0000-000000000000'
    invalid_record.subtitled_stream_id = '00000000-0000-4444-0000-000000000000'
    invalid_record.valid?
    expect(invalid_record.errors.size).to eq 4
  end

  shared_examples 'a change that affects the audio' do
    before do
      Xikolo.config.audio_extraction = true
    end

    it 'schedules an audio extraction job' do
      expect { subject }.to change { ExtractAudioWorker.jobs.size }.by(1)
    end

    it 'selects the correct stream for audio extraction' do
      subject
      expect(ExtractAudioWorker.jobs.last['args']).to eq [extraction_stream_id]
    end
  end

  shared_examples 'a change that does not affect the audio' do
    before do
      Xikolo.config.audio_extraction = true
    end

    it 'does not schedule an audio extraction job' do
      expect { subject }.not_to change { ExtractAudioWorker.jobs.size }
    end
  end

  describe '#create' do
    subject { described_class.create attributes_for(:video).merge(video_attributes) }

    let(:pip_stream) { create(:stream) }
    let(:lecturer_stream) { create(:stream) }
    let(:slides_stream) { create(:stream) }

    context 'with pip stream' do
      let(:video_attributes) { {pip_stream_id: pip_stream.id} }

      it_behaves_like 'a change that affects the audio' do
        let(:extraction_stream_id) { pip_stream.id }
      end
    end

    context 'with lecturer stream' do
      let(:video_attributes) { {lecturer_stream_id: lecturer_stream.id} }

      it_behaves_like 'a change that affects the audio' do
        let(:extraction_stream_id) { lecturer_stream.id }
      end
    end

    context 'with pip and lecturer stream' do
      let(:video_attributes) do
        {
          pip_stream_id: pip_stream.id,
          lecturer_stream_id: lecturer_stream.id,
        }
      end

      it_behaves_like 'a change that affects the audio' do
        let(:extraction_stream_id) { pip_stream.id }
      end
    end

    context 'only with slides stream' do
      let(:video_attributes) { {slides_stream_id: slides_stream.id} }

      it_behaves_like 'a change that does not affect the audio'
    end
  end

  describe '#update' do
    subject { described_class.update video.id, video_attributes }

    let(:pip_stream) { create(:stream) }
    let(:lecturer_stream) { create(:stream) }
    let(:slides_stream) { create(:stream) }

    # Load the video once so that our test cases ignore jobs scheduled by its creation
    before { video }

    context 'changing the pip stream' do
      let(:video_attributes) { {pip_stream_id: pip_stream.id} }

      it_behaves_like 'a change that affects the audio' do
        let(:extraction_stream_id) { pip_stream.id }
      end
    end

    context 'changing the pip stream to nil (delete)' do
      let(:video_attributes) { {pip_stream_id: nil} }

      it_behaves_like 'a change that affects the audio' do
        let(:extraction_stream_id) { video.lecturer_stream_id }
      end
    end

    context 'changing the lecturer stream' do
      let(:video_attributes) { {lecturer_stream_id: lecturer_stream.id} }

      it_behaves_like 'a change that does not affect the audio'

      context 'when there is no pip stream' do
        before { video.update_column :pip_stream_id, nil }

        it_behaves_like 'a change that affects the audio' do
          let(:extraction_stream_id) { lecturer_stream.id }
        end
      end
    end

    context 'changing the lecturer stream to nil (delete)' do
      let(:video_attributes) { {lecturer_stream_id: nil} }

      it_behaves_like 'a change that does not affect the audio'
    end

    context 'changing the slides stream' do
      let(:video_attributes) { {slides_stream_id: slides_stream.id} }

      it_behaves_like 'a change that does not affect the audio'
    end
  end

  describe 'thumbnail' do
    subject(:thumbnail) { video.thumbnail }

    let(:stream) { create(:stream, :with_poster) }

    context 'with pip stream poster' do
      let(:video) { create(:video, :without_streams, pip_stream: stream) }

      it { is_expected.to eq stream.poster }
    end

    context 'with lecturer stream poster' do
      let(:video) { create(:video, :without_streams, lecturer_stream: stream) }

      it { is_expected.to eq stream.poster }
    end

    context 'with slides stream poster' do
      let(:video) { create(:video, :without_streams, slides_stream: stream) }

      it { is_expected.to eq stream.poster }
    end
  end

  describe 'duration' do
    subject(:duration) { video.duration }

    let(:stream) { create(:stream, :from_api) }

    context 'with pip stream poster' do
      let(:video) { create(:video, :without_streams, pip_stream: stream) }

      it { is_expected.to eq stream.duration }
    end

    context 'with lecturer stream poster' do
      let(:video) { create(:video, :without_streams, lecturer_stream: stream) }

      it { is_expected.to eq stream.duration }
    end

    context 'with slides stream poster' do
      let(:video) { create(:video, :without_streams, slides_stream: stream) }

      it { is_expected.to eq stream.duration }
    end

    context 'with lecturer and slides stream poster' do
      let(:longer_stream) { create(:stream, :longer_duration) }
      let(:video) do
        create(:video, :without_streams,
          lecturer_stream: longer_stream,
          slides_stream: stream)
      end

      it { is_expected.to eq longer_stream.duration }
    end
  end

  describe '#trigger_subtitle_sync' do
    subject(:trigger) { video.trigger_subtitle_sync('en') }

    it 'schedules a worker to do the synchronization' do
      expect { trigger }.to change(SyncSubtitleWorker.jobs, :size).from(0).to(1)
    end
  end
end
