# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Download: Show', type: :request do
  subject(:show) { api.rel(:download).get(id: stream.id, quality:).value! }

  let(:expire_at) { '' }
  let(:provider) { create(:provider) }
  let(:hd_url) { 'https://player.vimeo.com/progressive_redirect/download/137864042/initial_hd_stream.mp4' }
  let(:stream) { create(:stream, :from_api, provider:, hd_download_url: hd_url, downloads_expire: expire_at) }
  let(:quality) { 'hd' }
  let(:api) { Restify.new(:test).get.value }

  describe '(download links)' do
    context 'for a Kaltura provider' do
      let(:provider) { create(:provider, :kaltura) }

      it { is_expected.to respond_with :ok }

      it 'matches the stored stream download url' do
        expect(show['link']).to eq stream.hd_download_url
      end
    end

    context 'for a Vimeo provider' do
      let(:expiration_date) { 12.hours.from_now.iso8601.to_s }
      let(:new_hd_url) { 'https://player.vimeo.com/progressive_redirect/download/137864042/hd_stream.mp4' }
      let!(:vimeo_api_stub) do
        stub_request(:get, 'https://api.vimeo.com/videos/137864042?fields=download')
          .to_return body:
            {
              download: [
                {
                  quality: 'hd',
                  size: '10000',
                  link: new_hd_url,
                  expires: expiration_date,
                },
                {
                  quality: 'sd',
                  size: '5000',
                  link: 'https://player.vimeo.com/progressive_redirect/download/137864042/sd_stream.mp4',
                  expires: expiration_date,
                },
                {
                  quality: 'sd',
                  size: '2000',
                  link: 'https://player.vimeo.com/progressive_redirect/download/137864042/other_sd_stream.mp4',
                  expires: expiration_date,
                },
              ],
            }.to_json
      end

      context 'for non-expired download links' do
        let(:expire_at) { 10.minutes.from_now }

        it { is_expected.to respond_with :ok }

        it 'does not refresh the download links' do
          expect(vimeo_api_stub).not_to have_been_requested
        end

        it 'matches the stored stream download url' do
          expect(show['link']).to eq stream.hd_download_url
        end
      end

      context 'for a stream without expiration date stored' do
        it { is_expected.to respond_with :ok }

        it 'refreshes the download links in the database' do
          show
          expect(vimeo_api_stub).to have_been_requested

          stream.reload
          expect(stream.hd_download_url).to eq 'https://player.vimeo.com/progressive_redirect/download/137864042/hd_stream.mp4'
          expect(stream.sd_download_url).to eq 'https://player.vimeo.com/progressive_redirect/download/137864042/sd_stream.mp4'
          expect(stream.downloads_expire).to eq expiration_date
        end

        it 'matches the new download links' do
          expect(show['link']).to eq 'https://player.vimeo.com/progressive_redirect/download/137864042/hd_stream.mp4'
        end
      end

      context 'for expired download links' do
        let(:expire_at) { 5.minutes.before }

        it { is_expected.to respond_with :ok }

        it 'refreshes the download links in the database' do
          show
          expect(vimeo_api_stub).to have_been_requested

          stream.reload
          expect(stream.hd_download_url).to eq 'https://player.vimeo.com/progressive_redirect/download/137864042/hd_stream.mp4'
          expect(stream.sd_download_url).to eq 'https://player.vimeo.com/progressive_redirect/download/137864042/sd_stream.mp4'
          expect(stream.downloads_expire).to eq expiration_date
        end

        it 'matches the new download links' do
          expect(show['link']).to eq 'https://player.vimeo.com/progressive_redirect/download/137864042/hd_stream.mp4'
        end

        context 'with failing Vimeo API' do
          subject(:request) { get "/downloads/#{stream.id}?quality=hd" }

          let(:vimeo_api_stub) do
            stub_request(:get, 'https://api.vimeo.com/videos/137864042?fields=download')
              .to_timeout
          end

          before { vimeo_api_stub; request }

          it 'responds with HTTP 504 Gateway Timeout' do
            expect(response).to have_http_status :gateway_timeout

            json = JSON.parse response.body
            expect(json['error']).to eq 'vimeo_api_did_not_respond'
          end
        end
      end

      context 'with unavailable quality' do
        subject(:request) { get "/downloads/#{stream.id}?quality=hd" }

        let(:hd_url) { nil }
        let(:new_hd_url) { nil }

        before { vimeo_api_stub; request }

        it 'responds with 404 Not Found' do
          expect(response).to have_http_status :not_found

          json = JSON.parse response.body
          expect(json['error']).to eq 'quality_not_available'
        end
      end

      context 'with unsupported quality' do
        subject(:request) { get "/downloads/#{stream.id}?quality=super" }

        before { vimeo_api_stub; request }

        it 'responds with HTTP 422 Unprocessable Entity' do
          expect(response).to have_http_status :unprocessable_entity

          json = JSON.parse response.body
          expect(json['error']).to eq 'quality_not_supported'
        end
      end
    end
  end
end
