# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Video: Clone', type: :request do
  subject(:clone) { video_api.rel(:clone).post.value! }

  let(:video) { create(:video) }

  let(:api) { Restify.new(:test).get.value }
  let(:video_api) { api.rel(:video).get(id: video.id).value! }
  let(:new_video) { Video.find clone['id'] }

  it 'creates the cloned video successfully' do
    expect(clone).to respond_with(:created)
  end

  it 'returns a matching video id expect for the video id' do
    expect(clone.to_h).to include(
      video_api.slice('title', 'description', 'lecturer_stream_id', 'slides_stream_id', 'pip_stream_id', 'thumbnail', 'duration', 'subtitled_stream_id', 'subtitles')
    )
  end

  it 'copies subtitles' do
    create(:subtitle, video:)
    expect { clone }.to change(Subtitle, :count).from(1).to(2)
  end

  it 'does not copy thumbnails, but instead triggers thumbnail generation for the clone' do
    create(:thumbnail, video:)

    # This is triggered multiple times, due to implementation details.
    # For the purpose of the test, "at least once"" is enough.
    expect { clone }.to change { ThumbnailGenerationWorker.jobs.size }.by(1)

    expect(new_video.thumbnails.count).to eq 0
    expect(new_video.thumbnails_uri).to be_nil
  end

  # filename_v1.pdf
  context 'with old file versioning schema' do
    let(:video_id) { SecureRandom.uuid }

    context 'with slides attached' do
      let(:video) { create(:video, :with_slides, id: video_id, slides_uri: "s3://xikolo-video/videos/#{video_id}/slides_v1.pdf") }
      let!(:copy_slides_stub) do
        stub_request(
          :put, %r{http://s3.xikolo.de/xikolo-video/videos/[a-zA-Z0-9]+/slides_v1.pdf}
        ).and_return(
          status: 200,
          headers: {'Content-Disposition' => 'attachment; filename="slides.pdf"'},
          body: '<xml></xml>'
        )
      end

      before do
        stub_request(:head, "http://s3.xikolo.de/xikolo-video/videos/#{video_id}/slides_v1.pdf")
          .to_return(status: 200, headers: {'Content-Disposition' => 'attachment; filename="slides.pdf"'})
      end

      it 'clones the slides with its content disposition' do
        clone
        expect(copy_slides_stub).to have_been_requested
        expect(new_video.slides_uri).to start_with('s3://xikolo-video/videos/')
      end

      it 'updates the associated file' do
        clone
        expect(new_video.slides_uri).not_to eq video.slides_uri
        expect(new_video.slides_uri).to eq "s3://xikolo-video/videos/#{UUID4(new_video.id).to_s(format: :base62)}/slides_v1.pdf"
      end
    end

    context 'with a transcript attached' do
      let(:video) { create(:video, :with_transcript, id: video_id, transcript_uri: "s3://xikolo-video/videos/#{video_id}/transcript_v1.pdf") }
      let!(:copy_transcript_stub) do
        stub_request(
          :put, %r{http://s3.xikolo.de/xikolo-video/videos/[a-zA-Z0-9]+/transcript_v1.pdf}
        ).and_return(
          status: 200,
          headers: {'Content-Disposition' => 'attachment; filename="transcript.pdf"'},
          body: '<xml></xml>'
        )
      end

      before do
        stub_request(:head, "http://s3.xikolo.de/xikolo-video/videos/#{video_id}/transcript_v1.pdf")
          .to_return(status: 200, headers: {'Content-Disposition' => 'attachment; filename="transcript.pdf"'})
      end

      it 'clones the transcript with its content disposition' do
        clone
        expect(copy_transcript_stub).to have_been_requested
      end

      it 'updates the associated file' do
        clone
        expect(new_video.transcript_uri).not_to eq video.transcript_uri
        expect(new_video.transcript_uri).to eq "s3://xikolo-video/videos/#{UUID4(new_video.id).to_s(format: :base62)}/transcript_v1.pdf"
      end
    end

    context 'with reading material attached' do
      let(:video) { create(:video, :with_reading_material, id: video_id, reading_material_uri: "s3://xikolo-video/videos/#{video_id}/reading_material_v1.pdf") }
      let!(:copy_reading_material_stub) do
        stub_request(
          :put, %r{http://s3.xikolo.de/xikolo-video/videos/[a-zA-Z0-9]+/reading_material_v1.pdf}
        ).and_return(
          status: 200,
          headers: {'Content-Disposition' => 'attachment; filename="reading_material.pdf"'},
          body: '<xml></xml>'
        )
      end

      before do
        stub_request(:head, "http://s3.xikolo.de/xikolo-video/videos/#{video.id}/reading_material_v1.pdf")
          .to_return(status: 200, headers: {'Content-Disposition' => 'attachment; filename="reading_material.pdf"'})
      end

      it 'clones the reading material with its content disposition' do
        clone
        expect(copy_reading_material_stub).to have_been_requested
      end

      it 'updates the associated file' do
        clone
        expect(new_video.reading_material_uri).not_to eq video.reading_material_uri
        expect(new_video.reading_material_uri).to eq "s3://xikolo-video/videos/#{UUID4(new_video.id).to_s(format: :base62)}/reading_material_v1.pdf"
      end
    end
  end

  # unique_sanitized_name(filename).pdf --> uuid/filename.pdf
  context 'with new file versioning schema' do
    context 'with slides attached' do
      let!(:video) { create(:video, :with_slides) }
      let!(:copy_slides_stub) do
        stub_request(
          :put, %r{http://s3.xikolo.de/xikolo-video/videos/[a-zA-z0-9]+/encodedUUUID/slides.pdf}
        ).and_return(status: 200,
          headers: {'Content-Disposition' => 'attachment; filename="slides.pdf"'},
          body: '<xml></xml>')
      end

      before do
        stub_request(:head, "http://s3.xikolo.de/xikolo-video/videos/#{video.id}/encodedUUUID/slides.pdf")
          .to_return(status: 200, headers: {'Content-Disposition' => 'attachment; filename="slides.pdf"'})
      end

      it 'clones the slides with its content disposition' do
        clone
        expect(copy_slides_stub).to have_been_requested
      end

      it 'updates the associated file' do
        clone
        expect(new_video.slides_uri).not_to eq video.slides_uri
        expect(new_video.slides_uri).to eq "s3://xikolo-video/videos/#{UUID4(new_video.id).to_s(format: :base62)}/encodedUUUID/slides.pdf"
      end
    end

    context 'with a transcript attached' do
      let(:video) { create(:video, :with_transcript) }
      let!(:copy_transcript_stub) do
        stub_request(
          :put, %r{http://s3.xikolo.de/xikolo-video/videos/[a-zA-Z0-9]+/encodedUUUID/transcript.pdf}
        ).and_return(status: 200,
          headers: {'Content-Disposition' => 'attachment; filename="transcript.pdf"'},
          body: '<xml></xml>')
      end

      before do
        stub_request(:head, "http://s3.xikolo.de/xikolo-video/videos/#{video.id}/encodedUUUID/transcript.pdf")
          .to_return(status: 200, headers: {'Content-Disposition' => 'attachment; filename="transcript.pdf"'})
      end

      it 'clones the transcript with its content disposition' do
        clone
        expect(copy_transcript_stub).to have_been_requested
      end

      it 'updates the associated file' do
        clone
        expect(new_video.transcript_uri).not_to eq video.transcript_uri
        expect(new_video.transcript_uri).to eq "s3://xikolo-video/videos/#{UUID4(new_video.id).to_s(format: :base62)}/encodedUUUID/transcript.pdf"
      end
    end

    context 'with reading material attached' do
      let(:video) { create(:video, :with_reading_material) }
      let!(:copy_reading_material_stub) do
        stub_request(
          :put, %r{http://s3.xikolo.de/xikolo-video/videos/[a-zA-Z0-9]+/encodedUUUID/reading_material.pdf}
        ).and_return(status: 200,
          headers: {'Content-Disposition' => 'attachment; filename="reading_material.pdf"'},
          body: '<xml></xml>')
      end

      before do
        stub_request(:head, "http://s3.xikolo.de/xikolo-video/videos/#{video.id}/encodedUUUID/reading_material.pdf")
          .to_return(status: 200, headers: {'Content-Disposition' => 'attachment; filename="reading_material.pdf"'})
      end

      it 'clones the reading material with its content disposition' do
        clone
        expect(copy_reading_material_stub).to have_been_requested
      end

      it 'updates the associated file' do
        clone
        expect(new_video.reading_material_uri).not_to eq video.reading_material_uri
        expect(new_video.reading_material_uri).to eq "s3://xikolo-video/videos/#{UUID4(new_video.id).to_s(format: :base62)}/encodedUUUID/reading_material.pdf"
      end
    end
  end
end
