# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Video: Create with description', type: :request do
  subject(:create_video) { api.rel(:videos).post(params).value! }

  let(:params) do
    {
      title: 'A nice video!',
      pip_stream_id: create(:stream).id,
      description: markup,
    }
  end
  let(:markup) { 'Some Text' }
  let(:api) { Restify.new(:test).get.value }

  it { is_expected.to respond_with :created }

  it 'creates a new result object' do
    expect { create_video }.to change(Video, :count).from(0).to(1)
  end

  context 'with file upload references' do
    let(:markup) { 'upload://b5f99337-224f-40f5-aa82-44ee8b272579/foo.jpg' }

    it 'stores valid upload and creates a new richtext' do
      stub_request(
        :head,
        'http://s3.xikolo.de/xikolo-uploads/' \
        'uploads/b5f99337-224f-40f5-aa82-44ee8b272579/foo.jpg'
      ).and_return(
        status: 200,
        headers: {
          'X-Amz-Meta-Xikolo-Purpose' => 'video_description',
          'X-Amz-Meta-Xikolo-State' => 'accepted',
        }
      )
      store_regex = %r{http://s3.xikolo.de/xikolo-video
                       /videos/[0-9a-zA-Z]+/rtfiles/[0-9a-zA-Z]+/foo.jpg}x
      stub_request(:head, store_regex).and_return(status: 404)
      store = stub_request(:put, store_regex).and_return(status: 200, body: '<xml></xml>')
      expect { create_video }.to change(Video, :count).from(0).to(1)
      expect(store).to have_been_requested
      expect(Video.last.description).to include 's3://xikolo-video/videos'
    end

    it 'rejects invalid upload and does not creates a new page' do
      stub_request(
        :head,
        'http://s3.xikolo.de/xikolo-uploads/' \
        'uploads/b5f99337-224f-40f5-aa82-44ee8b272579/foo.jpg'
      ).and_return(
        status: 200,
        headers: {
          'X-Amz-Meta-Xikolo-Purpose' => 'video_description',
          'X-Amz-Meta-Xikolo-State' => 'rejected',
        }
      )

      expect { create_video }.to raise_error(Restify::ClientError) do |error|
        expect(error.status).to eq :unprocessable_entity
        expect(error.errors).to eq 'description' => ['rtfile_rejected']
      end
    end

    it 'rejects upload on storage errors' do
      stub_request(
        :head,
        'http://s3.xikolo.de/xikolo-uploads/' \
        'uploads/b5f99337-224f-40f5-aa82-44ee8b272579/foo.jpg'
      ).and_return(
        status: 200,
        headers: {
          'X-Amz-Meta-Xikolo-Purpose' => 'video_description',
          'X-Amz-Meta-Xikolo-State' => 'accepted',
        }
      )
      store_regex = %r{http://s3.xikolo.de/xikolo-video
                       /videos/[0-9a-zA-Z]+/rtfiles/[0-9a-zA-Z]+/foo.jpg}x
      stub_request(:head, store_regex).and_return(status: 404)
      stub_request(:put, store_regex).and_return(status: 503)

      expect { create_video }.to raise_error(Restify::ClientError) do |error|
        expect(error.status).to eq :unprocessable_entity
        expect(error.errors).to eq 'description' => ['rtfile_error']
      end
    end
  end
end
