# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Video: Create', type: :request do
  subject(:create_video) { api.rel(:videos).post(params).value! }

  let(:params) do
    {
      title: 'A nice video!',
      pip_stream_id: create(:stream).id,
      reading_material_upload_id: upload_id,
    }
  end
  let(:api) { Restify.new(:test).get.value }
  let(:upload_id) { 'f13d30d3-6369-4816-9695-af5318c8ac15' }
  let(:file_url) do
    'http://s3.xikolo.de/xikolo-uploads/' \
      'uploads/f13d30d3-6369-4816-9695-af5318c8ac15/reading_material.pdf'
  end

  before do
    stub_request(:get,
      'http://s3.xikolo.de/xikolo-uploads/?list-type=2&' \
      'prefix=uploads%2Ff13d30d3-6369-4816-9695-af5318c8ac15') \
      .to_return(
        status: 200,
        headers: {'Content-Type' => 'Content-Type: application/xml'},
        body: <<~XML)
          <?xml version="1.0" encoding="UTF-8"?>
          <ListBucketResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
            <Name>xikolo-uploads</Name>
            <Prefix>uploads/f13d30d3-6369-4816-9695-af5318c8ac15</Prefix>
            <IsTruncated>false</IsTruncated>
            <Contents>
              <Key>uploads/f13d30d3-6369-4816-9695-af5318c8ac15/reading_material.pdf</Key>
              <LastModified>2018-08-02T13:27:56.768Z</LastModified>
              <ETag>&#34;d41d8cd98f00b204e9800998ecf8427e&#34;</ETag>
            </Contents>
          </ListBucketResult>
        XML
  end

  context 'with a valid reading material file upload' do
    let!(:store_stub) do
      stub_request(:put, %r{http://s3.xikolo.de/xikolo-video/videos/[0-9a-zA-Z]+/[0-9a-zA-Z]+/reading_material.pdf}) \
        .to_return(
          status: 200,
          headers: {'Content-Type' => 'application/xml', 'Content-Disposition' => 'attachment; filename="reading_material.pdf"'},
          body: <<~XML)
            <CopyObjectResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
              <LastModified>2018-08-02T15:42:36.430Z</LastModified>
              <ETag>&#34;d41d8cd98f00b204e9800998ecf8427e&#34;</ETag>
            </CopyObjectResult>
          XML
    end

    before do
      stub_request(:head, file_url).to_return(
        status: 200,
        headers: {
          'Content-Type' => 'inode/x-empty',
          'X-Amz-Meta-Xikolo-Purpose' => 'video_material',
          'X-Amz-Meta-Xikolo-State' => 'accepted',
        }
      )
      stub_request(:get, file_url).to_return(
        body: File.new('spec/support/files/video-subtitles.zip'),
        status: 200
      )
    end

    it 'creates the video successfully' do
      expect(create_video).to respond_with(:created)
    end

    it 'stores the reading material file in S3' do
      create_video
      expect(store_stub).to have_been_requested
    end
  end

  context 'with an invalid reading material file upload' do
    shared_examples 'does not create the video' do |error_details|
      it 'does not create the video with the reading material uri' do
        expect { create_video }.to raise_error(Restify::ClientError)
      end

      it 'responds with 422 Unprocessable Entity' do
        expect { create_video }.to raise_error(Restify::UnprocessableEntity) do |error|
          expect(error.status).to eq :unprocessable_entity
          expect(error.errors).to eq error_details
        end
      end
    end

    context 'without access permission' do
      before do
        stub_request(:head, file_url).to_return(status: 403)
      end

      error_details = {'reading_material_upload_id' => ['could not process file upload']}
      include_examples 'does not create the video', error_details
    end

    context 'when upload was rejected' do
      before do
        stub_request(:head, file_url).to_return(
          status: 200,
          headers: {
            'Content-Type' => 'inode/x-empty',
            'X-Amz-Meta-Xikolo-Purpose' => 'video_material',
            'X-Amz-Meta-Xikolo-State' => 'rejected',
          }
        )
      end

      error_details = {'reading_material_upload_id' => ['invalid upload']}
      include_examples 'does not create the video', error_details
    end
  end
end
