# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Video: Create', type: :request do
  subject(:create_video) { api.rel(:videos).post(params).value! }

  let(:params) do
    {
      title: 'A nice video!',
      pip_stream_id: create(:stream).id,
    }
  end
  let(:api) { Restify.new(:test).get.value }

  context 'with subtitles' do
    let(:upload_id) { 'f13d30d3-6369-4816-9695-af5318c8ac15' }
    let(:params) { super().merge subtitles_upload_id: upload_id }
    let(:file_name) { 'subtiles' }
    let(:file_url) do
      'http://s3.xikolo.de/xikolo-uploads/' \
      'uploads/f13d30d3-6369-4816-9695-af5318c8ac15/' + file_name
    end

    before do
      stub_request(:get,
        'http://s3.xikolo.de/xikolo-uploads/?list-type=2&' \
        'prefix=uploads%2Ff13d30d3-6369-4816-9695-af5318c8ac15') \
        .to_return(
          status: 200,
          headers: {'Content-Type' => 'Content-Type: application/xml'},
          body: <<~XML)
            <?xml version="1.0" encoding="UTF-8"?>
            <ListBucketResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
              <Name>xikolo-uploads</Name>
              <Prefix>uploads/f13d30d3-6369-4816-9695-af5318c8ac15</Prefix>
              <IsTruncated>false</IsTruncated>
              <Contents>
                <Key>uploads/f13d30d3-6369-4816-9695-af5318c8ac15/#{file_name}</Key>
                <LastModified>2018-08-02T13:27:56.768Z</LastModified>
                <ETag>&#34;d41d8cd98f00b204e9800998ecf8427e&#34;</ETag>
              </Contents>
            </ListBucketResult>
          XML
    end

    context 'with .zip subtitles' do
      let(:file_name) { 'subtitles.zip' }

      before do
        stub_request(:head, file_url).to_return(
          status: 200,
          headers: {
            'Content-Type' => 'inode/x-empty',
            'X-Amz-Meta-Xikolo-Purpose' => 'video_subtitles',
            'X-Amz-Meta-Xikolo-State' => 'accepted',
          }
        )
        stub_request(:get, file_url).to_return(
          body: File.new('spec/support/files/video-subtitles.zip'),
          status: 200
        )
      end

      it 'creates subtitles' do
        expect { create_video }.to change(Subtitle, :count).from(0).to(2)

        expect(SubtitleCue.count).to eq 16
        expect(create_video).to respond_with :created
      end

      it 'triggers synchronization with the video provider for all subtitles' do
        expect { create_video }.to change(SyncSubtitleWorker.jobs, :size).from(0).to(2)
      end
    end

    context 'with .vtt subtitles' do
      let(:file_name) { 'video-subtitles-en.vtt' }

      before do
        stub_request(:head, file_url).to_return(
          status: 200,
          headers: {
            'Content-Type' => 'inode/x-empty',
            'X-Amz-Meta-Xikolo-Purpose' => 'video_subtitles',
            'X-Amz-Meta-Xikolo-State' => 'accepted',
          }
        )
        stub_request(:get, file_url).to_return(
          body: File.new('spec/support/files/video-subtitles-en.vtt'),
          status: 200
        )
      end

      it 'creates subtitles' do
        expect { create_video }.to change(Subtitle, :count).from(0).to(1)
        expect(SubtitleCue.count).to eq 8
        expect(create_video).to respond_with :created
      end

      it 'triggers subtitle synchronization with the video provider for the new subtitle' do
        expect { create_video }.to change(SyncSubtitleWorker.jobs, :size).from(0).to(1)
      end
    end

    it 'handles S3 errors' do
      stub_request(:head, file_url).to_return(
        status: 403
      )
      expect { create_video }.to raise_error(Restify::ClientError) do |error|
        expect(error.status).to eq :unprocessable_entity
        expect(error.errors).to eq 'subtitles_upload_id' => ['could not process file upload']
      end
    end
  end
end
