# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Video: Delete', type: :request do
  subject(:deletion) do
    api.rel(:video).delete(id: video.id).value!.tap do
      FileDeletionWorker.drain
    end
  end

  let(:params) { {} }
  let!(:video) { create(:video, params) }
  let(:api) { Restify.new(:test).get.value }

  it { is_expected.to respond_with :no_content }

  context 'with attached reading material' do
    let(:params) do
      {
        reading_material_uri: 's3://xikolo-video/videos/1/reading_v11.pdf',
      }
    end
    let!(:delete_stub) do
      stub_request(:delete,
        'http://s3.xikolo.de/xikolo-video/videos/1/reading_v11.pdf')
    end

    it 'removes the video' do
      expect { deletion }.to change(Video, :count).from(1).to(0)
    end

    it 'removes the attached file eventually' do
      deletion
      expect(delete_stub).to have_been_requested
    end

    context 'with cloned video' do
      before do
        create(:video, reading_material_uri: video.reading_material_uri)
      end

      it 'does not remove the attached file' do
        deletion
        expect(delete_stub).not_to have_been_requested
      end
    end
  end

  context 'with attached transcript' do
    let(:params) do
      {
        transcript_uri: 's3://xikolo-video/videos/1/transcript_v1.pdf',
      }
    end
    let!(:delete_stub) do
      stub_request(:delete,
        'http://s3.xikolo.de/xikolo-video/videos/1/transcript_v1.pdf')
    end

    it 'removes the video' do
      expect { deletion }.to change(Video, :count).from(1).to(0)
    end

    it 'removes the attached file eventually' do
      deletion
      expect(delete_stub).to have_been_requested
    end

    context 'with cloned video' do
      before do
        create(:video, transcript_uri: video.transcript_uri)
      end

      it 'does not remove the attached file' do
        deletion
        expect(delete_stub).not_to have_been_requested
      end
    end
  end

  context 'with attached slides' do
    let(:params) do
      {
        slides_uri: 's3://xikolo-video/videos/1/slides_v1.pdf',
      }
    end
    let!(:delete_stub) do
      stub_request(:delete,
        'http://s3.xikolo.de/xikolo-video/videos/1/slides_v1.pdf')
    end

    it 'removes the video' do
      expect { deletion }.to change(Video, :count).from(1).to(0)
    end

    it 'removes the attached file eventually' do
      deletion
      expect(delete_stub).to have_been_requested
    end

    context 'with cloned video' do
      before do
        create(:video, slides_uri: video.slides_uri)
      end

      it 'does not remove the attached file' do
        deletion
        expect(delete_stub).not_to have_been_requested
      end
    end
  end
end
