# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Video: Update', type: :request do
  subject(:update_video) { api.rel(:video).patch(update_params, id: video.id).value! }

  let(:video_id) { SecureRandom.uuid }
  let(:reading_material_uri) { "s3://xikolo-video/videos/#{UUID4(video_id).to_s(format: :base62)}/encodedUUUID/reading_material.pdf" }
  let(:initial_params) { {id: video_id, reading_material_uri:} }
  let(:video) { create(:video, initial_params) }
  let(:update_params) { {} }
  let(:api) { Restify.new(:test).get.value }

  context 'with newly uploaded reading material' do
    let(:upload_id) { SecureRandom.uuid }
    let(:update_params) { super().merge reading_material_upload_id: upload_id }
    let(:file_url) do
      'http://s3.xikolo.de/xikolo-uploads/' \
        "uploads/#{upload_id}/new_reading_material.pdf"
    end

    before do
      stub_request(:get,
        'http://s3.xikolo.de/xikolo-uploads/?list-type=2&' \
        "prefix=uploads%2F#{upload_id}") \
        .to_return(
          status: 200,
          headers: {'Content-Type' => 'Content-Type: application/xml'},
          body: <<~XML)
            <?xml version="1.0" encoding="UTF-8"?>
            <ListBucketResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
              <Name>xikolo-uploads</Name>
              <Prefix>uploads/#{upload_id}</Prefix>
              <IsTruncated>false</IsTruncated>
              <Contents>
                <Key>uploads/#{upload_id}/new_reading_material.pdf</Key>
                <LastModified>2018-08-02T13:27:56.768Z</LastModified>
                <ETag>&#34;d41d8cd98f00b204e9800998ecf8427e&#34;</ETag>
              </Contents>
            </ListBucketResult>
          XML
    end

    context 'with a valid reading material file upload' do
      let!(:store_stub) do
        stub_request(:put,
          %r{http://s3.xikolo.de/xikolo-video/videos/[0-9a-zA-Z]+/[0-9a-zA-Z]+/new_reading_material.pdf}) \
          .to_return(
            status: 200,
            headers: {'Content-Type' => 'application/xml'},
            body: <<~XML)
              <CopyObjectResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                <LastModified>2018-08-02T15:42:36.430Z</LastModified>
                <ETag>&#34;d41d8cd98f00b204e9800998ecf8427e&#34;</ETag>
              </CopyObjectResult>
            XML
      end

      before do
        stub_request(:head, file_url).to_return(
          status: 200,
          headers: {
            'Content-Type' => 'inode/x-empty',
            'X-Amz-Meta-Xikolo-Purpose' => 'video_material',
            'X-Amz-Meta-Xikolo-State' => 'accepted',
          }
        )
      end

      it 'updates the video successfully' do
        expect(update_video).to respond_with(:no_content)
      end

      it 'stores the reading material file in S3' do
        update_video
        expect(store_stub).to have_been_requested
      end

      it 'updates the reading_material_uri on the video' do
        update_video
        expect(video.reload.reading_material_uri).not_to eq reading_material_uri
        expect(video.reload.reading_material_uri).to match(%r{s3://xikolo-video/videos/#{UUID4(video_id).to_s(format: :base62)}/[0-9a-zA-Z]+/new_reading_material.pdf})
      end

      it 'deletes the old file' do
        update_video
        expect(FileDeletionWorker.jobs.last['args']).to eq [reading_material_uri]
      end
    end

    context 'with an invalid reading material file upload' do
      shared_examples 'does not update the reading material' do |error_details|
        it 'does not update the video with the reading material uri' do
          expect { update_video }.to raise_error(Restify::ClientError)
          expect(video.reload.reading_material_uri).to match reading_material_uri
        end

        it 'responds with 422 Unprocessable Entity' do
          expect { update_video }.to raise_error(Restify::UnprocessableEntity) do |error|
            expect(error.status).to eq :unprocessable_entity
            expect(error.errors).to eq error_details
          end
        end
      end

      context 'when upload was rejected' do
        before do
          stub_request(:head, file_url).to_return(
            status: 200,
            headers: {
              'Content-Type' => 'inode/x-empty',
              'X-Amz-Meta-Xikolo-Purpose' => 'video_material',
              'X-Amz-Meta-Xikolo-State' => 'rejected',
            }
          )
        end

        error_details = {'reading_material_upload_id' => ['invalid upload']}
        include_examples 'does not update the reading material', error_details
      end

      context 'without access permission' do
        before do
          stub_request(:head, file_url).to_return(status: 403)
        end

        error_details = {'reading_material_upload_id' => ['could not process file upload']}
        include_examples 'does not update the reading material', error_details
      end

      context 'when saving to destination is forbidden' do
        before do
          stub_request(:head, file_url).to_return(
            status: 200,
            headers: {
              'Content-Type' => 'inode/x-empty',
              'X-Amz-Meta-Xikolo-Purpose' => 'video_material',
              'X-Amz-Meta-Xikolo-State' => 'accepted',
            }
          )
          stub_request(:put, %r{http://s3.xikolo.de/xikolo-video/videos/[0-9a-zA-Z]+/[0-9a-zA-Z]+/new_reading_material.pdf}) \
            .to_return(status: 403)
        end

        error_details = {'reading_material_upload_id' => ['could not process file upload']}
        include_examples 'does not update the reading material', error_details
      end
    end

    context 'with copied video references the same S3 reference' do
      let!(:store_stub) do
        stub_request(:put,
          %r{http://s3.xikolo.de/xikolo-video/videos/[0-9a-zA-Z]+/[0-9a-zA-Z]+/new_reading_material.pdf}) \
          .to_return(
            status: 200,
            headers: {'Content-Type' => 'application/xml', 'Content-Disposition' => 'attachment; filename="new_reading_material.pdf"'},
            body: <<~XML)
              <CopyObjectResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
                <LastModified>2018-08-02T15:42:36.430Z</LastModified>
                <ETag>&#34;d41d8cd98f00b204e9800998ecf8427e&#34;</ETag>
              </CopyObjectResult>
            XML
      end

      before do
        # Create another video with the same file reference
        create(:video, reading_material_uri: video.reading_material_uri)

        stub_request(:head, file_url).to_return(
          status: 200,
          headers: {
            'Content-Type' => 'inode/x-empty',
            'X-Amz-Meta-Xikolo-Purpose' => 'video_material',
            'X-Amz-Meta-Xikolo-State' => 'accepted',
          }
        )
      end

      it 'updates the video successfully' do
        expect(update_video).to respond_with(:no_content)
      end

      it 'stores the reading material file in S3' do
        update_video
        expect(store_stub).to have_been_requested
      end

      it 'updates the reading_material_uri on the video' do
        update_video
        expect(video.reload.reading_material_uri).not_to eq reading_material_uri
        expect(video.reload.reading_material_uri).to match(%r{s3://xikolo-video/videos/#{UUID4(video_id).to_s(format: :base62)}/[0-9a-zA-Z]+/new_reading_material.pdf})
      end

      it 'does not delete the old file' do
        update_video
        expect(FileDeletionWorker.jobs).to be_empty
      end
    end
  end

  context 'without newly uploaded reading material' do
    let(:update_params) { super().merge reading_material_url: nil }

    it 'clears the reading_material_uri on the video' do
      expect { update_video }.to change { video.reload.reading_material_uri }.from(reading_material_uri).to(nil)
    end

    it 'deletes the old file' do
      update_video
      expect(FileDeletionWorker.jobs.last['args']).to eq [reading_material_uri]
    end
  end
end
