# frozen_string_literal: true

require 'spec_helper'

RSpec.describe 'Video: Update', type: :request do
  subject(:update_video) { api.rel(:video).patch(params, id: video.id).value! }

  let(:video) { create(:video) }
  let(:params) { {} }
  let(:api) { Restify.new(:test).get.value }

  context 'with subtitles' do
    let(:upload_id) { 'f13d30d3-6369-4816-9695-af5318c8ac15' }
    let(:params) { super().merge subtitles_upload_id: upload_id }
    let(:file_url) { "http://s3.xikolo.de/xikolo-uploads/uploads/f13d30d3-6369-4816-9695-af5318c8ac15/#{file_name}" }

    before do
      stub_request(:get,
        'http://s3.xikolo.de/xikolo-uploads/?list-type=2&' \
        'prefix=uploads%2Ff13d30d3-6369-4816-9695-af5318c8ac15') \
        .to_return(
          status: 200,
          headers: {'Content-Type' => 'Content-Type: application/xml'},
          body: <<~XML)
            <?xml version="1.0" encoding="UTF-8"?>
            <ListBucketResult xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
              <Name>xikolo-uploads</Name>
              <Prefix>uploads/f13d30d3-6369-4816-9695-af5318c8ac15</Prefix>
              <IsTruncated>false</IsTruncated>
              <Contents>
                <Key>uploads/f13d30d3-6369-4816-9695-af5318c8ac15/#{file_name}</Key>
                <LastModified>2018-08-02T13:27:56.768Z</LastModified>
                <ETag>&#34;d41d8cd98f00b204e9800998ecf8427e&#34;</ETag>
              </Contents>
            </ListBucketResult>
          XML

      stub_request(:head, file_url).to_return(
        status: 200,
        headers: {
          'Content-Type' => 'inode/x-empty',
          'X-Amz-Meta-Xikolo-Purpose' => 'video_subtitles',
          'X-Amz-Meta-Xikolo-State' => 'accepted',
        }
      )
      stub_request(:get, file_url).to_return(
        body: File.new(Rails.root.join("spec/support/files/subtitles/#{file_name}")),
        status: 200
      )
    end

    context 'with .vtt subtitles' do
      describe 'with a valid file' do
        let(:file_name) { 'valid_en.vtt' }

        it 'update the video and creates the subtitle' do
          expect { update_video }.to change(video.subtitles, :count).from(0).to(1)
          expect(SubtitleCue.count).to eq 3
        end
      end

      describe 'with an invalid file' do
        let(:file_name) { 'invalid_en.vtt' }

        it 'does not create the subtitle, and return an error' do
          expect { update_video }.to raise_error(Restify::UnprocessableEntity) do |error|
            expect(error.errors).to eq 'invalid_subtitle' => [{'identifiers' => [1]}]
            expect(video.subtitles.count).to be_zero
          end
        end
      end

      describe 'with an invalid file with multiple errors' do
        let(:file_name) { 'multiple_invalid_en.vtt' }

        it 'does not create the subtitle, and return an error' do
          expect { update_video }.to raise_error(Restify::UnprocessableEntity) do |error|
            expect(error.errors).to eq 'invalid_subtitle' => [{'identifiers' => [1, 2]}]
            expect(video.subtitles.count).to be_zero
          end
        end
      end
    end
  end
end
