# frozen_string_literal: true

ENV['RAILS_ENV'] ||= 'test'

require 'simplecov'
require 'simplecov-teamcity-summary'

if ENV['TEAMCITY_VERSION']
  SimpleCov.formatters = [
    SimpleCov::Formatter::HTMLFormatter,
    SimpleCov::Formatter::TeamcitySummaryFormatter,
  ]
end

require File.expand_path('../config/environment', __dir__)

require 'rspec/rails'
require 'sidekiq/api'
require 'sidekiq/testing'
require 'webmock/rspec'
require 'pry'

require 'restify'
require 'restify/adapter/typhoeus'

Restify::Registry.store :test, 'http://video.xikolo.tld',
  adapter: Restify::Adapter::Typhoeus.new(sync: true)

# Requires supporting ruby files with custom matchers and macros, etc,
# in spec/support/ and its subdirectories.
Dir[Rails.root.join('spec/support/**/*.rb')].each {|f| require f }

# Maintain test schema
ActiveRecord::Migration.maintain_test_schema!

RSpec.configure do |config|
  config.order = 'random'
  config.expose_dsl_globally = false
  config.example_status_persistence_file_path = 'spec/examples.txt'

  config.expect_with :rspec do |expectations|
    expectations.syntax = :expect
  end

  config.mock_with :rspec do |mocks|
    mocks.syntax = :expect
    mocks.verify_partial_doubles = true
  end

  # Include useful methods for easier testing
  # This allows to directly use route helpers and factory methods

  config.include WebMock::API
  config.include FactoryBot::Syntax::Methods
  config.include Rails.application.routes.url_helpers

  # Web requests

  config.before do
    stub_request(:any, /video\.xikolo\.tld/)
      .to_rack(Xikolo::VideoService::Application)
  end

  config.before(:each, type: :decorator) do
    # Require for draper to be able to generate full URLs
    default_url_options[:host] = 'test.host'
  end

  # Messaging

  config.after do
    Msgr.client.stop delete: true
  end

  # Background jobs

  config.before do
    Sidekiq::Testing.fake!
    Sidekiq::Worker.clear_all
    Sidekiq::Queue.all.each(&:clear) # rubocop:disable Rails/FindEach
  end

  # Database

  # Run tests in transaction using DatabaseCleaner, not Rails' own facility
  config.use_transactional_fixtures = false

  # Allow disabling transaction for specific tests / test groups
  config.around do |example|
    if example.metadata[:transaction] == false
      DatabaseCleaner.strategy = :deletion
    else
      DatabaseCleaner.strategy = :transaction
    end
    DatabaseCleaner.cleaning(&example)
  end

  config.before(:all) do
    DatabaseCleaner.clean_with :truncation
  end
end
