# frozen_string_literal: true

require 'spec_helper'

RSpec.describe FetchThumbnailsWorker, type: :worker do
  describe '#perform' do
    let(:url) { 'http://external.xikolo/thumbnails.zip' }
    let(:vid) { UUID4(video.id).to_s(format: :base62) }
    let(:s3base) do
      Addressable::URI.parse(
        "http://s3.xikolo.de/xikolo-video/videos/#{vid}/"
      )
    end
    let(:perform) do
      Sidekiq::Testing.inline! do
        described_class.perform_async(video.id, counter, url)
      end
    end

    context 'initial fetch' do
      let(:video) { create(:video, thumbnails_uri: nil) }
      let(:counter) { 1 }

      before do
        stub_request(:get, url).to_return(
          body: File.new("spec/support/files/#{file}"),
          status: 200
        )
        stub_request(:put, s3base.join('thumbnails_v1.zip'))
      end

      context 'with zip with slides' do
        let(:file) { 'video-thumbnails-new-with-slides.zip' }

        before do
          stub_request(:put, s3base.join('slides_v1/1.jpg'))
          stub_request(:put, s3base.join('slides_v1/2.jpg'))
        end

        it 'creates 2 thumbnail entries' do
          expect { perform }.to change { Thumbnail.where(video_id: video.id).count }.from(0).to(2)
          thumbnails = Thumbnail.where(video_id: video.id).order(:start_time)
          expect(thumbnails[0].start_time).to eq 1
          expect(thumbnails[0].file_uri).to end_with('slides_v1/1.jpg')
          expect(thumbnails[1].start_time).to eq 2
          expect(thumbnails[1].file_uri).to end_with('slides_v1/2.jpg')
        end

        it 'storages the thumbnail archive' do
          expect { perform }.to change { video.reload.thumbnails_uri }.from(nil)
          expect(video.thumbnails_uri).to end_with('/thumbnails_v1.zip')
        end
      end

      context 'with zip without slides' do
        let(:file) { 'video-thumbnails-new-no-slides.zip' }

        before do
          stub_request(:put, s3base.join('thumbnails_v1/6728.jpg'))
          stub_request(:put, s3base.join('thumbnails_v1/6820.jpg'))
        end

        it 'creates 2 thumbnail entries' do
          expect { perform }.to change { Thumbnail.where(video_id: video.id).count }.from(0).to(2)
          thumbnails = Thumbnail.where(video_id: video.id).order(:start_time)
          expect(thumbnails[0].start_time).to eq 224
          expect(thumbnails[0].file_uri).to end_with('thumbnails_v1/6728.jpg')
          expect(thumbnails[1].start_time).to eq 227
          expect(thumbnails[1].file_uri).to end_with('thumbnails_v1/6820.jpg')
        end

        it 'storages the thumbnail archive' do
          expect { perform }.to change { video.reload.thumbnails_uri }.from(nil)
          expect(video.thumbnails_uri).to end_with('/thumbnails_v1.zip')
        end
      end

      context 'with old zip format' do
        let(:file) { 'video-thumbnails-old.zip' }

        before do
          stub_request(:put, s3base.join('thumbnails_v1/150.jpg'))
          stub_request(:put, s3base.join('thumbnails_v1/3306.jpg'))
        end

        it 'creates 2 thumbnail entries' do
          expect { perform }.to change { Thumbnail.where(video_id: video.id).count }.from(0).to(2)
          thumbnails = Thumbnail.where(video_id: video.id).order(:start_time)
          expect(thumbnails[0].start_time).to eq 6
          expect(thumbnails[0].file_uri).to end_with('thumbnails_v1/150.jpg')
          expect(thumbnails[1].start_time).to eq 12
          expect(thumbnails[1].file_uri).to end_with('thumbnails_v1/3306.jpg')
        end

        it 'storages the thumbnail archive' do
          expect { perform }.to change { video.reload.thumbnails_uri }.from(nil)
          expect(video.thumbnails_uri).to end_with('/thumbnails_v1.zip')
        end
      end
    end

    context 'updating fetch' do
      let(:video) { create(:video, thumbnails_uri: 's3://xikolo-video/videos/1/thumbnails_v1.zip') }
      let(:counter) { 2 }

      let!(:delete_old_thumbnails) do
        stub_request(:delete, s3base.join('thumbnails_v1/1.jpg'))
      end

      before do
        stub_request(:get, url).to_return(
          body: File.new("spec/support/files/#{file}"),
          status: 200
        )
        stub_request(:put, s3base.join('thumbnails_v2.zip'))
        create(:thumbnail, video_id: video.id, start_time: 1,
          file_uri: "s3://xikolo-video/videos/#{vid}/thumbnails_v1/1.jpg")
      end

      context 'with zip with slides' do
        let(:file) { 'video-thumbnails-new-with-slides.zip' }

        before do
          stub_request(:put, s3base.join('slides_v2/1.jpg'))
          stub_request(:put, s3base.join('slides_v2/2.jpg'))
        end

        it 'creates 2 thumbnail entries' do
          expect { perform }.to change { Thumbnail.where(video_id: video.id).count }.from(1).to(2)
          thumbnails = Thumbnail.where(video_id: video.id).order(:start_time)
          expect(thumbnails[0].start_time).to eq 1
          expect(thumbnails[0].file_uri).to end_with('slides_v2/1.jpg')
          expect(thumbnails[1].start_time).to eq 2
          expect(thumbnails[1].file_uri).to end_with('slides_v2/2.jpg')
        end

        it 'schedules removal of old thumbnails' do
          perform
          expect(delete_old_thumbnails).to have_been_requested
        end

        it 'storages the thumbnail archive' do
          expect { perform }.to change { video.reload.thumbnails_uri }
          expect(video.thumbnails_uri).to end_with('/thumbnails_v2.zip')
        end
      end

      context 'with zip without slides' do
        let(:file) { 'video-thumbnails-new-no-slides.zip' }

        before do
          stub_request(:put, s3base.join('thumbnails_v2/6728.jpg'))
          stub_request(:put, s3base.join('thumbnails_v2/6820.jpg'))
        end

        it 'creates 2 thumbnail entries' do
          expect { perform }.to change { Thumbnail.where(video_id: video.id).count }.from(1).to(2)
          thumbnails = Thumbnail.where(video_id: video.id).order(:start_time)
          expect(thumbnails[0].start_time).to eq 224
          expect(thumbnails[0].file_uri).to end_with('thumbnails_v2/6728.jpg')
          expect(thumbnails[1].start_time).to eq 227
          expect(thumbnails[1].file_uri).to end_with('thumbnails_v2/6820.jpg')
        end

        it 'schedules removal of old thumbnails' do
          perform
          expect(delete_old_thumbnails).to have_been_requested
        end

        it 'storages the thumbnail archive' do
          expect { perform }.to change { video.reload.thumbnails_uri }
          expect(video.thumbnails_uri).to end_with('/thumbnails_v2.zip')
        end
      end

      context 'with old zip format' do
        let(:file) { 'video-thumbnails-old.zip' }

        before do
          stub_request(:put, s3base.join('thumbnails_v2/150.jpg'))
          stub_request(:put, s3base.join('thumbnails_v2/3306.jpg'))
        end

        it 'creates 2 thumbnail entries' do
          expect { perform }.to change { Thumbnail.where(video_id: video.id).count }.from(1).to(2)
          thumbnails = Thumbnail.where(video_id: video.id).order(:start_time)
          expect(thumbnails[0].start_time).to eq 6
          expect(thumbnails[0].file_uri).to end_with('thumbnails_v2/150.jpg')
          expect(thumbnails[1].start_time).to eq 12
          expect(thumbnails[1].file_uri).to end_with('thumbnails_v2/3306.jpg')
        end

        it 'schedules removal of old thumbnails' do
          perform
          expect(delete_old_thumbnails).to have_been_requested
        end

        it 'storages the thumbnail archive' do
          expect { perform }.to change { video.reload.thumbnails_uri }
          expect(video.thumbnails_uri).to end_with('/thumbnails_v2.zip')
        end
      end
    end

    context 'updating fetch on cloned video' do
      let(:video) { create(:video, thumbnails_uri: 's3://xikolo-video/videos/1/thumbnails_v1.zip') }
      let(:other_video) { create(:video, thumbnails_uri: video.thumbnails_uri) }
      let(:counter) { 2 }

      let!(:delete_old_thumbnails) do
        stub_request(:delete, s3base.join('thumbnails_v1/1.jpg'))
      end

      before do
        stub_request(:get, url).to_return(
          body: File.new("spec/support/files/#{file}"),
          status: 200
        )
        stub_request(:put, s3base.join('thumbnails_v2.zip'))
        create(:thumbnail, video_id: video.id, start_time: 1,
          file_uri: "s3://xikolo-video/videos/#{vid}/thumbnails_v1/1.jpg")
        create(:thumbnail, video_id: other_video.id, start_time: 1,
          file_uri: "s3://xikolo-video/videos/#{vid}/thumbnails_v1/1.jpg")
      end

      context 'with zip with slides' do
        let(:file) { 'video-thumbnails-new-with-slides.zip' }

        before do
          stub_request(:put, s3base.join('slides_v2/1.jpg'))
          stub_request(:put, s3base.join('slides_v2/2.jpg'))
        end

        it 'creates 2 thumbnail entries' do
          expect { perform }.to change { Thumbnail.where(video_id: video.id).count }.from(1).to(2)
          thumbnails = Thumbnail.where(video_id: video.id).order(:start_time)
          expect(thumbnails[0].start_time).to eq 1
          expect(thumbnails[0].file_uri).to end_with('slides_v2/1.jpg')
          expect(thumbnails[1].start_time).to eq 2
          expect(thumbnails[1].file_uri).to end_with('slides_v2/2.jpg')
        end

        it 'does not schedule the removal of old thumbnails (needed for clone video)' do
          perform
          expect(delete_old_thumbnails).not_to have_been_requested
        end

        it 'storages the thumbnail archive' do
          expect { perform }.to change { video.reload.thumbnails_uri }
          expect(video.thumbnails_uri).to end_with('/thumbnails_v2.zip')
        end
      end

      context 'with zip without slides' do
        let(:file) { 'video-thumbnails-new-no-slides.zip' }

        before do
          stub_request(:put, s3base.join('thumbnails_v2/6728.jpg'))
          stub_request(:put, s3base.join('thumbnails_v2/6820.jpg'))
        end

        it 'creates 2 thumbnail entries' do
          expect { perform }.to change { Thumbnail.where(video_id: video.id).count }.from(1).to(2)
          thumbnails = Thumbnail.where(video_id: video.id).order(:start_time)
          expect(thumbnails[0].start_time).to eq 224
          expect(thumbnails[0].file_uri).to end_with('thumbnails_v2/6728.jpg')
          expect(thumbnails[1].start_time).to eq 227
          expect(thumbnails[1].file_uri).to end_with('thumbnails_v2/6820.jpg')
        end

        it 'does not schedule the removal of old thumbnails (needed for clone video)' do
          perform
          expect(delete_old_thumbnails).not_to have_been_requested
        end

        it 'storages the thumbnail archive' do
          expect { perform }.to change { video.reload.thumbnails_uri }
          expect(video.thumbnails_uri).to end_with('/thumbnails_v2.zip')
        end
      end

      context 'with old zip format' do
        let(:file) { 'video-thumbnails-old.zip' }

        before do
          stub_request(:put, s3base.join('thumbnails_v2/150.jpg'))
          stub_request(:put, s3base.join('thumbnails_v2/3306.jpg'))
        end

        it 'creates 2 thumbnail entries' do
          expect { perform }.to change { Thumbnail.where(video_id: video.id).count }.from(1).to(2)
          thumbnails = Thumbnail.where(video_id: video.id).order(:start_time)
          expect(thumbnails[0].start_time).to eq 6
          expect(thumbnails[0].file_uri).to end_with('thumbnails_v2/150.jpg')
          expect(thumbnails[1].start_time).to eq 12
          expect(thumbnails[1].file_uri).to end_with('thumbnails_v2/3306.jpg')
        end

        it 'does not schedule the removal of old thumbnails (needed for clone video)' do
          perform
          expect(delete_old_thumbnails).not_to have_been_requested
        end

        it 'storages the thumbnail archive' do
          expect { perform }.to change { video.reload.thumbnails_uri }
          expect(video.thumbnails_uri).to end_with('/thumbnails_v2.zip')
        end
      end
    end
  end
end
