# frozen_string_literal: true

require 'spec_helper'

RSpec.describe FileDeletionWorker, type: :worker do
  subject(:worker) { described_class.new }

  let(:uri) { 's3://xikolo-video/videos/asdfasdf/slides_v1.pdf' }

  describe '#perform' do
    it 'removes the S3 object' do
      deletion = stub_request(
        :delete,
        'http://s3.xikolo.de/xikolo-video/videos/asdfasdf/slides_v1.pdf'
      ).to_return(status: 200)

      worker.perform uri

      expect(deletion).to have_been_requested
    end

    it 'raises S3 errors' do
      stub_request(
        :delete,
        'http://s3.xikolo.de/xikolo-video/videos/asdfasdf/slides_v1.pdf'
      ).to_return(status: 500)

      expect do
        worker.perform(uri)
      end.to raise_error(Aws::S3::Errors::Http500Error)
    end

    it 'skips removal of still referenced slides files' do
      create(:video,
        slides_uri: 's3://xikolo-video/videos/asdfasdf/slides_v1.pdf')
      deletion = stub_request(
        :delete,
        'http://s3.xikolo.de/xikolo-video/videos/asdfasdf/slides_v1.pdf'
      ).to_return(status: 200)

      worker.perform uri

      expect(deletion).not_to have_been_requested
    end

    it 'skips removal of still referenced reading material files' do
      uri = 's3://xikolo-video/videos/asdfasdf/read_v1.pdf'
      create(:video, reading_material_uri: uri)
      deletion = stub_request(
        :delete,
        'http://s3.xikolo.de/xikolo-video/videos/asdfasdf/read_v1.pdf'
      ).to_return(status: 200)

      worker.perform uri

      expect(deletion).not_to have_been_requested
    end

    it 'skips removal of still referenced transcript files' do
      uri = 's3://xikolo-video/videos/asdfasdf/transcript_v1.pdf'
      create(:video, transcript_uri: uri)
      deletion = stub_request(
        :delete,
        'http://s3.xikolo.de/xikolo-video/videos/asdfasdf/transcript_v1.pdf'
      ).to_return(status: 200)

      worker.perform uri

      expect(deletion).not_to have_been_requested
    end

    it 'skips removal of still referenced description files' do
      uri = 's3://xikolo-video/videos/asdfasdf/description_v1.pdf'
      create(:video, description: "something\n #{uri}; more ...")
      deletion = stub_request(
        :delete,
        'http://s3.xikolo.de/xikolo-video/videos/asdfasdf/description_v1.pdf'
      ).to_return(status: 200)

      worker.perform uri

      expect(deletion).not_to have_been_requested
    end
  end
end
