# frozen_string_literal: true

require 'spec_helper'

RSpec.describe SyncSubtitleWorker, type: :worker do
  describe '#perform' do
    subject(:perform) do
      Sidekiq::Testing.inline! do
        described_class.perform_async(video.id, 'de')
      end
    end

    let(:video) { create(:video, :without_streams, pip_stream: stream) }
    let(:stream) { create(:stream, provider:) }
    let(:provider) { create(:provider, :vimeo) }
    let(:adapter) { instance_double(VimeoAdapter, remove_subtitles!: true, attach_subtitle!: true) }

    before { allow(VimeoAdapter).to receive(:new).and_return(adapter) }

    context 'when there are no subtitles for the given language' do
      before do
        video.subtitles << create(:subtitle, :with_cues, lang: 'en')
      end

      it 'removes existing subtitles for the video\'s PIP stream' do
        perform
        expect(adapter).to have_received(:remove_subtitles!).with(stream, 'de').once
      end
    end

    context 'when there are subtitles for the given language' do
      let(:subtitle_de) { create(:subtitle, :with_cues, lang: 'de') }

      before do
        video.subtitles << create(:subtitle, :with_cues, lang: 'en')
        video.subtitles << subtitle_de
      end

      it 'removes existing subtitles and attaches the new subtitle to the video\'s PIP stream' do
        perform
        expect(adapter).to have_received(:remove_subtitles!).with(stream, 'de').once
        expect(adapter).to have_received(:attach_subtitle!).with(stream, subtitle_de).once
      end

      context 'when the video does not have a PIP stream' do
        let(:video) { create(:video, pip_stream: nil, lecturer_stream: stream) }

        it 'removes existing subtitles and attaches the new subtitle to the video\'s lecturer stream' do
          perform
          expect(adapter).to have_received(:remove_subtitles!).with(stream, 'de').once
          expect(adapter).to have_received(:attach_subtitle!).with(stream, subtitle_de).once
        end
      end

      context 'when the video does not even have a lecturer stream' do
        let(:video) { create(:video, pip_stream: nil, lecturer_stream: nil, slides_stream: stream) }

        it 'removes existing subtitles and attaches the new subtitle to the video\'s slides stream' do
          perform
          expect(adapter).to have_received(:remove_subtitles!).with(stream, 'de').once
          expect(adapter).to have_received(:attach_subtitle!).with(stream, subtitle_de).once
        end
      end
    end
  end
end
