# frozen_string_literal: true

require 'spec_helper'
require 'rspec/expectations'

describe ItemsController, type: :controller do
  let(:course_id) { '00000001-3300-4444-9999-000000000001' }
  let(:item_id) { '00000003-3300-4444-9999-000000000001' }
  let(:video_item_id) { '00000003-3300-4444-9999-000000000002' }
  let(:video_id) { '00000001-3600-4444-9999-000000000001' }
  let(:lti_exercise) { create(:lti_exercise) }
  let(:lti_exercise_id) { lti_exercise.id }
  let(:section_id) { '00000002-3300-4444-9999-000000000001' }
  let(:user_id) { SecureRandom.uuid }
  let(:permissions) { [] }
  let(:course_state) { 'archive' }
  let(:request_context_id) { course_context_id }
  let(:section) { build(:'course:section', id: section_id, course_id:) }
  let(:item_stub) do
    Stub.json({
      id: item_id,
      content_id: lti_exercise_id,
      section_id:,
      content_type: 'lti_exercise',
      position: 2,
      published: true,
    })
  end

  before do
    Stub.service(
      :account,
      session_url: '/sessions/{id}'
    )
    Stub.service(
      :course,
      course_url: '/courses/{id}',
      enrollments_url: '/enrollments',
      section_url: '/sections/{id}',
      sections_url: '/sections',
      item_url: '/items/{id}',
      items_url: '/items'
    )

    Stub.request(
      :account, :get, "/users/#{user_id}/preferences"
    ).to_return Stub.json({
      properties: {},
    })
    Stub.request(
      :account, :get, "/users/#{user_id}/permissions",
      query: {context: 'root', user_id:}
    ).to_return Stub.json(permissions)

    Stub.request(
      :course, :get, "/courses/#{course_id}"
    ).to_return Stub.json({
      id: course_id,
      title: 'A Course',
      status: course_state,
      course_code: 'test',
      context_id: course_context_id,
      video_course_codes: [],
    })
    Stub.request(
      :course, :get, '/enrollments',
      query: {course_id:, user_id:}
    ).to_return Stub.json([
      {course_id:, user_id:},
    ])
    Stub.request(
      :course, :get, "/sections/#{section_id}"
    ).to_return Stub.json(
      section
    )
    Stub.request(
      :course, :get, '/sections',
      query: {course_id:}
    ).to_return Stub.json([])
    Stub.request(
      :course, :get, "/items/#{UUID(item_id)}",
      query: {user_id:}
    ).to_return item_stub
    # The video item
    Stub.request(
      :course, :get, "/items/#{video_item_id}",
      query: {user_id:}
    ).to_return Stub.json({
      id: video_item_id,
      title: 'A Video',
      content_id: video_id,
      section_id:,
      content_type: 'video',
      position: 2,
      start_date: 10.days.ago.iso8601,
      end_date: 17.days.from_now.iso8601,
      published: true,
    })

    stub_user id: user_id, permissions:
  end

  describe 'GET #new' do
    subject do
      get :new, params: {course_id:, section_id:}
    end

    before do
      Stub.request(
        :course, :get, '/items',
        query: {section_id:, state_for: ''}
      ).to_return Stub.json([])
    end

    context 'as non admin' do
      it { is_expected.to have_http_status :found }
    end

    context 'via get with course.content.edit permissions' do
      let(:request_context_id) { course_context_id }
      let(:permissions)        { ['course.content.edit', 'course.content.access'] }

      before do
        Stub.service(:peerassessment, build(:'peerassessment:root'))
        Stub.request(
          :peerassessment, :get, '/peer_assessments',
          query: {course_id:}
        ).to_return Stub.json([])

        Stub.service(:video, build(:'video:root'))
        Stub.request(
          :video, :get, '/streams'
        ).to_return Stub.json([])
      end

      it { is_expected.to have_http_status :ok }
    end
  end

  describe 'GET #show' do
    let(:permissions) { ['course.content.access.available'] }
    let(:item_stub) do
      Stub.json({
        id: item_id,
        content_id: lti_exercise_id,
        section_id:,
      })
    end

    context 'via get' do
      subject { get :show, params: {course_id:, id: item_id} }

      it { is_expected.to have_http_status :found }
    end

    context 'with an LTI exercise' do
      subject(:action) do
        get :show, params: {
          id: item_id,
          course_id:,
          section_id:,
        }
      end

      let(:lti_gradebook) { create(:lti_gradebook, exercise: lti_exercise, user_id:) }

      let(:item_stub) do
        Stub.json({
          id: item_id,
          content_id: lti_exercise_id,
          content_type: 'lti_exercise',
          published: true,
          unlocked: true,
          section_id:,
          title: 'Coding Exercise',
          max_points: 10.5,
        })
      end

      before do
        Stub.request(
          :course, :get, '/items',
          query: {section_id:, state_for: user_id, published: true}
        ).to_return Stub.json([
          {id: item_id, max_points: 10.5},
        ])
        Stub.request(
          :course, :post, "/items/#{item_id}/users/#{user_id}/visit"
        ).to_return Stub.json({item_id:, user_id:})
      end

      it 'assigns all required resources' do
        action
        expect(assigns(:item_presenter)).to be_a(LtiExerciseItemPresenter)
      end

      context 'with LTI grades present' do
        before do
          create(:lti_grade, gradebook: lti_gradebook, value: 0.25)
          create(:lti_grade, gradebook: lti_gradebook, value: 0.5)
          create(:lti_grade, gradebook: lti_gradebook, value: 0)
        end

        it 'assigns the item presenter, which exposes the best score' do
          action
          presenter = assigns(:item_presenter)
          expect(presenter.score).to eq 5.25
        end

        it { is_expected.to render_template layout: 'course_area_two_cols' }
      end

      context 'without LTI grades' do
        it { is_expected.to render_template layout: 'course_area_two_cols' }
      end

      it { is_expected.to have_http_status :ok }
      it { is_expected.to render_template 'show' }
    end

    context 'for video' do
      render_views

      subject(:action) do
        get :show, params: {course_id:, id: UUID(video_item_id)}
      end

      let(:course_state) { 'active' }

      before do
        Stub.service(:news, build(:'news:root'))
        Stub.service(:pinboard, build(:'pinboard:root'))

        Stub.request(
          :news, :get, '/current_alerts',
          query: {language: 'en'}
        ).to_return Stub.json([])

        Stub.request(
          :pinboard, :get, '/topics',
          query: {item_id: UUID(video_item_id)}
        ).to_return Stub.json([])

        Stub.request(
          :course, :get, '/items',
          query: {section_id:, state_for: user_id, published: 'true'}
        ).to_return Stub.json([
          {
            id: UUID(video_item_id),
            content_id: video_id,
            section_id:,
            content_type: 'video',
            start_date: 10.days.ago.iso8601,
            end_date: 17.days.from_now.iso8601,
            published: true,
          },
        ])
        Stub.request(
          :course, :post, "/items/#{video_item_id}/users/#{user_id}/visit"
        ).to_return Stub.json({item_id: video_item_id, user_id:})

        create(:video, id: video_id)
      end

      it 'sets the meta tags properly' do
        action
        expect(response.body).to _have_xpath '//title'
        expect(response.body).to _have_xpath "//meta[@property='og:title']"
        expect(response.body).to _have_xpath "//meta[@property='og:image']"
        expect(response.body).to _have_xpath "//meta[@property='og:type']"
        expect(response.body).to _have_xpath "//meta[@property='og:url']"
      end
    end

    context 'for quiz' do
      render_views

      subject(:show_item) { get :show, params: {course_id:, id: UUID(quiz_item_id)} }

      let(:course_state) { 'active' }
      let(:quiz_item_id) { generate(:item_id) }
      let(:quiz_id) { generate(:quiz_id) }

      before do
        Stub.service(:quiz, build(:'quiz:root'))

        Stub.request(
          :course, :get, '/items',
          query: {section_id:, state_for: user_id, published: 'true'}
        ).to_return Stub.json([
          {
            id: UUID(quiz_item_id),
            content_id: quiz_id,
            section_id:,
            content_type: 'quiz',
            start_date: 10.days.ago.iso8601,
            end_date: 17.days.from_now.iso8601,
            published: true,
          },
        ])
        Stub.request(
          :course, :get, "/items/#{quiz_item_id}",
          query: {user_id:}
        ).to_return Stub.json({
          id: quiz_item_id,
          title: 'A Quiz',
          content_id: quiz_id,
          section_id:,
          content_type: 'quiz',
          position: 2,
          start_date: 10.days.ago.iso8601,
          end_date: 17.days.from_now.iso8601,
          published: true,
        })
        Stub.request(
          :quiz, :get, "/quizzes/#{quiz_id}"
        ).to_return Stub.json(build(:'quiz:quiz', id: quiz_id))
        Stub.request(
          :quiz, :get, '/questions',
          query: hash_including(quiz_id:)
        ).to_return Stub.json(build_list(:'quiz:question', 2, quiz_id:))

        Stub.request(
          :course, :post, "/items/#{quiz_item_id}/users/#{user_id}/visit"
        ).to_return Stub.json({item_id: quiz_item_id, user_id:})
      end

      context 'without an enrollment' do
        before do
          Stub.request(
            :course, :get, '/enrollments',
            query: {course_id:, user_id:}
          ).to_return Stub.json([])
        end

        it 'shows a friendly error message' do
          show_item
          expect(response.body).to have_content 'Please enroll to see this quiz'
        end
      end
    end
  end
end
