# frozen_string_literal: true

require 'spec_helper'

describe 'Admin: Create Channel', gen: 2, type: :feature do
  let(:user) { build(:'account:user') }

  before do
    stub_user id: user['id'], permissions: %w[course.channel.create course.channel.index]
    Stub.request(:account, :get, "/users/#{user['id']}")
      .to_return Stub.json(user)
    Stub.service(:course, build(:'course:root'))
    Stub.request(:course, :get, '/channels', query: hash_including({}))
      .and_return Stub.json([])
    Stub.request(:course, :get, '/api/v2/course/courses', query: hash_including({}))
      .and_return Stub.json([])
  end

  def toggle(name)
    find('label', text: name).click
  end

  it 'prefills the form on unprocessable channel data' do
    visit '/channels/new'

    fill_in 'Code', with: 'subset'
    fill_in 'Name', with: 'Important Subset of Courses'
    fill_markdown_editor 'Description (in English)', with: "Headline\n===\n\nenglish!"
    fill_markdown_editor 'Description (in German)', with: "Headline\n===\n\ngerman!"

    click_link_or_button 'Advanced settings'
    fill_in 'Info link URL (in English)', with: 'https://www.example.com/faq'
    fill_in 'Info link URL (in German)', with: 'https://www.example.com/de/faq'
    fill_in 'Info link label (in English)', with: 'Our FAQ'
    fill_in 'Info link label (in German)', with: 'Unsere FAQ'

    toggle 'Public'

    Stub.request(:course, :post, '/channels')
      .to_return Stub.json({
        'errors' => {'code' => ['upload_error', 'Custom Error message']},
      }, status: 422)
    click_link_or_button 'Add channel'

    expect(page).to have_content 'Custom Error message'
    expect(page).to have_content 'Your file upload could not be stored.'

    expect(page).to have_field('Code', with: 'subset')
    expect(page).to have_field('Name', with: 'Important Subset of Courses')
    expect(page).to have_markdown_editor('Description (in English)', with: "Headline\n===\n\nenglish!")
    expect(page).to have_markdown_editor('Description (in German)', with: "Headline\n===\n\ngerman!")

    click_link_or_button 'Advanced settings'
    expect(page).to have_field('Info link URL (in English)', with: 'https://www.example.com/faq')
    expect(page).to have_field('Info link URL (in German)', with: 'https://www.example.com/de/faq')
    expect(page).to have_field('Info link label (in English)', with: 'Our FAQ')
    expect(page).to have_field('Info link label (in German)', with: 'Unsere FAQ')

    attributes = nil
    Stub.request(:course, :post, '/channels').with do |request|
      attributes = JSON.parse request.body
      true
    end.to_return(status: 201)

    click_link_or_button 'Add channel'

    expect(attributes).to include(
      'code' => 'subset',
      'name' => 'Important Subset of Courses',
      'public' => true,
      'stage_statement' => nil
    )

    expect(attributes['description']).to include(
      'en' => "Headline\n===\n\nenglish!",
      'de' => "Headline\n===\n\ngerman!"
    )

    expect(attributes['info_link']).to include(
      'href' => {'en' => 'https://www.example.com/faq', 'de' => 'https://www.example.com/de/faq'},
      'label' => {'en' => 'Our FAQ', 'de' => 'Unsere FAQ'}
    )
  end
end
