# frozen_string_literal: true

require 'spec_helper'

describe 'Item: Time Effort', gen: 2, type: :feature do
  let(:user_id) { generate(:user_id) }
  let(:features) { {'advanced_tooltip' => 'true'} }
  let(:course) do
    build(:'course:course',
      course_code: 'the_course',
      title: 'Proctored course',
      proctored: true)
  end
  let(:section) do
    build(:'course:section', course_id: course['id'], title: 'Week 1')
  end
  let(:video) { create(:video) }
  let(:item) { build(:'course:item', :video, item_params) }
  let(:item_params) do
    {
      course_id: course['id'],
      section_id: section['id'],
      content_id: video.id,
      title: 'Awesome video',
      time_effort: 180,
    }
  end

  before do
    stub_user(id: user_id,
      permissions: %w[course.content.access.available],
      features:)
    Stub.service(:course, build(:'course:root'))

    Stub.request(:account, :get, "/users/#{user_id}")
      .and_return Stub.json({id: user_id})
    Stub.request(:account, :get, "/users/#{user_id}/preferences")
      .and_return Stub.json({properties: {}})

    Stub.request(:course, :get, '/courses/the_course')
      .to_return Stub.json(course)
    Stub.request(
      :course, :get, '/enrollments',
      query: {course_id: course['id'], user_id:}
    ).to_return Stub.json([{}])
    Stub.request(
      :course, :get, '/items',
      query: hash_including(section_id: section['id'])
    ).to_return Stub.json([item])
    Stub.request(
      :course, :get, "/items/#{item['id']}",
      query: {user_id:}
    ).to_return Stub.json(item)
    Stub.request(:course, :get, "/sections/#{section['id']}")
      .to_return Stub.json(section)
    Stub.request(
      :course, :get, '/sections',
      query: {course_id: course['id']}
    ).to_return Stub.json([section])
    Stub.request(
      :course, :get, '/next_dates',
      query: hash_including({})
    ).to_return Stub.json([])
    Stub.request(
      :course, :post, "/items/#{item['id']}/users/#{user_id}/visit",
      body: hash_including({})
    ).to_return Stub.response(status: 201)
  end

  context '(video)' do
    before do
      Stub.service(:pinboard, build(:'pinboard:root'))

      Stub.request(
        :pinboard, :get, '/topics',
        query: {item_id: item['id']}
      ).to_return Stub.json([])
    end

    context 'without time effort being enabled' do
      it 'does not add the time effort information' do
        visit "/courses/the_course/sections/#{section['id']}/items/#{item['id']}"

        expect(page).to have_no_content 'Time effort: approx. 3 minutes'
        expect(page.find('li.video > a')['data-tooltip']).not_to include '"item-info":"(Video, \u0026sim;3 minutes)"'
      end
    end

    context 'time effort enabled for all items' do
      let(:features) { super().merge('time_effort' => 'true') }

      it 'adds the time effort information to the item header and tooltip' do
        visit "/courses/the_course/sections/#{section['id']}/items/#{item['id']}"

        expect(page).to have_content 'Time effort: approx. 3 minutes'
        expect(page.find('li.video > a')['data-tooltip']).to include '"item-info":"(Video, \u0026sim;3 minutes)"'
      end

      context 'but value set to 0 (i.e., manually hidden)' do
        let(:item_params) do
          {**super(), time_effort: 0}
        end

        it 'does not add the time effort information' do
          visit "/courses/the_course/sections/#{section['id']}/items/#{item['id']}"

          expect(page).to have_no_content 'Time effort: approx. 3 minutes'
          expect(page.find('li.video > a')['data-tooltip']).not_to include '"item-info":"(Video, \u0026sim;3 minutes)"'
        end
      end
    end

    context 'time effort enabled for videos only' do
      let(:features) { super().merge('time_effort.video_only' => 'true') }

      it 'adds the time effort information to the item header and tooltip' do
        visit "/courses/the_course/sections/#{section['id']}/items/#{item['id']}"

        expect(page).to have_content 'Time effort: approx. 3 minutes'
        expect(page.find("[aria-label='Awesome video (Video)']")['data-tooltip']).to include '"item-info":"(Video, \u0026sim;3 minutes)"'
      end
    end
  end

  context '(quiz)' do
    let(:quiz) { build(:'quiz:quiz', :exam) }
    let(:item) { build(:'course:item', :quiz, :exam, item_params) }
    let(:item_params) do
      {
        **super(),
        content_id: quiz['id'],
        title: 'Awesome exam',
        time_effort: 200,
      }
    end

    before do
      Stub.service(:quiz, build(:'quiz:root'))
      Stub.request(
        :quiz, :get, "/quizzes/#{quiz['id']}",
        query: hash_including({})
      ).to_return Stub.json(quiz)
      Stub.request(
        :quiz, :get, '/questions',
        query: hash_including(quiz_id: quiz['id'])
      ).to_return Stub.json([])
      Stub.request(
        :quiz, :get, '/quiz_submissions',
        query: hash_including(quiz_id: quiz['id'], user_id:)
      ).to_return Stub.json([])
      Stub.request(
        :quiz, :get, '/user_quiz_attempts',
        query: hash_including(quiz_id: quiz['id'], user_id:)
      ).to_return Stub.json({attempts: 0, additional_attempts: 0})
    end

    context 'time effort enabled for all items' do
      let(:features) { super().merge('time_effort' => 'true') }

      it 'adds the time effort information to the item header and tooltip' do
        visit "/courses/the_course/sections/#{section['id']}/items/#{item['id']}"

        expect(page).to have_content 'Time effort: approx. 4 minutes'
        expect(page.find('li.quiz > a')['data-tooltip']).to include '"item-info":"(Graded Test, \u0026sim;4 minutes)"'
      end
    end

    context 'time effort enabled for videos only' do
      let(:features) { super().merge('time_effort.video_only' => 'true') }

      it 'does not add the time effort information' do
        visit "/courses/the_course/sections/#{section['id']}/items/#{item['id']}"

        expect(page).to have_no_content 'Time effort: approx. 4 minutes'
        expect(page.find('li.quiz > a')['data-tooltip']).not_to include '"item-info":"(Graded Test, \u0026sim;4 minutes)"'
      end
    end
  end
end
