# frozen_string_literal: true

require 'spec_helper'

describe ApplicationHelper, type: :helper do
  describe 'webcomponents registry' do
    let(:es6_browser) { BrowserSupport.new Browser.new 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.149 Safari/537.36' }
    let(:es5_browser) { BrowserSupport.new Browser.new 'Mozilla/5.0 (Windows NT 10.0; Trident/7.0; rv:11.0) like Gecko' }

    context 'existing entries' do
      before do
        assign(:webcomponents_registry, Set.new(%w[m.e.i.n.e.l]))
      end

      it 'returns registered webcomponents' do
        expect(helper.webcomponents_registry).to match_array(%w[m.e.i.n.e.l])
      end

      it 'returns true if webcomponents are used' do
        expect(helper.webcomponents?).to be true
      end

      it 'recognizes webcomponents used' do
        expect(helper.webcomponents?('m.e.i.n.e.l')).to be true
        expect(helper.webcomponents?('paper-button')).to be false
      end

      it 'creates valid ES5 paths with webcomponents-bundle and es5-adapter' do
        allow(helper).to receive(:browser_support).and_return(es5_browser)
        expect(helper.webcomponents_assets).to match(%r{<script src=".*/webcomponents/custom-elements-es5-adapter.*\.js"></script><script src=".*/webcomponents/webcomponents-bundle.*\.js"></script><script src=".*/m\.e\.i\.n\.e\.l/es5/m\.e\.i\.n\.e\.l.*\.js"></script>})
      end

      it 'creates valid ES6 paths with webcomponents-bundle' do
        allow(helper).to receive(:browser_support).and_return(es6_browser)
        expect(helper.webcomponents_assets).to match(%r{<script src=".*/webcomponents/webcomponents-bundle.*\.js"></script><script src=".*/m\.e\.i\.n\.e\.l/es6/m\.e\.i\.n\.e\.l.*\.js"></script>})
      end

      it 'allows new entries' do
        helper.register_webcomponent('paper-button')
        expect(helper.webcomponents_registry.size).to be 2
        expect(helper.webcomponents_registry).to match_array(%w[m.e.i.n.e.l paper-button])
      end

      it 'does not duplicate entries' do
        helper.register_webcomponent('m.e.i.n.e.l')
        expect(helper.webcomponents_registry.size).to be 1
        expect(helper.webcomponents_registry).to match_array(%w[m.e.i.n.e.l])
      end
    end

    context 'new entries' do
      it 'returns registered webcomponents' do
        expect(helper.webcomponents_registry).to match Set.new
      end

      it 'returns false if webcomponents are not used' do
        expect(helper.webcomponents?).to be false
      end

      it 'recognizes webcomponents used' do
        expect(helper.webcomponents?('m.e.i.n.e.l')).to be false
        expect(helper.webcomponents?('paper-button')).to be false
      end

      it 'does not create any ES5 paths except for the webcomponents-bundle' do
        allow(helper).to receive(:browser_support).and_return(es5_browser)
        expect(helper.webcomponents_assets).to match(%r{<script src=".*/webcomponents-bundle.*\.js"></script>})
      end

      it 'does not create any ES6 paths except for the webcomponents-bundle' do
        allow(helper).to receive(:browser_support).and_return(es6_browser)
        expect(helper.webcomponents_assets).to match(%r{<script src=".*/webcomponents-bundle.*\.js"></script>})
      end

      it 'allows new entries' do
        helper.register_webcomponent('m.e.i.n.e.l')
        expect(helper.webcomponents_registry).to contain_exactly('m.e.i.n.e.l')
        expect(helper.webcomponents_registry.size).to be 1
      end

      it 'does not duplicate entries' do
        helper.register_webcomponent('m.e.i.n.e.l')
        helper.register_webcomponent('m.e.i.n.e.l')
        expect(helper.webcomponents_registry).to contain_exactly('m.e.i.n.e.l')
        expect(helper.webcomponents_registry.size).to be 1
      end
    end
  end
end
