# frozen_string_literal: true

require 'spec_helper'

describe Video::VimeoAdapter do
  describe '#sync' do
    subject(:sync_provider) { provider.sync }

    let(:provider) { create(:video_provider, :vimeo) }

    let(:videos_url) do
      uri = Addressable::Template.new \
        'https://api.vimeo.com/me/videos{?fields,page,sort}'

      uri = uri.partial_expand \
        sort: 'modified_time',
        fields: %w[
          uri
          name
          duration
          width
          height
          status
          modified_time
          pictures.sizes.link
          pictures.sizes.width
          files.quality
          files.size
          files.md5
          files.link
          download
        ]

      ->(**kwargs) { uri.expand(kwargs) }
    end

    let!(:page1) do
      stub_request(:get, videos_url.call)
        .with(headers: {
          'Authorization' => "Bearer #{provider.credentials['token']}",
          'Accept' => 'application/vnd.vimeo.*+json;version=3.4',
        })
        .to_return(body: page1_response)
    end

    let(:page1_response) do
      File.read 'spec/support/files/video/vimeo/videos/1.json'
    end

    let!(:page2) do
      stub_request(:get, videos_url.call(page: 2))
        .with(headers: {
          'Authorization' => "Bearer #{provider.credentials['token']}",
          'Accept' => 'application/vnd.vimeo.*+json;version=3.4',
          'Accept-Encoding' => 'gzip;q=1.0,deflate;q=0.6,identity;q=0.3',
        })
        .to_return(body: File.read('spec/support/files/video/vimeo/videos/2.json'))
    end

    let!(:page3) do
      stub_request(:get, videos_url.call(page: 3))
        .with(headers: {
          'Authorization' => "Bearer #{provider.credentials['token']}",
          'Accept' => 'application/vnd.vimeo.*+json;version=3.4',
        })
        .to_return(body: File.read('spec/support/files/video/vimeo/videos/3.json'))
    end

    it 'calls the Vimeo API' do
      sync_provider
      expect(page1).to have_been_requested.once
      expect(page2).to have_been_requested.once
      expect(page3).to have_been_requested.once
    end

    it 'chooses the larger one out of two SD streams in a Vimeo video resource' do
      sync_provider
      expect(Video::Stream.find_by(provider_video_id: '124597457').sd_size).to eq 250
    end

    it 'chooses the largest download URL both for SD and HD in a Vimeo video resource' do
      sync_provider

      stream = Video::Stream.find_by(provider_video_id: '124597457')
      expect(stream.sd_download_url).to eq 'https://download.vimeo.com/larger.sd.mp4'
      expect(stream.hd_download_url).to eq 'https://download.vimeo.com/larger.hd.mp4'
    end

    context 'without any data' do
      let(:page1_response) do
        File.read 'spec/support/files/video/vimeo/videos/no_data.json'
      end

      it 'raises' do
        expect { sync_provider }.to raise_error KeyError
      end
    end

    context 'without downloads in response' do
      let(:page1_response) do
        File.read 'spec/support/files/video/vimeo/videos/no_downloads.json'
      end

      it 'creates a stream' do
        expect { sync_provider }.to change(Video::Stream, :count).from(0).to(1)
      end
    end

    context 'without an expiration date for downloads' do
      let(:page1_response) do
        File.read 'spec/support/files/video/vimeo/videos/no_expiration.json'
      end

      it 'creates a stream' do
        expect { sync_provider }.to change(Video::Stream, :count).from(0).to(1)
      end
    end

    context 'with a synchronized_at date' do
      let(:sync_date) do
        # A date between before last item on page 2
        Time.iso8601('2017-10-15T16:00:00+02:00')
      end

      before { provider.update! synchronized_at: sync_date }

      it 'stops iteration before page 3' do
        sync_provider
        expect(page1).to have_been_requested.once
        expect(page2).to have_been_requested.once
        expect(page3).not_to have_been_requested
      end

      it 'updates synchronized_at' do
        Timecop.freeze do
          # Force microsecond precision instead of nanoseconds
          time = Time.iso8601 Time.zone.now.iso8601(6)

          expect do
            sync_provider
          end.to change {
            provider.reload.synchronized_at
          }.from(sync_date).to(time)
        end
      end
    end
  end
end
