# frozen_string_literal: true

require 'spec_helper'

describe 'Course: Items: Update', type: :request do
  subject(:action) do
    put("/courses/#{course['course_code']}/sections/#{section['id']}/items/#{item['id']}",
      params:,
      headers:)
    response
  end

  let(:course) { build(:'course:course', course_code: 'example') }
  let(:section) { build(:'course:section', course_id: course['id']) }
  let(:params) { {} }
  let(:headers) { {} }

  before do
    Stub.service(:course, build(:'course:root'))
    Stub.request(:course, :get, "/courses/#{course['course_code']}")
      .to_return Stub.json(course)
  end

  context 'for anonymous user' do
    let(:item) { build(:'course:item') }

    it 'redirects to login page' do
      expect(action).to be_redirect
      expect(action).to redirect_to 'https://xikolo.de/sessions/new'
    end
  end

  context 'for a logged in user' do
    let(:user_id) { generate(:user_id) }
    let(:headers) { {'Authorization' => "Xikolo-Session session_id=#{stub_session_id}"} }
    let(:permissions) { %w[course.content.access course.content.edit] }
    let(:update_stub) { Stub.request(:course, :put, "/items/#{item['id']}").to_return Stub.json(item) }

    before do
      stub_user_request(id: user_id, permissions:)
      Stub.request(
        :course, :get, "/items/#{item['id']}"
      ).to_return Stub.json(item)
      Stub.request(
        :course, :get, '/enrollments',
        query: {course_id: course['id'], user_id:}
      ).to_return Stub.json([])
      Stub.request(
        :course, :get, '/next_dates',
        query: hash_including(course_id: course['id'])
      ).to_return Stub.json([])
      Stub.request(
        :course, :get, '/items',
        query: hash_including(section_id: section['id'])
      ).to_return Stub.json([])
      Stub.request(
        :course, :get, '/sections',
        query: {course_id: course['id']}
      ).to_return Stub.json([section])
      Stub.request(
        :course, :get, "/sections/#{section['id']}"
      ).to_return Stub.json([section])
      update_stub
    end

    context 'for LTI items' do
      let(:exercise) { create(:lti_exercise) }
      let(:item) { build(:'course:item', :lti, content_id: exercise.id) }
      let(:params) do
        {
          xikolo_course_item: {
            title: item['title'],
            content_type: item['content_type'],
          },
          lti_exercise: exercise_params,
        }
      end

      context 'when the item content is invalid' do
        let(:exercise_params) do
          {
            instructions: 'These are the LTI instructions.',
            lti_provider_id: nil,
          }
        end

        it 'shows validation message' do
          expect(action.body).to include('can&#39;t be blank')
        end
      end

      context 'when the item content is valid' do
        let(:exercise_params) do
          {
            instructions: 'These are the LTI instructions.',
            lti_provider_id: exercise.lti_provider_id,
          }
        end

        it 'redirects to the edit page' do
          expect(action).to redirect_to edit_course_section_item_path
        end
      end
    end

    context 'for video items' do
      let(:item) { build(:'course:item', :video, content_id: video.id, open_mode: false) }
      let(:video) { create(:video) }
      let!(:stream) { create(:stream) }
      let(:video_params) do
        {video: {
          lecturer_stream_id: stream.id,
        }}
      end
      let(:params) do
        {
          xikolo_course_item: {
            title: item['title'],
            content_type: item['content_type'],
          },
        }.merge(video_params)
      end

      before do
        allow_any_instance_of(Video::Video).to receive(:id).and_return video.id # rubocop:disable RSpec/AnyInstance
      end

      shared_examples 'displays generic and specific error messages' do |specific_error|
        it 'displays a generic error message at the top of the page' do
          action
          expect(flash[:error]).to include 'Something went wrong while updating this item.'
        end

        it 'displays a specific error message near the related field' do
          action
          expect(response.body).to include 'We found some errors. Please review your form input.'
          expect(response.body).to have_css('.has-error', text: specific_error)
        end
      end

      it 'updates the video item' do
        action
        expect(Video::Video.first).to match an_object_having_attributes(
          lecturer_stream_id: params[:video][:lecturer_stream_id]
        )
      end

      context 'with invalid video params (i.e. no streams)' do
        let(:video_params) do
          {video: {
            description: 'This is the video description.',
            pip_stream_id: nil,
          }}
        end

        # Description: Your file upload could not be stored.
        specific_error = 'At least one of these streams is required. Please add a pip, a lecturer or a slides stream.'
        include_examples 'displays generic and specific error messages', specific_error

        it 'does not update the video item' do
          action
          expect(Video::Video.first.reload).to eq video
          expect(Video::Video.first.description.to_s).to eq 'Video for testing.'
        end
      end

      context 'with an invalid item' do
        let(:update_stub) do
          Stub.request(:course, :put, "/items/#{item['id']}")
            .to_return Stub.json({errors: {title: "can't be blank"}}, status: 422)
        end

        it 'does not update the video item' do
          action
          expect(Video::Video.first.reload).to eq video
          expect(Video::Video.first.description.to_s).to eq 'Video for testing.'
        end

        it 'displays an error message' do
          action
          expect(flash[:error]).to include 'The item title must not be blank.'
        end
      end

      context '(updating description)' do
        let(:video_id) { SecureRandom.uuid }
        let(:video) { create(:video, id: video_id, description: old_description) }
        let(:old_file_uri) { "s3://xikolo-video/videos/#{video_id}/rtfiles/b5f99337-224f-40f5-aa82-44ee8b272579/foo.jpg" }
        let(:old_file_url) { "https://s3.xikolo.de/xikolo-video/videos/#{video_id}/rtfiles/b5f99337-224f-40f5-aa82-44ee8b272579/foo.jpg" }
        let(:old_description) { "Some Text with a file: #{old_file_uri}" }
        let(:video_params) do
          {video: {
            description: 'upload://b5f99337-224f-40f5-aa82-44ee8b272579/foo_bar.jpg',
          }}
        end
        let(:store_regex) { %r{https://s3.xikolo.de/xikolo-video/videos/[0-9a-zA-Z]+/rtfiles/[0-9a-zA-Z]+/foo_bar.jpg}x }
        let!(:store_stub) do
          stub_request(:put, store_regex).and_return(status: 200, body: '<xml></xml>')
        end

        before do
          stub_request(:head, store_regex).and_return(status: 404)
        end

        context 'with a valid description with valid files' do
          before do
            stub_request(
              :head,
              'https://s3.xikolo.de/xikolo-uploads/' \
              'uploads/b5f99337-224f-40f5-aa82-44ee8b272579/foo_bar.jpg'
            ).and_return(
              status: 200,
              headers: {
                'X-Amz-Meta-Xikolo-Purpose' => 'video_description',
                'X-Amz-Meta-Xikolo-State' => 'accepted',
              }
            )
          end

          it 'stores the file in the description in S3' do
            action
            expect(store_stub).to have_been_requested
          end

          it 'updates the description on the video' do
            action
            expect(Video::Video.first[:description].to_s).to start_with("s3://xikolo-video/videos/#{UUID4(video_id).to_str(format: :base62)}/rtfiles")
              .and end_with 'foo_bar.jpg'
          end

          it 'deletes the old file' do
            expect { action }.to have_enqueued_job(S3FileDeletionJob).with(old_file_uri)
          end
        end

        context 'with an invalid file in the description' do
          before do
            stub_request(
              :head,
              'https://s3.xikolo.de/xikolo-uploads/' \
              'uploads/b5f99337-224f-40f5-aa82-44ee8b272579/foo_bar.jpg'
            ).and_return(
              status: 200,
              headers: {
                'X-Amz-Meta-Xikolo-Purpose' => 'video_description',
                'X-Amz-Meta-Xikolo-State' => 'rejected',
              }
            )
          end

          specific_error = 'Your file upload has been rejected due to policy violations.'
          include_examples 'displays generic and specific error messages', specific_error

          it 'does not update the description' do
            action
            expect(Video::Video.first[:description].to_s).to eq old_description
          end

          it 'does not delete the file in the old description' do
            expect { action }.not_to have_enqueued_job(S3FileDeletionJob).with(old_file_uri)
          end
        end

        context 'with a storage error' do
          let(:store_stub) do
            stub_request(:put, store_regex).and_return(status: 503)
          end

          before do
            store_stub
            stub_request(
              :head,
              'https://s3.xikolo.de/xikolo-uploads/' \
              'uploads/b5f99337-224f-40f5-aa82-44ee8b272579/foo_bar.jpg'
            ).and_return(
              status: 200,
              headers: {
                'X-Amz-Meta-Xikolo-Purpose' => 'video_description',
                'X-Amz-Meta-Xikolo-State' => 'accepted',
              }
            )
          end

          specific_error = 'Your file upload could not be stored.'
          include_examples 'displays generic and specific error messages', specific_error

          it 'does not update the description' do
            action
            expect(Video::Video.first[:description].to_s).to eq old_description
          end

          it 'does not delete the file in the old description' do
            expect { action }.not_to have_enqueued_job(S3FileDeletionJob).with(old_file_uri)
          end
        end
      end

      # Invalid subtitles and additional files are covered in operation/video/store_spec
    end
  end
end
