# frozen_string_literal: true

require 'spec_helper'

describe 'Portal API: Create enrollment', type: :request do
  subject(:request) { post '/portalapi-beta/enrollments', params:, headers: }

  let(:headers) { {} }
  let(:params) { {} }
  let(:json) { JSON.parse response.body }
  let(:enrollment) { build(:'course:enrollment') }

  before do
    Stub.service(:course, build(:'course:root'))
  end

  context 'without Authorization header' do
    it 'responds with 401 Unauthorized' do
      request
      expect(response).to have_http_status :unauthorized
      expect(response.headers['WWW-Authenticate']).to eq 'Bearer realm="test-realm"'
      expect(response.headers['Content-Type']).to eq 'application/problem+json; charset=utf-8'
      expect(json).to eq(
        'type' => 'https://dev.xikolo.de/api-docs/portalapi-beta/#error-unauthenticated',
        'title' => 'You must provide an Authorization header to access this resource.',
        'status' => 401
      )
    end

    it 'answers in context of the configured realm' do
      request
      expect(response.header['WWW-Authenticate']).to include('realm="test-realm"')
    end
  end

  context 'when trying to authorize with an invalid token' do
    let(:headers) { super().merge('Authorization' => 'Bearer canihackyou') }

    it 'responds with 401 Unauthorized' do
      request
      expect(response).to have_http_status :unauthorized
      expect(response.headers['WWW-Authenticate']).to eq 'Bearer realm="test-realm", error="invalid_token"'
      expect(response.headers['Content-Type']).to eq 'application/problem+json; charset=utf-8'
      expect(json).to eq(
        'type' => 'https://dev.xikolo.de/api-docs/portalapi-beta/#error-invalid_token',
        'title' => 'The bearer token you provided was invalid, has expired or has been revoked.',
        'status' => 401
      )
    end
  end

  context 'when authorized (with a hardcoded token)' do
    let(:headers) { super().merge('Authorization' => 'Bearer 78f6d8ca88c65a67c9dffa3c232313d64b4e338e29d7c83ef39c2e963894b966') }

    context 'without Accept header' do
      let(:headers) { super().merge('Accept' => nil) }

      it 'responds with HTTP 406 Not Acceptable' do
        request
        expect(response).to have_http_status :not_acceptable
        expect(response.headers['Content-Type']).to eq 'application/problem+json; charset=utf-8'
        expect(json).to eq(
          'type' => 'https://dev.xikolo.de/api-docs/portalapi-beta/#error-accept_header_missing',
          'title' => 'You must provide the desired content type in the Accept request header.',
          'status' => 406
        )
      end
    end

    context 'Accept=application/vnd.openhpi.enrollment+json;v=0.9 (obsolete)' do
      let(:headers) { super().merge('Accept' => 'application/vnd.openhpi.enrollment+json;v=0.9') }

      it 'responds with HTTP 406 Not Acceptable' do
        request
        expect(response).to have_http_status :not_acceptable
        expect(response.headers['Content-Type']).to eq 'application/problem+json; charset=utf-8'
        expect(json).to eq(
          'type' => 'https://dev.xikolo.de/api-docs/portalapi-beta/#error-unsupported_content_type',
          'title' => 'The media type provided in the "Accept" request header is not supported by this endpoint.',
          'status' => 406
        )
      end
    end

    context 'Accept=application/vnd.openhpi.enrollment+json;v=1.0 (current version)' do
      let(:headers) { super().merge('Accept' => 'application/vnd.openhpi.enrollment+json;v=1.0') }

      context 'with missing user ID' do
        let(:course_id) { generate(:course_id) }
        let(:params) { super().merge(course_id:) }

        it 'responds with HTTP 422' do
          request
          expect(response).to have_http_status :unprocessable_entity
          expect(response.headers['Content-Type']).to eq 'application/problem+json; charset=utf-8'
          expect(json).to eq(
            'type' => 'https://dev.xikolo.de/api-docs/portalapi-beta/#error-parameter_missing',
            'title' => 'The user_id and course_id cannot be blank.',
            'status' => 422
          )
        end
      end

      context 'with unknown user and course' do
        let(:unknown_user_id) { SecureRandom.uuid }
        let(:unknown_course_id) { SecureRandom.uuid }
        let(:params) { super().merge(user_id: unknown_user_id, course_id: unknown_course_id) }

        before do
          Stub.request(
            :course, :get, '/enrollments',
            query: {
              user_id: unknown_user_id,
              course_id: unknown_course_id,
              learning_evaluation: true,
            }
          ).to_return Stub.json([])

          Stub.request(
            :course, :post, '/enrollments',
            body: {
              user_id: unknown_user_id,
              course_id: unknown_course_id,
            }
          ).to_return status: 404
        end

        it 'responds with an empty result' do
          request
          expect(response).to have_http_status :not_found
          expect(response.headers['Content-Type']).to include 'application/problem+json'
          expect(json).to eq(
            'type' => 'https://dev.xikolo.de/api-docs/portalapi-beta/#error-course_or_user_not_found',
            'title' => 'Course or user not found.',
            'status' => 404
          )
        end
      end

      context 'with valid parameters' do
        let(:params) { super().merge(user_id: enrollment['user_id'], course_id: enrollment['course_id']) }
        let(:enrollments_response) { [] }

        let!(:create_enrollment_stub) do
          Stub.request(
            :course, :post, '/enrollments',
            body: {
              user_id: enrollment['user_id'],
              course_id: enrollment['course_id'],
            }
          ).to_return Stub.json(enrollment)
        end

        before do
          Stub.request(
            :course, :get, '/enrollments',
            query: {
              user_id: enrollment['user_id'],
              course_id: enrollment['course_id'],
              learning_evaluation: true,
            }
          ).to_return Stub.json(enrollments_response)
        end

        it 'creates and returns the enrollment' do
          request
          expect(create_enrollment_stub).to have_been_requested
          expect(response).to have_http_status :created
          expect(response.headers['Content-Type']).to eq 'application/vnd.openhpi.enrollment+json;v=1.0'
          expect(JSON.parse(response.body)).to eq enrollment
        end

        context 'when the enrollment already exists' do
          let(:enrollments_response) { [enrollment] }

          it 'responds with HTTP 409' do
            request
            expect(response).to have_http_status :conflict
            expect(response.headers['Content-Type']).to eq 'application/problem+json; charset=utf-8'
            expect(json).to eq(
              'type' => 'https://dev.xikolo.de/api-docs/portalapi-beta/#error-enrollment_already_present',
              'title' => 'An enrollment for this user and course already exists.',
              'status' => 409
            )
          end
        end
      end
    end
  end
end
